<?php
/*
*
* @name cron.php
* @package cBB Blog
* @version $Id: cron.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
*/
define('IN_PHPBB', true);
define('IN_BLOG', true);
define('IN_BLOG_CRON', true);
define('IMAGE_OUTPUT', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : '../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Do not update users last page entry
$user->session_begin(false);
$auth->acl($user->data);

$use_shutdown_function = (@function_exists('register_shutdown_function')) ? true : false;
$time_now = (!empty($user->time_now) && is_int($user->time_now)) ? $user->time_now : time();

// Output transparent gif
header('Cache-Control: no-cache');
header('Content-type: image/gif');
header('Content-length: 43');

echo base64_decode('R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==');

if(!isset($config['blog_cron_lock']))
{
	set_config('blog_cron_lock', '0', true);
}

// make sure cron doesn't run multiple times in parallel
if($config['blog_cron_lock'])
{
	$time = explode(' ', $config['blog_cron_lock']);
	$time = $time[0];

	if(($time_now - 3600) < $time)
	{
		exit;
	}
}

define('CRON_ID', time() . ' ' . unique_id());

$sql = 'UPDATE ' . CONFIG_TABLE . "
	SET config_value = '" . $db->sql_escape(CRON_ID) . "'
	WHERE config_name = 'blog_cron_lock' AND config_value = '" . $db->sql_escape($config['blog_cron_lock']) . "'";
$db->sql_query($sql);

// another cron process altered the table between script start and UPDATE query so exit
if($db->sql_affectedrows() != 1)
{
	exit;
}

$rss_sources = array();

// Find and import rss
$sql = 'SELECT *
	FROM ' . BLOG_SOURCES_TABLE . "
	WHERE $time_now - source_gc > source_last_gc";
$result = $db->sql_query($sql);
while($row = $db->sql_fetchrow($result))
{
	$rss_sources[$row['source_id']] = $row;
}
$db->sql_freeresult($result);

if(sizeof($rss_sources))
{
	include($blog_hook->root_path . 'includes/functions_rss.' . $phpEx);
	$blog_rss = new blog_rss();
	
	$entry_ary = array();
	
	// Get last post time
	$sql = 'SELECT MAX(post_time) AS max_post_time
		FROM ' . BLOG_POSTS_TABLE;
	$result = $db->sql_query($sql);
	$max_post_time = (int)$db->sql_fetchfield('max_post_time');
	$db->sql_freeresult($result);

	foreach($rss_sources as $source_id => $source)
	{
		if(!$blog_rss->read($source['source_url']))
		{
			add_log('critical', 'LOG_BLOG_SOURCE_ERROR', $source['source_url']);
			unset($rss_sources[$source_id]);
			continue;
		}
		
		if($blog_rss->get_build_time() <= $max_post_time)
		{
			unset($rss_sources[$source_id]);
			continue;
		}

		switch($blog_rss->rss_type)
		{
			case 'rss':
				$rss_entries = $blog_rss->get('item');
				
				foreach($rss_entries as $row)
				{
					$row_time = $blog_rss->get_build_time($row['pubDate']);
					
					if($row_time <= $max_post_time)
					{
						continue;
					}
					
					$entry_ary[$row_time] = array(
						'post_title'	=> $row['title'],
						'poster_id'		=> $source['source_poster'],
						'post_text'		=> $row['description'],
						
						'cats'			=> ($source['source_cats']) ? explode('|', $source['source_cats']) : false,
					);
				}
			break;
			
			case 'feed':
				$rss_entries = $this->get('entry');

				foreach($rss_entries as $row)
				{
					$row_time = $blog_rss->get_build_time($row['updated']);
					
					if($row_time <= $max_post_time)
					{
						continue;
					}
					
					$entry_ary[$row_time] = array(
						'post_title'	=> $row['title'],
						'poster_id'		=> $source['source_poster'],
						'post_text'		=> $row['content'],
						
						'cats'			=> ($source['source_cats']) ? explode('|', $source['source_cats']) : false,
					);
				}
			break;	
		}
	}
	
	if(sizeof($entry_ary))
	{
		if(!function_exists('blog_submit_post'))
		{
			include($blog_hook->root_path . 'includes/functions_posting.' . $phpEx);
		}
		
		ksort($entry_ary);
		
		foreach($entry_ary as $row_time => $row_data)
		{
			$row_data += array(
				'post_id'			=> 0,
				'archive_id'		=> 0,
				'post_approved'		=> 1,
				'post_username'		=> '',
				'post_time'			=> $row_time,
				'post_type'			=> POST_TYPE_HTML,
				'bbcode_uid'		=> '',
				'bbcode_bitfield'	=> '',
			);
			
			$row_cats = $row_data['cats'];
			unset($row_data['cats']);
			
			blog_submit_post('post', $row_data, $row_cats);
		}
	}
	
	if(sizeof($rss_sources))
	{
		$sql = 'UPDATE ' . BLOG_SOURCES_TABLE . "
			SET source_last_gc = $time_now
			WHERE " . $db->sql_in_set('source_id', array_keys($rss_sources));
		$db->sql_query($sql);
	}
}

// Unloading cache and closing db after having done the dirty work.
if($use_shutdown_function)
{
	register_shutdown_function('garbage_collection');
}
else
{
	garbage_collection();
}

exit;

?>