<?php
/*
*
* @name functions.php
* @package phpBB3 cBB Blog
* @version $Id: functions.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

class blog
{
	static public function append_post_categories(&$postrow, $post_id = 0)
	{
		global $db;

		if(!empty($postrow))
		{
			$cat_rows = self::get_category('ids');
			$post_ids = ($post_id) ? array($post_id) : array_keys($postrow);

			$sql = 'SELECT *
				FROM ' . BLOG_POSTS_TRACK_TABLE . '
				WHERE ' . $db->sql_in_set('post_id', $post_ids);
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				$row_post_id 	= (int)$row['post_id'];
				$cat_row		= $cat_rows[$row['cat_id']];
				$clean_name		= strtolower($cat_row['cat_name']);

				if($post_id)
				{
					if(!isset($postrow['cats']))
					{
						$postrow['cats'] = array();
					}
					
					if(isset($postrow['cats'][$clean_name]))
					{
						$clean_name .= ' - ' . time();
					}

					$postrow['cats'][$clean_name] = $cat_row;
				}
				else
				{
					if(!isset($postrow[$row_post_id]['cats']))
					{
						$postrow[$row_post_id]['cats'] = array();
					}
					
					if(isset($postrow[$row_post_id]['cats'][$clean_name]))
					{
						$clean_name .= ' - ' . time();
					}

					$postrow[$row_post_id]['cats'][$clean_name] = $cat_row;
				}
			}
			$db->sql_freeresult($result);
		}
	}

	static public function append_sid($filename, $row = false, $params = false, $use_cat = false, $force_full_url = false)
	{
		global $blog_hook, $phpbb_root_path, $phpEx;
		
		$params = (is_array($params) ? $params : array());
		
		if($row !== false)
		{
			if(is_numeric($use_cat))
			{
				$params['c'] = $use_cat;
			}
			else if($use_cat === true && ($cat_id = self::get_row_cat_id($row)))
			{
				$params['c'] = $cat_id;
			}

			$params['p'] = $row['post_id'];
		}

		if($use_cat !== false && $use_cat !== true && !isset($params['c']))
		{
			$params['c'] = $use_cat;
		}
		
		// Clear url
		foreach($params as $name => $value)
		{
			if(!$value)
			{
				unset($params[$name]);
			}
		}
		
		$web_path = $blog_hook->root_path;
		
		if($blog_hook->domain)
		{
			$web_path = $blog_hook->domain;
		}
		else if((defined('PHPBB_USE_BOARD_URL_PATH') && PHPBB_USE_BOARD_URL_PATH) || $force_full_url)
		{
			$web_path = generate_board_url() . '/' . str_replace($phpbb_root_path, '', $blog_hook->root_path);
		}
		
		$url = $web_path . (($filename == 'index') ? '' : $filename . ".$phpEx");
		
		if(!empty($blog_hook->seo))
		{
			$url = $web_path . $filename . ".$phpEx";
			$blog_hook->seo->rewrite($url, $params);
		}

		return append_sid($url, (!empty($params) ? $params : false));
	}
	
	static public function auth($auth_option, $data = false)
	{
		global $user, $auth;
		
		// Disallow post to guests
		if($auth_option == 'post' && $user->data['user_id'] == ANONYMOUS)
		{
			return false;
		}
	
		if($data === false)
		{
			if($auth->acl_get("u_blog_$auth_option") || $auth->acl_get("m_blog_$auth_option"))
			{
				return true;
			}
			
			return false;
		}
		else
		{
			// Check comment auth
			if(!empty($data['user_id']) && $auth->acl_get("u_blog_$auth_option") && $data['user_id'] == $user->data['user_id'])
			{
				return true;
			}
			
			// Check post auth
			if(!empty($data['poster_id']) && $auth->acl_get("u_blog_$auth_option") && $data['poster_id'] == $user->data['user_id'])
			{
				return true;
			}
			
			if($auth->acl_get("m_blog_$auth_option"))
			{
				return true;
			}
		}
		
		return false;
	}
	
	static public function build_category_tree($cat_ids = 0)
	{
		global $template;

		$parents 	= self::get_category('parents');
		$rows		= self::get_category('ids');
		
		if(!is_array($cat_ids))
		{
			$cat_ids = ($cat_ids === false) ? array() : array($cat_ids);
		}
		
		foreach($parents[0] as $row_id)
		{
			if(!isset($rows[$row_id]))
			{
				continue;
			}
			
			if(!blog::category_auth($rows[$row_id]))
			{
				continue;
			}

			$template->assign_block_vars('catrow', array(
				'S_ID'			=> $row_id,
				'S_TITLE'		=> $rows[$row_id]['cat_name'],
				'S_DESC'		=> $rows[$row_id]['cat_description'],
				'S_CHECKED'		=> (in_array($row_id, $cat_ids) ? ' checked="checked"' : ''),
				'S_SELECTED'	=> (in_array($row_id, $cat_ids) ? ' selected="selected"' : ''),
				
				'U_ACTION'	=> blog::append_sid('index', false, array('c' => $row_id)),
			));
			
			$child_ary = array();
			
			if(!empty($parents[$row_id]))
			{
				foreach($parents[$row_id] as $subrow_id)
				{
					if(blog::category_auth($rows[$subrow_id]))
					{
						$child_ary[] = $subrow_id;
					}
				}
			}
			
			foreach($child_ary as $child_id)
			{
				$template->assign_block_vars('catrow.child', array(
					'S_ID'			=> $child_id,
					'S_TITLE'		=> $rows[$child_id]['cat_name'],
					'S_DESC'		=> $rows[$child_id]['cat_description'],
					'S_CHECKED'		=> (in_array($child_id, $cat_ids) ? ' checked="checked"' : ''),
					'S_SELECTED'	=> (in_array($child_id, $cat_ids) ? ' selected="selected"' : ''),
					
					'U_ACTION'	=> blog::append_sid('index', false, array('c' => $child_id)),
				));
			}
		}
	}
	
	static public function build_navbar($data = false, $custom = false)
	{
		global $user, $template;

		$ary = array(
			'BLOG'		=> array('index'),
		);
		
		if($data === false && is_array($custom))
		{
			$ary = array_merge($ary, $custom);
		}
		
		if(!empty($data['archive_id']))
		{
			$ary[$data['archive_data']] = array('index', false, array('a' => $data['archive_id']));
		}
		else if(!empty($data['cat_id']))
		{
			$cat_data = blog::get_category($data['cat_id']);
			
			if($cat_data['parent_id'])
			{
				$parent_name = blog::get_category($cat_data['parent_id'], 'cat_name');
				$ary[$parent_name] = array('index', false, array('c' => $cat_data['parent_id']));
			}
			
			$ary[$cat_data['cat_name']] = array('index', false, array('c' => $data['cat_id']));
		}
		
		if(!empty($data['post_id']))
		{
			$ary[$data['post_title']] = array('view', false, array(
				'c' => (isset($data['cat_id']) ? $data['cat_id'] : 0),
				'p'	=> $data['post_id'],
			));
		}
		
		foreach($ary as $lang_key => $params)
		{
			$template->assign_block_vars('navlinks', array(
				'FORUM_NAME'	=> (isset($user->lang[$lang_key]) ? $user->lang[$lang_key] : $lang_key),
				'U_VIEW_FORUM'	=> call_user_func_array(array('self', 'append_sid'), $params),
			));
		}
	}
	
	static public function category_auth($data, $allow_disabled = false)
	{
		global $user;

		if(is_numeric($data))
		{
			$data = self::get_category($data);
		}
		
		$parent_auth = ($data['parent_id']) ? self::category_auth($data['parent_id']) : true;
		
		if($parent_auth && $data['cat_groups'])
		{
			$group_ary = explode(',', $data['cat_groups']);
			
			if(!$allow_disabled && !$data['cat_enabled'])
			{
				return false;
			}

			if($data['cat_groups'] == 'all' || in_array($user->data['group_id'], $group_ary))
			{
				return true;
			}
		}
		
		return false;
	}
	
	static public function generate_resume($row)
	{
		global $config, $user;
		
		$link 			= blog::append_sid('view', $row, false, true);
		$resume_text	= $row['post_text'];
		
		// Delete images, objects, attachments and some HTML entities
		$match = array(
			'#<img([^>]*)>#i',
			'#<object([^>]*)>(.*)</object>#i',
			'#\[blog-attach([^\]]+)*\](.*)\[/blog-attach\]#i',
			
			'#<blockquote>(.*?)</blockquote>#is',
			'#<code>(.*?)</code>#is',
			'#<p([^>]+)?>(.*?)</p>#is',
			'#<div([^>]+)?>(.*?)</div>#is',
			'#<br />#'
		);
		
		$replace = array('', '', '', '<em>\1</em>', '', '\2', '\2', '');
		
		// Find resume parts
		preg_match_all('#\[blog\-resume\](.*?)\[\/blog\-resume\]#i', $resume_text, $resume_parts);
		
		if(sizeof($resume_parts[1]))
		{
			$resume_text = implode(' ', $resume_parts[1]);
		}
		
		$resume_text = preg_replace($match, $replace, $resume_text);
		
		self::clear_resume_tags($resume_text);

		return html_truncate($resume_text, $config['blog_resume_chars'], '... ' . $user->lang('ENTRY_READ_MORE', $link));
	}
	
	static public function get_category($cat_id, $cat_value = '', $force = false)
	{
		global $db, $cache;
		static $static_blog_categories;
		
		if(empty($static_blog_categories) || $force)
		{
			if (($static_blog_categories = $cache->get('_blog_categories')) === false)
			{
				$static_blog_categories = array(
					'ids'		=> array(),
					'parents'	=> array()
				);

				$sql = 'SELECT *
					FROM ' . BLOG_CATS_TABLE . '
					ORDER BY parent_id ASC, LOWER(cat_name) ASC';
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$static_blog_categories['ids'][$row['cat_id']] = $row;
					
					if(!isset($static_blog_categories['parents'][$row['parent_id']]))
					{
						$static_blog_categories['parents'][$row['parent_id']] = array();
					}
					
					$static_blog_categories['parents'][$row['parent_id']][] = $row['cat_id'];
				}
				$db->sql_freeresult($result);

				$cache->put('_blog_categories', $static_blog_categories);
			}
		}
		
		if($cat_id === false)
		{
			return $static_blog_categories;
		}
		else if(isset($static_blog_categories[$cat_id]))
		{
			return $static_blog_categories[$cat_id];
		}
		
		if(isset($static_blog_categories['ids'][$cat_id]))
		{
			if($cat_value)
			{
				return (isset($static_blog_categories['ids'][$cat_id][$cat_value]) ? $static_blog_categories['ids'][$cat_id][$cat_value] : '');
			}
			
			return $static_blog_categories['ids'][$cat_id];
		}
		
		return false;
	}
	
	static public function get_row_cat_id($row)
	{
		if(empty($row['cats']))
		{
			return 0;
		}
		
		$cat_id = request_var('c', 0);
		
		if($cat_id)
		{
			foreach($row['cats'] as $null => $cat_row)
			{
				if($cat_id == $cat_row['cat_id'])
				{
					return $cat_id;
				}
			}
		}
		
		if(sizeof($row['cats']) == 1)
		{
			$keys = array_keys($row['cats']);
			return $row['cats'][$keys[0]]['cat_id'];
		}
		
		return 0;
	}
	
	static public function get_widgets()
	{
		global $db, $user, $cache, $template, $blog_hook, $phpEx;
		
		if (($widgets = $cache->get('_blog_widgets')) === false)
		{
			$widgets = array();

			$sql = 'SELECT *
				FROM ' . BLOG_WIDGETS_TABLE . '
				ORDER BY widget_order ASC';
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				$widgets[] = $row;
			}
			$db->sql_freeresult($result);

			$cache->put('_blog_widgets', $widgets);
		}

		foreach($widgets as $row)
		{
			$classname	= $row['widget_basename'] . '_widget';
			$filename	= $blog_hook->root_path . "widgets/$classname.$phpEx";
			
			if(file_exists($filename) && !class_exists($classname))
			{
				include($filename);
			}

			if(!class_exists($classname))
			{
				continue;
			}
			
			// Initialize and load the widget
			$widget = new $classname();
			
			if(!empty($row['widget_options']))
			{
				$widget->options = @unserialize($row['widget_options']);
			}

			if($widget->init() === false)
			{
				unset($widget);
				continue;
			}
			
			$template->set_filenames(array(
				'widget_tpl'	=> 'blog/widgets/' . $widget->tpl_name . '.html'
			));
			
			$template->assign_var('WIDGET_ID', $row['widget_id']);
			
			// Put block into general template to display
			$template->assign_block_vars('widget', array(
				'ID'			=> $row['widget_id'],
				'S_TITLE'		=> (isset($user->lang[$row['widget_title']]) ? $user->lang[$row['widget_title']] : $row['widget_title']),
				'S_CONTENT'		=> $template->assign_display('widget_tpl'),
				'S_STYLE'		=> $row['widget_style'],
			));
			
			unset($widget);
		}
	}
	
	static public function print_content($content)
	{
		global $template;
		
		$template->assign_var('CUSTOM_CONTENT', $content);
		
		$template->set_filenames(array(
			'body' => 'blog/empty_body.html'
		));
		
		$template->display('body');
	}

	static public function row_categories($row, $filename = 'index', $params = array())
	{
		$cat_str_ary = array();

		if(empty($row['cats']))
		{
			return false;
		}
		
		ksort($row['cats']);

		foreach($row['cats'] as $null => $cat_row)
		{
			if(!self::category_auth($cat_row))
			{
				continue;
			}
			
			$params['c'] = $cat_row['cat_id'];
			
			$cat_str_ary[] = '<a href="' . self::append_sid($filename, false, $params) . '">' . $cat_row['cat_name'] . '</a>';
		}
		
		return implode(', ', $cat_str_ary);
	}
	
	static public function strip_vars($url, $strip_vars = false)
	{
		if($strip_vars !== false && strpos($url, '?') !== false)
		{
			$is_amp = false;

			if(strpos($url, '&amp;') !== false)
			{
				$url = str_replace('&amp;', '&', $url);
				$is_amp = true;
			}
			
			// Use default vars
			if($strip_vars === true)
			{
				$strip_vars = array('sid', 'style');
			}

			if(!is_array($strip_vars))
			{
				$strip_vars = array($strip_vars);
			}

			$query = $_query = array();
			
			list($url, $args) = explode('?', $url, 2);
			$args = ($args) ? explode('&', $args) : array();

			foreach($args as $argument)
			{
				$arguments = explode('=', $argument);
				$key = $arguments[0];

				if($key === '' || in_array($key, $strip_vars))
				{
					continue;
				}

				$query[$key] = $argument;
			}
			
			$url = $url . (sizeof($query) ? '?' . implode('&', $query) : '');
			
			if($is_amp)
			{
				$url = str_replace('&', '&amp;', $url);
			}
		}
		
		return $url;
	}
	
	/**
	* Get user avatar
	*
	* @param string $avatar Users assigned avatar name
	* @param int $avatar_type Type of avatar
	* @param string $avatar_width Width of users avatar
	* @param string $avatar_height Height of users avatar
	* @param string $alt Optional language string for alt tag within image, can be a language key or text
	*
	* @return string Avatar image
	*/
	static public function get_user_avatar($avatar, $avatar_type, $avatar_width, $avatar_height, $alt = 'USER_AVATAR')
	{
		global $user, $config, $phpbb_root_path, $phpEx;

		if(empty($avatar) || !$avatar_type || !$config['allow_avatar'])
		{
			return '';
		}

		$avatar_img	= '';
		$board_url	= (defined('PHPBB_USE_BOARD_URL_PATH') && PHPBB_USE_BOARD_URL_PATH) ? generate_board_url() . '/' : $phpbb_root_path;

		switch($avatar_type)
		{
			case AVATAR_UPLOAD:
				if (!$config['allow_avatar_upload'])
				{
					return '';
				}
				$avatar_img = $board_url . "images/avatars/upload/";
			break;

			case AVATAR_GALLERY:
				if (!$config['allow_avatar_local'])
				{
					return '';
				}
				$avatar_img = $board_url . $config['avatar_gallery_path'] . '/';
			break;

			case AVATAR_REMOTE:
				if (!$config['allow_avatar_remote'])
				{
					return '';
				}
			break;
		}

		$avatar_img .= $avatar;
		return '<img src="' . (str_replace(' ', '%20', $avatar_img)) . '" width="' . $avatar_width . '" height="' . $avatar_height . '" alt="' . ((!empty($user->lang[$alt])) ? $user->lang[$alt] : $alt) . '" />';
	}
	
	static public function load_cron()
	{
		global $user, $config, $template;

		// Call cron-type script
		$call_cron = false;
		if(!defined('IN_BLOG_CRON') && !$config['board_disable'])
		{
			$call_cron = true;
			$time_now = (!empty($user->time_now) && is_int($user->time_now)) ? $user->time_now : time();

			// Load the cron every hour
			if (!empty($config['blog_cron_lock']))
			{
				$cron_time = explode(' ', $config['blog_cron_lock']);

				// If 1 hour lock is present we do not call cron.php
				if (($time_now - 3600) < $cron_time[0])
				{
					$call_cron = false;
				}
			}
		}

		// Call cron job?
		if($call_cron)
		{
			$template->assign_var('BLOG_CRON_TASK', '<img src="' . self::append_sid('cron') . '" width="1" height="1" alt="cron" />');
		}
	}
	
	static public function parse_attachments($action, $post_id, &$post_text, $attach_ary = false)
	{
		global $db, $user, $config, $template;
		global $blog_hook;

		$attach_count = 0;
		
		if(!$config['blog_allow_attachments'])
		{
			$action = 'remove';
		}
		
		switch($action)
		{
			case 'view':
			case 'preview':
			case 'edit':
				if($attach_ary !== false)
				{
					break;
				}
				
				$attach_ary = array();

				$sql = 'SELECT *
					FROM ' . BLOG_ATTACHMENTS_TABLE . '
					WHERE post_id = ' . $post_id;
					
				if($action == 'edit')
				{
					$sql .= ' OR (is_orphan = 1
						AND poster_id = ' . $user->data['user_id'] . ')';
				}
					
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$attach_id = (int)$row['attach_id'];
					$attach_ary[$attach_id] = $row;
					
					$attach_count++;
				}
				$db->sql_freeresult($result);
			break;
			
			case 'remove':
				$post_text = preg_replace('#\[blog-attach=([^\]]+)\]([^\[]+)*\[/blog-attach\]#is', '', $post_text);
				return true;
			break;
		}
		
		switch($action)
		{
			case 'preview':
			case 'view':
				preg_match_all('#\[blog-attach=([0-9]+)(?:;([0-9]+),([0-9]+))?\]([^\[]+)\[/blog-attach\]#is', $post_text, $matches);

				foreach($matches[1] as $id => $attach_id)
				{
					$replace = $matches[4][$id];

					if(isset($attach_ary[$attach_id]))
					{
						$replace = self::generate_attach_view('view', $attach_ary[$attach_id], $matches[2][$id], $matches[3][$id]);
						unset($attach_ary[$attach_id]);
					}
					
					$post_text = str_replace($matches[0][$id], $replace, $post_text);
				}
			break;
			
			case 'edit':
			break;
		}
		
		if($attach_ary !== false && sizeof($attach_ary) && $action != 'preview')
		{
			foreach($attach_ary as $attach_id => $attach_data)
			{
				$url_param = 'view';
				
				switch($attach_data['mimetype'])
				{
					case 'application/octet-stream':
						$url_param = 'download';
					break;
				}
				
				if(!empty($blog_hook->seo))
				{
					$blog_hook->seo->prepare_url('attach', $attach_data, 'real_filename');
				}
				
				$template->assign_block_vars('attachment', array(
					'ID'			=> $attach_id,
					'S_NAME'		=> substr($attach_data['real_filename'], 0, strrpos($attach_data['real_filename'], '.')),
					'S_NAME_FULL'	=> $attach_data['real_filename'],
					'S_FILENAME'	=> blog::append_sid('file', false, array($url_param => 1, 'i' => $attach_id)),
					'S_THUMBNAIL'	=> self::generate_attach_view('thumbnail', $attach_data),
					'IS_LINK'		=> ($action == 'view') ? true : false,
				));
			}
		}
		
		return $attach_count;
	}
	
	static public function delete_attachments($attach_ids = false, $post_ids = false)
	{
		global $db, $config, $blog_hook;

		$attach_ary = array();
		
		$deleted_attach_size = 0;
		
		if($attach_ids !== false)
		{
			if(!is_array($attach_ids))
			{
				$attach_ids = array($attach_ids);
			}
			
			$sql_where = $db->sql_in_set('attach_id', $attach_ids);
		}
		else
		{
			if(!is_array($post_ids))
			{
				$post_ids = array($post_ids);
			}
			
			$sql_where = $db->sql_in_set('post_id', $post_ids);
		}
		
		// Obtain the filenames and filesizes
		$sql = 'SELECT physical_filename, filesize
			FROM ' . BLOG_ATTACHMENTS_TABLE . "
			WHERE $sql_where";
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			$attach_ary[$row['physical_filename']] = $row['filesize'];
		}
		$db->sql_freeresult($result);
		
		// Remove the selected attachments
		$sql = 'DELETE FROM ' . BLOG_ATTACHMENTS_TABLE . "
			WHERE $sql_where";
		$db->sql_query($sql);
		
		// Find duplicates to deny file delete
		if(sizeof($attach_ary))
		{
			$sql = 'SELECT physical_filename
				FROM ' . BLOG_ATTACHMENTS_TABLE . '
				WHERE ' . $db->sql_in_set('physical_filename', array_keys($attach_ary));
			$result = $db->sql_query($sql);
			while($row = $db->sql_fetchrow($result))
			{
				unset($attach_ary[$row['physical_filename']]);
			}
			$db->sql_freeresult($result);
		}
		
		// Remove the file if no used by another post
		if(sizeof($attach_ary))
		{
			foreach($attach_ary as $physical_filename => $filesize)
			{
				@unlink($blog_hook->root_path . 'files/' . $physical_filename);
			}
		}
	}
	
	static public function delete_orphan_attachments()
	{
		global $db, $user;
		
		$orphan_ary = array();

		$sql = 'SELECT attach_id
			FROM ' . BLOG_ATTACHMENTS_TABLE . '
			WHERE poster_id = ' . $user->data['user_id'] . '
			AND is_orphan = 1
			AND post_id = 0';
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			$orphan_ary[] = $row['attach_id'];
		}
		$db->sql_freeresult($result);
		
		if(sizeof($orphan_ary))
		{
			self::delete_attachments($orphan_ary);
		}
	}
	
	static public function generate_attach_view($mode, $attach_data, $width = 0, $height = 0)
	{
		global $blog_hook;
		
		if(!empty($blog_hook->seo))
		{
			$blog_hook->seo->prepare_url('attach', $attach_data, 'real_filename');
		}

		$filename = blog::append_sid('file', false, array('i' => $attach_data['attach_id']));
		
		$width	= (int)$width;
		$height	= (int)$height;

		switch($attach_data['mimetype'])
		{
			case 'image/gif':
			case 'image/jpeg':
			case 'image/png':
				$image_size = '';
				$image_size .= ($width) ? ' width="' . $width . '"' : '';
				$image_size .= ($height) ? ' height="' . $height . '"' : '';

				return '<img class="blog-attach" src="' . $filename . '" alt="' . $attach_data['real_filename'] . '"' . $image_size . ' />';
			break;
			
			case 'application/pdf':
			case 'application/x-shockwave-flash':
			case 'text/html':
			case 'text/plain':
				if($mode == 'view')
				{
					$filename .= '&amp;view=1';
					$file_id = 'file-' . $attach_data['attach_id'];
					
					$object_size = 'autoadjust: true';
					
					if($width || $height)
					{
						$object_size = 'width: ' . (($width) ? $width : '"100%"') . ',
									height: ' . (($height) ? $height : 'auto');
					}

					return "
						<div id=\"$file_id\"></div>
						<script type=\"text/javascript\">
						// <![CDATA[
							jblog(function($) {
								$('#$file_id').appendObject({
									type: '" . $attach_data['mimetype'] . "',
									id: '$file_id',
									url: '$filename',
									$object_size,
									params: {
										wmode: 'transparent'
									}
								});
							});
						// ]]>
						</script>";
				}
			break;
			
			default:
				if($mode == 'view')
				{
					return '<a href="' . $filename . '">' . $attach_data['real_filename'] . '</a>';
				}
			break;
		}
		
		if($mode == 'thumbnail')
		{
			$thumbnail_dir = 'images/attach/';
			$thumbnail = $thumbnail_dir . 'no_extension.png';

			if(file_exists($blog_hook->root_path . $thumbnail_dir . 'image_' . $attach_data['extension'] . '.png'))
			{
				$thumbnail = $thumbnail_dir . 'image_' . $attach_data['extension'] . '.png';
			}

			return '<img src="' . (($blog_hook->domain) ? $blog_hook->domain : $blog_hook->root_path) . $thumbnail . '" alt="' . $attach_data['real_filename'] . '" />';
		}
		
		return '';
	}
	
	static public function clear_resume_tags(&$text)
	{
		$text = preg_replace('#\[\/?blog-resume\]#', '', $text);
	}
	
	/**
	* Smiley processing
	*/
	static public function smiley_text($text, $force_option = false)
	{
		global $config, $user, $phpbb_root_path;

		if ($force_option || !$config['allow_smilies'] || !$user->optionget('viewsmilies'))
		{
			$replace = '\1';
		}
		else
		{
			$root_path = (defined('PHPBB_USE_BOARD_URL_PATH') && PHPBB_USE_BOARD_URL_PATH) ? generate_board_url() . '/' : $phpbb_root_path;
			$replace = '<img class="js-resized" src="' . $root_path . $config['smilies_path'] . '/\2 />';
		}
		
		return preg_replace('#<!\-\- s(.*?) \-\-><img src="\{SMILIES_PATH\}\/(.*?) \/><!\-\- s\1 \-\->#', $replace, $text);
	}
}

/**
 * html_truncate
 * Truncates string while retaining special characters if going over the max length
 *
 * @param string	$input . The text to truncate to the given length.
 * @param int		$limit . Maximum length of string
 * @param string	$append . String to be appended
 * @param bool		$count_all . Defines if count all or only plain text
*/
if(!function_exists('html_truncate'))
{
	function html_truncate($input = '', $limit = 0, $append = '', $count_all = false)
	{
		$check_len = ($count_all) ? utf8_strlen($input) : utf8_strlen(strip_tags($input));
		if(!$input || $limit <= 0 || $check_len <= $limit)
		{
			return $input;
		}

		$splits = preg_split('#(</?[^>]*/?>)#', $input, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);

		$text = '';
		$open_tag = false;
		foreach($splits as $split)
		{
			// If no open tag, open it if necessary
			if($open_tag === false)
			{
				if(preg_match('#^<([a-zA-Z]+).*>#', $split, $match))
				{
					if(utf8_substr($match[0], -2) !== '/>')
					{
						$open_tag = $match[1];
					}
				}
			}

			// Close tag, if the current
			if($open_tag !== false)
			{
				if($split === "</$open_tag>")
				{
					$open_tag = false;
				}
			}

			// Adjust the string
			if($open_tag === false && $split[0] != '<')
			{ 
				$length = ($count_all) ? utf8_strlen($text) : utf8_strlen(strip_tags($text));

				if(($length + utf8_strlen($split)) > $limit)
				{
					$split = utf8_substr($split, 0, $limit - $length);
				}
			}

			$text .= $split;

			// If exceed the limit and no open tags finish here,
			// and it is looking backwards to remove the tag open.
			$check_len = ($count_all) ? utf8_strlen($text) : utf8_strlen(strip_tags($text));
			if($check_len >= $limit)
			{
				if($open_tag !== false)
				{
					$tag = '<' . $open_tag; 
					$text = utf8_substr($text, 0, utf8_strrpos($text, $tag));
					break;
				}
				break;
			}
		}

		return $text . (($append) ? $append : '');
	}
}

if(!function_exists('json_encode'))
{
	function json_encode($arr)
	{
		$parts		= array();
		$is_list	= false;
		$keys		= array_keys($arr);
		$max_length	= count($arr) - 1;

		//See if the first key is 0 and last key is length - 1
		if ($keys[0] == 0 && $keys[$max_length] == $max_length)
		{
			$is_list = true;

			for ($i = 0; $i < count($keys); $i++)
			{
				if($i != $keys[$i])
				{
					$is_list = false; //It is an associative array.
					break;
				}
			}
		}

		foreach ($arr as $key => $value)
		{
			$str = (($is_list) ? '' : "'$key':");

			//Custom handling for arrays
			if(is_array($value))
			{ 
				$parts[] = $str . json_encode($value); /* :RECURSION: */
			}
			else
			{
				//Custom handling for multiple data types
				if (is_numeric($value))
				{
					$str .= $value; //Numbers
				}
				else if ($value === true || $value === false)
				{
					$str .= ($value) ? 'true' : 'false'; //The booleans
				}
				else
				{
					$str .= '"' . addslashes($value) . '"'; //All other things
				}

				$parts[] = $str;
			}
		}

		$json = implode(',', $parts);

		//Return numerical JSON
		if($is_list)
		{
			return '[' . $json . ']';
		}

		//Return associative JSON
		return '{' . $json . '}';
	}
}
?>