<?php
/*
*
* @name functions.php
* @package phpBB3 cBB Blog
* @version $Id: functions.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

class blog_ajax
{
	static public function parse()
	{
		$module	= request_var('module', '');
		$action	= request_var('action', '');
		$id		= request_var('id', 0);
		
		switch($module)
		{
			case 'attachment':
				self::parse_attachments($action, $id);
			break;
			
			case 'dialog':
				self::parse_dialog($action);
			break;
		}
		
		garbage_collection();
		exit_handler();
	}
	
	static private function parse_dialog($action)
	{
		global $db, $template, $user, $auth, $config, $phpbb_root_path, $phpEx;
		global $blog_hook;
		
		if(!class_exists('blog'))
		{
			include($blog_hook->root_path . 'includes/functions.' . $phpEx);
		}
		
		$s_title = $s_message = '';
		$json = array();
		
		switch($action)
		{
			case 'confirmbox':
				$user->add_lang('mods/blog/posting');

				$message = request_var('message', '');
				
				if($message)
				{
					$template->assign_var('S_CONFIRM_MESSAGE', $user->lang($message));
				}
				
				$s_title = 'CONFIRM';
			break;
			
			case 'delete_entry':
				$post_id	= request_var('p', 0);
				$cat_id		= request_var('c', 0);
				$start		= request_var('start', 0);

				// Get post info
				$sql = 'SELECT p.*, u.username
					FROM ' . BLOG_POSTS_TABLE . ' p
						LEFT JOIN ' . USERS_TABLE . " u ON(u.user_id = p.poster_id)
					WHERE p.post_id = $post_id";
				$result = $db->sql_query($sql);
				$post_data = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
				
				if(!$post_data)
				{
					break;
				}
				
				if(blog::auth('delete', $post_data))
				{
					if(!function_exists('blog_delete_posts'))
					{
						include($blog_hook->root_path . 'includes/functions_posting.' . $phpEx);
					}

					blog_delete_posts($post_id);
				}
				
				$json['redirect'] = blog::append_sid('index', false, array(
					'c'		=> $cat_id,
					'start'	=> $start,
				));
			break;

			case 'load_blog_cats':
				blog::build_category_tree(false);
				$s_title = 'BLOG_POST_ENTRY';
			break;
			
			case 'load_forums':
				if(!function_exists('make_forum_select'))
				{
					include($phpbb_root_path . 'includes/functions_admin.' . $phpEx);
				}

				$template->assign_var('S_FORUM_SELECT', make_forum_select(0, 0, false, true, true, true));
				$s_title = 'FORUM_POST_ENTRY';
			break;
			
			case 'post_to_blog':
				$topic_id			= request_var('t', 0);
				$cat_ids			= request_var('category', array(0));
				$import_attachments	= (isset($_POST['import_attachments']) ? true : false);
				$import_replies		= (isset($_POST['import_replies']) ? true : false);

				$attach_ary = $cat_ary = $comment_ary = $posts = array();
				
				$sql = 'SELECT topic_status
					FROM ' . TOPICS_TABLE . "
					WHERE topic_id = $topic_id";
				$result = $db->sql_query($sql);
				$topic_data = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
				
				$sql = 'SELECT *
					FROM ' . POSTS_TABLE . '
					WHERE topic_id = ' . $topic_id . '
					ORDER BY post_time ASC';

				if(!$import_replies)
				{
					$sql .= ' LIMIT 1';
				}
					
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$posts[] = $row;
				}
				$db->sql_freeresult($result);
				
				if(!sizeof($posts) || !sizeof($cat_ids))
				{
					$s_message	= 'POST_TO_BLOG_ERROR';
					break;
				}
				
				$first_post = array_shift($posts);

				// Remove inline attachments of post
				$first_post['post_text'] = preg_replace('#\\n?\[attachment\=[0-9]+\:([0-9a-z]{5,})\](.*)\[\/attachment\:\\1\]\\n?#i', "\n", $first_post['post_text']);
				
				// Save the entry
				$sql_ary = array(
					'post_id'			=> 0,
					'archive_id'		=> 0,
					'poster_id'			=> $first_post['poster_id'],
					'post_username'		=> $first_post['post_username'],
					'post_approved'		=> (blog::auth('noapprove') ? 1 : 0),
					'post_title'		=> $first_post['post_subject'],
					'post_text'			=> $first_post['post_text'],
					'post_time'			=> $first_post['post_time'],
					'post_type'			=> POST_TYPE_BBCODE,
					'allow_comments'	=> ($config['blog_allow_comments'] && $topic_data['topic_status'] != ITEM_LOCKED) ? 1 : 0,
					'comment_count'		=> ($import_replies) ? sizeof($posts) : 0,
					
					'bbcode_uid'		=> $first_post['bbcode_uid'],
					'bbcode_bitfield'	=> $first_post['bbcode_bitfield'],
				);

				if(!function_exists('blog_submit_post'))
				{
					include($blog_hook->root_path . 'includes/functions_posting.' . $phpEx);
				}
				
				$post_id = blog_submit_post('post', $sql_ary, $cat_ids);
				
				// Import attachments
				if($import_attachments)
				{
					$sql = 'SELECT *
						FROM ' . ATTACHMENTS_TABLE . '
						WHERE post_msg_id = ' . $first_post['post_id'] . '
						AND ' . $db->sql_in_set('extension', explode('|', $config['blog_attach_extensions']));
					$result = $db->sql_query($sql);
					while($row = $db->sql_fetchrow($result))
					{
						$old_filename = $phpbb_root_path . $config['upload_path'] . '/' . $row['physical_filename'];
						$new_filename = $phpbb_root_path . $blog_hook->pathname . 'files/' . $row['physical_filename'];
						
						if(@copy($old_filename, $new_filename))
						{
							$attach_ary[] = array(
								'post_id'			=> $post_id,
								'poster_id'			=> $row['poster_id'],
								'physical_filename'	=> $row['physical_filename'],
								'real_filename'		=> $row['real_filename'],
								'extension'			=> $row['extension'],
								'mimetype'			=> $row['mimetype'],
								'filesize'			=> $row['filesize'],
								'filetime'			=> $row['filetime'],
								'is_orphan'			=> 0,
							);
						}
					}
					$db->sql_freeresult($result);
					
					if(sizeof($attach_ary))
					{
						$db->sql_multi_insert(BLOG_ATTACHMENTS_TABLE, $attach_ary);
					}
				}

				// Import replies as comments if allowed
				if($import_replies && $config['blog_allow_comments'] && sizeof($posts))
				{
					if(!function_exists('strip_bbcode'))
					{
						include($phpbb_root_path . 'includes/functions_content.' . $phpEx);
					}

					foreach($posts as $row)
					{
						// Delete BBcode
						strip_bbcode($row['post_text'], $row['bbcode_uid']);
						
						$row['post_text'] = trim($row['post_text']);
						
						if($row['post_text'])
						{
							$comment_ary[] = array(
								'post_id'		=> $post_id,
								'user_id'		=> $row['poster_id'],
								'user_ip'		=> $row['poster_ip'],
								'username'		=> $row['post_username'],
								'user_email'	=> '',
								'comment_text'	=> $row['post_text'],
								'comment_time'	=> $row['post_time'],
							);
						}
					}
					
					$db->sql_multi_insert(BLOG_COMMENTS_TABLE, $comment_ary);
				}

				$s_message = $user->lang['POST_TO_BLOG_SUCCESS'];
				$s_message .= '<br /><br />' . $user->lang('VIEW_ENTRY', '<a class="blog-button" href="' . blog::append_sid('view', false, array('p' => $post_id)) . '">', '</a>');
			break;
			
			case 'post_to_forum':
				$post_id			= request_var('p', 0);
				$to_forum_id 		= request_var('to_forum_id', 0);
				$import_attachments	= (isset($_POST['import_attachments']) ? true : false);
				$import_comments	= (isset($_POST['import_comments']) ? true : false);

				$attach_ary = $reply_ary = $user_counts = array();
				$first_post_id = $last_post_id = $post_count = $post_real_count = 0;
				
				$sql = 'SELECT p.*, u.username, u.user_colour, u.user_ip
					FROM ' . BLOG_POSTS_TABLE . ' p
						LEFT JOIN ' . USERS_TABLE . " u ON(u.user_id = p.poster_id)
					WHERE p.post_id = $post_id";
				$result = $db->sql_query($sql);
				$post_data = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
				
				if(!$to_forum_id || !$post_data || !$auth->acl_get('f_post', $to_forum_id))
				{
					$s_message	= 'POST_TO_FORUM_ERROR';
					break;
				}
				
				// Get attachments if allowed
				if($import_attachments)
				{
					$sql = 'SELECT *
						FROM ' . BLOG_ATTACHMENTS_TABLE . '
						WHERE post_id = ' . $post_id;
					$result = $db->sql_query($sql);
					while($row = $db->sql_fetchrow($result))
					{
						$old_filename = $phpbb_root_path . $blog_hook->pathname . 'files/' . $row['physical_filename'];
						$new_filename = $phpbb_root_path . $config['upload_path'] . '/' . $row['physical_filename'];

						if(@copy($old_filename, $new_filename))
						{
							$attach_ary[] = array(
								'post_msg_id'		=> 0,
								'topic_id'			=> 0,
								'poster_id'			=> $post_data['poster_id'],
								'is_orphan'			=> 0,
								'physical_filename'	=> $row['physical_filename'],
								'real_filename'		=> $row['real_filename'],
								'attach_comment'	=> '',
								'extension'			=> $row['extension'],
								'mimetype'			=> $row['mimetype'],
								'filesize'			=> $row['filesize'],
								'filetime'			=> $row['filetime'],
							);
						}
					}
					$db->sql_freeresult($result);
				}
				
				// Remove inline attachments of post
				$post_data['post_text'] = preg_replace('#\\n?\[blog\-attach\=[0-9]+\](.*)\[\/blog\-attach\\]\\n?#i', "\n", $post_data['post_text']);

				// Remove resume tags
				blog::clear_resume_tags($post_data['post_text']);
				
				// Add new main post
				$decoded_text = $post_data['post_text'];
				decode_message($decoded_text, $post_data['bbcode_uid']);
				
				$post_ary = array(
					'topic_id'			=> 0,
					'forum_id'			=> $to_forum_id,
					'poster_id'			=> $post_data['poster_id'],
					'poster_ip'			=> $post_data['user_ip'],
					'post_time'			=> $post_data['post_time'],
					'post_approved'		=> ($post_data['post_approved'] && $auth->acl_get('f_noapprove', $to_forum_id)) ? 1 : 0,
					'post_username'		=> $post_data['post_username'],
					'post_subject'		=> $post_data['post_title'],
					'post_text'			=> $post_data['post_text'],
					'post_checksum'		=> md5($decoded_text),
					'post_attachment'	=> (sizeof($attach_ary) ? 1 : 0),
					'bbcode_bitfield'	=> $post_data['bbcode_bitfield'],
					'bbcode_uid'		=> $post_data['bbcode_uid'],
					'post_postcount'	=> ($auth->acl_get('f_postcount', $to_forum_id)) ? 1 : 0,
					'post_edit_reason'	=> '',
				);
				
				$sql = 'INSERT INTO ' . POSTS_TABLE . ' ' . $db->sql_build_array('INSERT', $post_ary);
				$db->sql_query($sql);
				
				$first_post_id = $db->sql_nextid();
				
				// Add comments as replies if allowed
				if($import_comments)
				{
					$sql = 'SELECT c.*, u.username as user_name, u.user_colour
						FROM ' . BLOG_COMMENTS_TABLE . ' c
							LEFT JOIN ' . USERS_TABLE . " u ON(u.user_id = c.user_id)
						WHERE c.post_id = $post_id
						ORDER BY c.comment_time ASC";
					$result = $db->sql_query($sql);
					while($row = $db->sql_fetchrow($result))
					{
						$reply_ary[] = array(
							'topic_id'			=> 0,
							'forum_id'			=> $to_forum_id,
							'poster_id'			=> $row['user_id'],
							'poster_ip'			=> $row['user_ip'],
							'post_time'			=> $row['comment_time'],
							'post_approved'		=> ($post_data['post_approved'] && $auth->acl_get('f_noapprove', $to_forum_id)) ? 1 : 0,
							'post_reported'		=> ($row['spam_mark']) ? 1 : 0,
							'post_username'		=> $row['username'],
							'post_subject'		=> 'Re: ' . $post_data['post_title'],
							'post_text'			=> $row['comment_text'],
							'post_checksum'		=> md5($row['comment_text']),
							'post_attachment'	=> 0,
							'bbcode_bitfield'	=> '',
							'bbcode_uid'		=> '',
							'post_postcount'	=> ($auth->acl_get('f_postcount', $to_forum_id)) ? 1 : 0,
							'post_edit_reason'	=> '',
						);
						
						$post_real_count++;
						
						if($post_data['post_approved'])
						{
							if($auth->acl_get('f_noapprove', $to_forum_id))
							{
								$post_count++;
							}
							
							if(!isset($user_counts[$row['user_id']]))
							{
								$user_counts[$row['user_id']] = 0;
							}
							$user_counts[$row['user_id']]++;
						}
					}
					$db->sql_freeresult($result);
					
					if(sizeof($reply_ary))
					{
						$db->sql_multi_insert(POSTS_TABLE, $reply_ary);

						$sql = 'SELECT MAX(post_id) AS last_post_id
							FROM ' . POSTS_TABLE . "
							WHERE forum_id = $to_forum_id
							AND post_id > $first_post_id";
						$result = $db->sql_query($sql);
						$last_post_id = (int)$db->sql_fetchfield('last_post_id');
						$db->sql_freeresult($result);
					}
				}
				
				// Get last post data
				if($last_post_id)
				{
					$sql = 'SELECT p.poster_id, p.post_username, p.post_subject, p.post_time,
						u.username, u.user_colour
						FROM ' . POSTS_TABLE . ' p
							LEFT JOIN ' . USERS_TABLE . " u ON(u.user_id = p.poster_id)
						WHERE p.post_id = $last_post_id";
					$result = $db->sql_query($sql);
					$last_post_data = $db->sql_fetchrow($result);
					$db->sql_freeresult($result);
				}
				else
				{
					$last_post_id	= $first_post_id;
					$last_post_data	= $post_data;
				}
				
				// Insert a new topic
				$topic_ary = array(
					'forum_id'					=> $to_forum_id,
					'topic_attachment'			=> (sizeof($attach_ary) ? 1 : 0),
					'topic_approved'			=> $post_ary['post_approved'],
					'topic_title'				=> $post_data['post_title'],
					'topic_poster'				=> $post_data['poster_id'],
					'topic_time'				=> $post_data['post_time'],
					'topic_replies'				=> $post_count,
					'topic_replies_real'		=> $post_real_count,
					'topic_status'				=> ($post_data['allow_comments']) ? ITEM_UNLOCKED : ITEM_LOCKED,
					'topic_type'				=> POST_NORMAL,
					'topic_first_post_id'		=> $first_post_id,
					'topic_first_poster_name'	=> ($post_data['post_username']) ? $post_data['post_username'] : $post_data['username'],
					'topic_first_poster_colour'	=> $post_data['user_colour'],
					'topic_last_post_id'		=> $last_post_id,
					'topic_last_poster_id'		=> $last_post_data['poster_id'],
					'topic_last_poster_name'	=> ($last_post_data['post_username']) ? $last_post_data['post_username'] : $last_post_data['username'],
					'topic_last_poster_colour'	=> $last_post_data['user_colour'],
					'topic_last_post_subject'	=> (isset($last_post_data['post_title']) ? $last_post_data['post_title'] : $last_post_data['post_subject']),
					'topic_last_post_time'		=> $last_post_data['post_time'],
				);
				
				$sql = 'INSERT INTO ' . TOPICS_TABLE . ' ' . $db->sql_build_array('INSERT', $topic_ary);
				$db->sql_query($sql);
				
				$topic_id = $db->sql_nextid();
				
				// Update topic_id in posts
				$sql = 'UPDATE ' . POSTS_TABLE . '
					SET topic_id = ' . $topic_id . '
					WHERE topic_id = 0
					AND forum_id = ' . $to_forum_id . '
					AND post_id >= ' . $first_post_id;
				$db->sql_query($sql);
				
				// Update and insert the attachments
				if($import_attachments && sizeof($attach_ary))
				{
					foreach($attach_ary as $row_id => $null)
					{
						$attach_ary[$row_id]['post_msg_id'] = $first_post_id;
						$attach_ary[$row_id]['topic_id'] = $topic_id;
					}
					
					$db->sql_multi_insert(ATTACHMENTS_TABLE, $attach_ary);
				}
				
				// Update the forum counts
				if($post_ary['post_approved'])
				{
					$post_count++;
					set_config_count('num_topics', 1);
					
					if(!isset($user_counts[$post_ary['poster_id']]))
					{
						$user_counts[$post_ary['poster_id']] = 0;
					}
					$user_counts[$post_ary['poster_id']]++;
				}
				
				set_config_count('num_posts', $post_count);
				
				$sync_sql = array('forum_topics_real = forum_topics_real + 1');
				$sync_sql[] = "forum_posts = forum_posts + $post_count";
				
				if($topic_ary['topic_approved'])
				{
					$sync_sql[] = 'forum_topics = forum_topics + 1';
				}
				
				$sql = 'UPDATE ' . FORUMS_TABLE . '
					SET ' . implode(',', $sync_sql) . "
					WHERE forum_id = $to_forum_id";
				$db->sql_query($sql);
				
				if(!function_exists('sync'))
				{
					include($phpbb_root_path . 'includes/functions_admin.' . $phpEx);
				}
				
				sync('forum', 'forum_id', $to_forum_id);
				
				if(sizeof($user_counts))
				{
					foreach($user_counts as $user_id => $post_count)
					{
						$sql = 'UPDATE ' . USERS_TABLE . "
							SET user_posts = user_posts + $post_count
							WHERE user_id = $user_id";
						$db->sql_query($sql);
					}
				}
				
				$user->add_lang('posting');

				$s_message = $user->lang['POST_TO_FORUM_SUCCESS'];
				$s_message .= '<br /><br />' . $user->lang('VIEW_MESSAGE', '<a class="blog-button" href="' . append_sid("{$phpbb_root_path}viewtopic.$phpEx?f=$to_forum_id&amp;t=$topic_id") . '">', '</a>');
			break;
		}
		
		$template->assign_vars(array(
			'S_ACTION'	=> $action,
			'S_JSON'	=> (sizeof($json) ? json_encode($json) : ''),
			'S_MESSAGE'	=> (isset($user->lang[$s_message]) ? $user->lang[$s_message] : $s_message),
			'S_TITLE'	=> (isset($user->lang[$s_title]) ? $user->lang[$s_title] : ''),
		));
		
		$template->set_filenames(array('body' => 'blog/ajax_body.html'));
		$template->display('body');

		garbage_collection();
		exit_handler();
	}
	
	static private function parse_attachments($action, $id)
	{
		global $db, $user, $template, $config, $cache, $blog_hook, $phpEx;
		
		if(!class_exists('blog'))
		{
			include($blog_hook->root_path . 'includes/functions.' . $phpEx);
		}
		
		$post_id = request_var('p', 0);
		
		// First, select the attachment count for the post
		$sql_where = ($post_id) ? "post_id = $post_id" : 'poster_id = ' . $user->data['user_id'] . ' AND is_orphan = 1';
		
		$sql = 'SELECT COUNT(*) AS attach_count
			FROM ' . BLOG_ATTACHMENTS_TABLE . "
			WHERE $sql_where";
		$result = $db->sql_query($sql);
		$attach_count = (int)$db->sql_fetchfield('attach_count');
		$db->sql_freeresult($result);
		
		$json = array(
			'actions'	=> array()
		);
		
		$attach_ary = array();

		switch($action)
		{
			case 'delete':
				blog::delete_attachments($id);
				
				$attach_count--;
				
				if(!$config['blog_max_attachments'] || $attach_count < $config['blog_max_attachments'])
				{
					$json['actions'][] = 'show_upload_buttons';
				}
			break;
			
			case 'upload':
				$form_name = 'filename';
				$upload_file = (isset($_FILES[$form_name]) && $_FILES[$form_name]['name'] != 'none' && trim($_FILES[$form_name]['name'])) ? true : false;
				$error = array();
				
				if($upload_file)
				{
					if($config['blog_max_attachments'] && $attach_count >= $config['blog_max_attachments'])
					{
						$error[] = $user->lang['ATTACH_QUOTA_REACHED'];
					}
					
					if(!sizeof($error))
					{
						$filedata = self::upload_attachment($form_name);
						$error = $filedata['error'];
					}
					
					if(!empty($filedata['post_attach']) && !sizeof($error))
					{
						$attach_data = array(
							'post_id'			=> $post_id,
							'poster_id'			=> $user->data['user_id'],
							'physical_filename'	=> $filedata['physical_filename'],
							'real_filename'		=> $filedata['real_filename'],
							'extension'			=> $filedata['extension'],
							'mimetype'			=> $filedata['mimetype'],
							'filesize'			=> $filedata['filesize'],
							'filetime'			=> $filedata['filetime'],
							'is_orphan'			=> ($post_id) ? 0 : 1,
						);

						$db->sql_query('INSERT INTO ' . BLOG_ATTACHMENTS_TABLE . ' ' . $db->sql_build_array('INSERT', $attach_data));

						$attach_data['attach_id'] = $db->sql_nextid();
						
						$attach_ary[] = $attach_data;

						$attach_count++;
					}
					else
					{
						$json['actions'][] 	= 'alert';
						$json['message']	=  htmlspecialchars(implode("\n", $error));
					}
				}
			break;
			
			case 'load_gallery':
				if(!$config['blog_attach_gallery'])
				{
					break;
				}
				
				$sql = 'SELECT *
					FROM ' . BLOG_ATTACHMENTS_TABLE . '
					WHERE ' . $db->sql_in_set('post_id', array($post_id, 0), true) . '
					GROUP BY real_filename
					ORDER BY LOWER(real_filename) ASC';
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$attach_ary[] = $row;
				}
				$db->sql_freeresult($result);
				
				$template->assign_var('S_IN_GALLERY', true);
			break;
			
			case 'upload_gallery':
				if(!$config['blog_attach_gallery'])
				{
					break;
				}

				$filename = request_var('filename', '');
				
				$sql = 'SELECT *
					FROM ' . BLOG_ATTACHMENTS_TABLE . '
					WHERE real_filename = "' . $db->sql_escape($filename) . '"';
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
				
				if(!$row)
				{
					break;
				}
				
				if($config['blog_max_attachments'] && $attach_count >= $config['blog_max_attachments'])
				{
					break;
				}
				
				$attach_data = array(
					'post_id'			=> $post_id,
					'poster_id'			=> $user->data['user_id'],
					'physical_filename'	=> $row['physical_filename'],
					'real_filename'		=> $row['real_filename'],
					'extension'			=> $row['extension'],
					'mimetype'			=> $row['mimetype'],
					'filesize'			=> $row['filesize'],
					'filetime'			=> $row['filetime'],
					'is_orphan'			=> ($post_id) ? 0 : 1,
				);

				$db->sql_query('INSERT INTO ' . BLOG_ATTACHMENTS_TABLE . ' ' . $db->sql_build_array('INSERT', $attach_data));

				$attach_data['attach_id'] = $db->sql_nextid();
				
				$attach_ary[] = $attach_data;

				$attach_count++;
			break;
		}
		
		if($config['blog_max_attachments'] && $attach_count >= $config['blog_max_attachments'])
		{
			$json['actions'][] = 'hide_upload_buttons';
		}
		
		if(sizeof($json['actions']))
		{
			$template->assign_var('JSON_RESULT', json_encode($json));
		}
		
		if(sizeof($attach_ary))
		{
			foreach($attach_ary as $row)
			{
				$template->assign_block_vars('attachment', array(
					'ID'			=> $row['attach_id'],
					'S_NAME'		=> substr($row['real_filename'], 0, strrpos($row['real_filename'], '.')),
					'S_NAME_FULL'	=> $row['real_filename'],
					'S_FILENAME'	=> ($action == 'load_gallery') ? '#!' . $row['real_filename'] : blog::append_sid('file', false, array('view' => 1, 'i' => $row['attach_id'])),
					'S_THUMBNAIL'	=> blog::generate_attach_view('thumbnail', $row),
					'IS_LINK'		=> ($action == 'load_gallery') ? true : false,
				));
			}
		}
		
		page_header();
		
		$template->set_filenames(array(
			'body' => 'blog/attachments_body.html'
		));

		page_footer();
	}
	
	static private function upload_attachment($form_name)
	{
		global $phpbb_root_path, $phpEx, $auth, $user, $config, $db, $cache;
		global $blog_hook;

		$filedata = array(
			'error'	=> array()
		);
		
		$upload_path = $blog_hook->pathname . 'files/';

		if(!class_exists('fileupload'))
		{
			include($phpbb_root_path . 'includes/functions_upload.' . $phpEx);
		}

		$upload = new fileupload();

		$filedata['post_attach'] = ($upload->is_valid($form_name)) ? true : false;

		if (!$filedata['post_attach'])
		{
			$filedata['error'][] = $user->lang['NO_UPLOAD_FORM_FOUND'];
			return $filedata;
		}
		
		$allowed_extensions = explode('|', $config['blog_attach_extensions']);
		$upload->set_allowed_extensions($allowed_extensions);

		$file = $upload->form_upload($form_name);

		if($file->init_error)
		{
			$filedata['post_attach'] = false;
			return $filedata;
		}
		
		$file->upload->set_max_filesize($config['blog_max_filesize']);
		$file->clean_filename('unique', $user->data['user_id'] . '_');

		$file->move_file($upload_path, false, true);

		if (sizeof($file->error))
		{
			$file->remove();
			$filedata['error'] = array_merge($filedata['error'], $file->error);
			$filedata['post_attach'] = false;

			return $filedata;
		}

		$filedata['filesize'] 			= $file->get('filesize');
		$filedata['mimetype'] 			= $file->get('mimetype');
		$filedata['extension'] 			= $file->get('extension');
		$filedata['physical_filename'] 	= $file->get('realname');
		$filedata['real_filename'] 		= $file->get('uploadname');
		$filedata['filetime'] 			= time();

		// Check free disk space
		if ($free_space = @disk_free_space($phpbb_root_path . $upload_path))
		{
			if ($free_space <= $file->get('filesize'))
			{
				if ($auth->acl_get('a_'))
				{
					$filedata['error'][] = $user->lang['ATTACH_DISK_FULL'];
				}
				else
				{
					$filedata['error'][] = $user->lang['ATTACH_QUOTA_REACHED'];
				}
				$filedata['post_attach'] = false;

				$file->remove();

				return $filedata;
			}
		}

		return $filedata;
	}
}

?>