<?php
/*
*
* @name functions_comment.php
* @package phpBB3 cBB Blog
* @version $Id: functions_comment.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

class comment_loader
{
	var $enabled	= false;
	var $post_id	= 0;
	var $comment_id	= 0;
	
	function comment_loader($post_id = 0)
	{
		global $db, $config, $blog_hook, $phpEx;
		
		if(!class_exists('blog'))
		{
			include($blog_hook->root_path . 'includes/functions.' . $phpEx);
		}
		
		$sql = 'SELECT allow_comments
			FROM ' . BLOG_POSTS_TABLE . "
			WHERE post_id = $post_id";
		$result = $db->sql_query($sql);
		$post_allow_comments = (int)$db->sql_fetchfield('allow_comments');
		$db->sql_freeresult($result);
		
		if($config['blog_allow_comments'] && $post_allow_comments)
		{
			$this->enabled = true;
			$this->post_id = $post_id;
		}
	}

	function parse()
	{
		global $config, $user, $db;

		if(!$this->enabled)
		{
			return false;
		}
		
		$action				= request_var('action', '');
		$this->comment_id	= request_var('id', 0);
		
		switch($action)
		{
			case 'save_comment':
				$comment_text = utf8_normalize_nfc(request_var('comment_text', '', true));

				if($comment_text)
				{
					$username = $user_email = '';
					
					if($user->data['user_id'] == ANONYMOUS)
					{
						$username	= utf8_normalize_nfc(request_var('username', '', true));
						$user_email	= request_var('email', '', true);
					}

					$sql_ary = array(
						'post_id'		=> $this->post_id,
						'user_id'		=> $user->data['user_id'],
						'user_ip'		=> $user->ip,
						'username'		=> $username,
						'user_email'	=> $user_email,
						'comment_text'	=> $comment_text,
						'comment_time'	=> time(),
					);

					$sql = 'INSERT INTO ' . BLOG_COMMENTS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
					$db->sql_query($sql);

					$sql = 'UPDATE ' . BLOG_POSTS_TABLE . '
						SET comment_count = comment_count + 1
						WHERE post_id = ' . $this->post_id;
					$db->sql_query($sql);
				}

			// no break
			
			case 'load_comments':
				global $template;

				$start	= request_var('start', 0);

				// Contar comentarios totales del mensaje
				$sql = 'SELECT COUNT(*) AS total FROM ' . BLOG_COMMENTS_TABLE . '
					WHERE post_id = ' . $this->post_id;
				$result = $db->sql_query($sql);
				$comment_count = (int)$db->sql_fetchfield('total');
				$db->sql_freeresult($result);
				
				$replaces = array(
					'#href=\"((?:\?|&amp;)+start=([0-9]+)[^\"]?)\"#'	=> 'href="#" onclick="jblog.comment_system(\'load\', \\2); return false;"',
					'#href=\"\"#'										=> 'href="#" onclick="jblog.comment_system(\'load\', 0); return false;"',
				);

				$pagination = preg_replace(array_keys($replaces), array_values($replaces), generate_pagination('', $comment_count, $config['blog_cmnt_per_page'], $start));
				
				$template->assign_vars(array(
					'FULL_PAGINATION'	=> ($config['blog_pagination'] == 'full') ? true : false,
					'PAGINATION'		=> $pagination,
				));

				$this->display_comments($start);
			break;
			
			case 'comment_spam':
			case 'comment_unspam':
				$sql = 'UPDATE ' . BLOG_COMMENTS_TABLE . '
					SET spam_mark = ' . (($action == 'comment_spam') ? 1 : 0) . '
					WHERE comment_id = ' . $this->comment_id;
				$db->sql_query($sql);

				$this->display_comments();
			break;
			
			case 'comment_show':
			case 'comment_hide':
				$this->display_comments(0, (($action == 'comment_show') ? true : false));
			break;

			case 'comment_remove':
				$sql = 'SELECT user_id
					FROM ' . BLOG_COMMENTS_TABLE . '
					WHERE comment_id = ' . $this->comment_id;
				$result = $db->sql_query($sql);
				$cmnt_row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
				
				if(!blog::auth('cmnt_delete', $cmnt_row))
				{
					break;
				}

				$sql = 'DELETE FROM ' . BLOG_COMMENTS_TABLE . '
					WHERE comment_id = ' . $this->comment_id;
				$db->sql_query($sql);

				$sql = 'UPDATE ' . BLOG_POSTS_TABLE . '
					SET comment_count = comment_count - 1
					WHERE comment_count > 0
					AND post_id = ' . $this->post_id;
				$db->sql_query($sql);
			break;
		}
	}
	
	function display_comments($start = 0, $force = false)
	{
		global $db, $user, $config, $template, $phpbb_root_path, $phpEx;

		if(!function_exists('get_user_avatar'))
		{
			include($phpbb_root_path . 'includes/functions_display.' . $phpEx);
		}

		if($this->comment_id)
		{
			$sql_q = 'WHERE c.comment_id = ' . $this->comment_id;
		}
		else
		{
			$sql_q = 'WHERE c.post_id = ' . $this->post_id . '
				ORDER BY c.comment_time ' . $config['blog_cmnt_order'];
		}

		$sql = 'SELECT c.*, u.username AS member_username, u.user_colour,
			u.user_avatar, u.user_avatar_type, u.user_avatar_width, u.user_avatar_height
			FROM ' . BLOG_COMMENTS_TABLE . ' c
				LEFT JOIN ' . USERS_TABLE . " u ON(u.user_id = c.user_id)
			$sql_q";
		$result = $db->sql_query_limit($sql, $config['blog_cmnt_per_page'], $start);

		while($row = $db->sql_fetchrow($result))
		{
			$act_tpl = '&nbsp;<a href="#" data-action="comment_%2$s" data-id="%1$d" onclick="jblog.comment_system(\'click\', this); return false;">%3$s</a>&nbsp;';

			$action_ary = array();
			$actions = '';

			if($row['spam_mark'])
			{
				$action_ary += (($force) ? array('hide' => 'HIDE') : array('show' => 'SHOW'));
				$action_ary['unspam'] = 'NO_SPAM';
			}
			else
			{
				$action_ary['spam'] = 'SPAM';
			}

			if(blog::auth('cmnt_delete', $row))
			{
				$action_ary['remove'] = 'REMOVE';
			}

			foreach($action_ary as $action_id => $action_lang)
			{
				$actions .= sprintf($act_tpl, $row['comment_id'], $action_id, $user->lang[$action_lang]);
			}

			$user_avatar = ($user->optionget('viewavatars')) ? blog::get_user_avatar($row['user_avatar'], $row['user_avatar_type'], $row['user_avatar_width'], $row['user_avatar_height']) : '';

			$row['comment_text'] = make_clickable($row['comment_text']);
			
			$template->assign_block_vars('comment', array(
				'AVATAR'			=> $user_avatar,
				'USERNAME'			=> get_username_string('full', $row['user_id'], $row['member_username'], $row['user_colour'], $row['username']),

				'ID'				=> $row['comment_id'],
				'CONTENT'			=> bbcode_nl2br($row['comment_text']),
				'PUBLISHED_DATE'	=> $user->format_date($row['comment_time']),
				'IS_SPAM'			=> ($row['spam_mark'] && !$force) ? true : false,

				'ACTIONS'			=> $actions,
			));
		}

		if(!defined('PHPBB_USE_BOARD_URL_PATH'))
		{
			define('PHPBB_USE_BOARD_URL_PATH', true);
		}

		page_header('');

		$template->assign_var('S_IS_LIST', (($this->comment_id) ? false : true));

		$template->set_filenames(array(
			'body' => 'blog/comment_row_body.html'
		));

		$template->display('body');
	}
}

?>