<?php
/*
*
* @name functions_posting.php
* @package phpBB3 cBB Blog
* @version $Id: functions_posting.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

function blog_approve_posts($posts_data, $multiple = false)
{
	global $db;

	$post_ids = $post_counts = $user_counts = array();
	
	if(!sizeof($posts_data))
	{
		return false;
	}

	if(!$multiple)
	{
		if(!$posts_data['post_approved'])
		{
			$post_ids[] = $posts_data['post_id'];
			$post_counts[$posts_data['archive_id']] = 1;
			$user_counts[$posts_data['poster_id']] = 1;
		}
	}
	else
	{
		foreach($posts_data as $post_id => $row)
		{
			if(!$row['post_approved'])
			{
				$post_ids[] = $post_id;
				
				if(!isset($post_counts[$row['archive_id']]))
				{
					$post_counts[$row['archive_id']] = 0;
				}
				
				$post_counts[$row['archive_id']]++;
				
				if(!isset($user_counts[$row['poster_id']]))
				{
					$user_counts[$row['poster_id']] = 0;
				}
				
				$user_counts[$row['poster_id']]++;
			}
		}
	}
	
	if(sizeof($post_ids))
	{
		// Update the posts
		$sql = 'UPDATE ' . BLOG_POSTS_TABLE . '
			SET post_approved = 1
			WHERE ' . $db->sql_in_set('post_id', $post_ids);
		$db->sql_query($sql);
	}
	
	if(sizeof($post_counts))
	{
		$total_count = 0;

		foreach($post_counts as $archive_id => $post_count)
		{
			// Update post count in the archive for the "normal" users
			$sql = 'UPDATE ' . BLOG_ARCHIVES_TABLE . "
				SET post_count = post_count + $post_count
				WHERE archive_id = $archive_id";
			$db->sql_query($sql);
			
			$total_count += $post_count;
		}
		
		if($total_count)
		{
			set_config_count('blog_num_posts', $post_count);
		}
	}
	
	if(sizeof($user_counts))
	{
		foreach($user_counts as $user_id => $post_count)
		{
			if($post_count)
			{
				$sql = 'UPDATE ' . USERS_TABLE . "
					SET user_blog_posts = user_blog_posts + $post_count
					WHERE user_id = $user_id";
				$db->sql_query($sql);
			}
		}
	}
}

function blog_delete_posts($post_ids)
{
	global $db, $config;
	
	if(empty($post_ids))
	{
		return false;
	}
	
	$table_ary = array(BLOG_POSTS_TABLE, BLOG_POSTS_TRACK_TABLE, BLOG_COMMENTS_TABLE, BLOG_TAGS_TABLE);

	$titles = $post_counts = $user_counts = array();
	$total_count = 0;

	// First, get titles for log and post counts
	$sql = 'SELECT archive_id, poster_id, post_title, post_approved
		FROM ' . BLOG_POSTS_TABLE . '
		WHERE ' . $db->sql_in_set('post_id', $post_ids);
	$result = $db->sql_query($sql);
	while($row = $db->sql_fetchrow($result))
	{
		$titles[] = $row['post_title'];
		
		if(!isset($post_counts[$row['archive_id']]))
		{
			$post_counts[$row['archive_id']] = array(
				'post_count'		=> 0,
				'post_count_real'	=> 0
			);
		}
		
		$post_counts[$row['archive_id']]['post_count_real']++;
		
		if($row['post_approved'])
		{
			if(!isset($user_counts[$row['poster_id']]))
			{
				$user_counts[$row['poster_id']] = 0;
			}
			
			$post_counts[$row['archive_id']]['post_count']++;
			$user_counts[$row['poster_id']]++;
			$total_count++;
		}
	}
	$db->sql_freeresult($result);
	
	foreach($table_ary as $table_name)
	{
		$sql = "DELETE FROM $table_name
			WHERE " . $db->sql_in_set('post_id', $post_ids);
		$db->sql_query($sql);
	}
	
	// Delete attachments
	blog::delete_attachments(false, $post_ids);
	
	if(sizeof($post_counts))
	{
		foreach($post_counts as $archive_id => $post_count)
		{
			$update_ary = array('post_count_real = post_count_real - ' . $post_count['post_count_real']);
			
			if($post_count['post_count'])
			{
				$update_ary[] = 'post_count = post_count - ' . $post_count['post_count'];
			}

			// Update post count in the archive for the "normal" users
			$sql = 'UPDATE ' . BLOG_ARCHIVES_TABLE . '
				SET ' . implode(',', $update_ary) . "
				WHERE archive_id = $archive_id
				AND post_count_real > 0";
			$db->sql_query($sql);
		}
		
		// Update post count
		if($total_count)
		{
			set_config('blog_num_posts', $config['blog_num_posts'] - $total_count);
		}
	}
	
	if(sizeof($user_counts))
	{
		foreach($user_counts as $user_id => $post_count)
		{
			if($post_count)
			{
				$sql = 'UPDATE ' . USERS_TABLE . "
					SET user_blog_posts = user_blog_posts - $post_count
					WHERE user_id = $user_id";
				$db->sql_query($sql);
			}
		}
	}
	
	// Delete empty archives
	$sql = 'DELETE FROM ' . BLOG_ARCHIVES_TABLE . '
		WHERE post_count = 0
		AND post_count_real = 0';
	$db->sql_query($sql);
	
	add_log('mod', 0, 0, 'LOG_BLOG_DELETE_POST', implode(', ', $titles));
	return true;
}

function blog_submit_post($mode, &$post_data, $post_cats = false, $post_tags = false)
{
	global $db, $user, $blog_hook, $phpEx;
	
	$cat_ary = $tag_ary = array();
	
	$post_id	= $post_data['post_id'];
	$archive_id	= $post_data['archive_id'];

	// Find archive id by time or create new
	$post_date = getdate($post_data['post_time']);

	$sql = 'SELECT archive_id
		FROM ' . BLOG_ARCHIVES_TABLE . '
		WHERE archive_month = ' . $post_date['mon'] . '
		AND archive_year = ' . $post_date['year'];
	$result = $db->sql_query($sql);
	$new_archive_id = (int)$db->sql_fetchfield('archive_id');
	$db->sql_freeresult($result);

	// Create new archive
	if(!$new_archive_id)
	{
		$sql_ary = array(
			'archive_month'		=> $post_date['mon'],
			'archive_year'		=> $post_date['year'],
			'post_count'		=> 0,
			'post_count_real'	=> 0,
		);
		
		$sql = 'INSERT INTO ' . BLOG_ARCHIVES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
		
		$new_archive_id = $db->sql_nextid();
	}
	
	if($post_data['post_approved'])
	{
		// Update post count of old poster
		if(!empty($post_data['old_poster_id']))
		{
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_blog_posts = user_blog_posts - 1
				WHERE user_blog_posts > 0
				AND user_id = ' . $post_data['old_poster_id'];
			$db->sql_query($sql);
		}
	
		// Update post count of current poster
		if($mode == 'post' || ($mode == 'edit' && !empty($post_data['old_poster_id'])))
		{
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_blog_posts = user_blog_posts + 1
				WHERE user_id = ' . $post_data['poster_id'];
			$db->sql_query($sql);
		}
	}
	
	unset($post_data['old_poster_id'], $post_data['post_id']);
	
	$post_data['archive_id'] = $new_archive_id;
	
	switch($mode)
	{
		case 'edit':
			// First, delete old tags
			$sql = 'DELETE FROM ' . BLOG_TAGS_TABLE . "
				WHERE post_id = $post_id";
			$db->sql_query($sql);
			
			// Delete old category track
			$sql = 'DELETE FROM ' . BLOG_POSTS_TRACK_TABLE . "
				WHERE post_id = $post_id";
			$db->sql_query($sql);
			
			// Update archives table if neccessary
			if($archive_id != $new_archive_id)
			{
				$update_ary = array('post_count_real = post_count_real - 1');
				
				if($post_data['post_approved'])
				{
					$update_ary[] = 'post_count = post_count - 1';
				}
				
				$sql = 'UPDATE ' . BLOG_ARCHIVES_TABLE . '
					SET ' . implode(',', $update_ary) . '
					WHERE archive_id = ' . $archive_id;
				$db->sql_query($sql);
			}
			
			// Delete deprecated archives if neccessary
			$sql = 'DELETE FROM ' . BLOG_ARCHIVES_TABLE . "
				WHERE post_count_real = 0
				AND archive_id <> $new_archive_id";
			$db->sql_query($sql);
			
			// Delete comments if no comments are allowed
			if(!$post_data['allow_comments'] && $post_data['comment_count'])
			{
				$sql = 'DELETE FROM ' . BLOG_COMMENTS_TABLE . "
					WHERE post_id = $post_id";
				$db->sql_query($sql);
			}
		
			$sql = 'UPDATE ' . BLOG_POSTS_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $post_data) . "
				WHERE post_id = $post_id";
			$db->sql_query($sql);
		break;
		
		case 'post':
			if(!class_exists('blog_seo'))
			{
				include($blog_hook->root_path . 'includes/functions_seo.' . $phpEx);
			}
			
			$blog_seo = new blog_seo();

			$post_data['post_url'] = $blog_seo->format_url($post_data['post_title']);
		
			$sql = 'INSERT INTO ' . BLOG_POSTS_TABLE . ' ' . $db->sql_build_array('INSERT', $post_data);
			$db->sql_query($sql);
			
			$post_id = $db->sql_nextid();
			
			$sql = 'UPDATE ' . BLOG_ATTACHMENTS_TABLE . "
				SET post_id = $post_id, is_orphan = 0
				WHERE poster_id = " . $user->data['user_id'] . '
				AND is_orphan = 1
				AND post_id = 0';
			$db->sql_query($sql);

			if($post_data['post_approved'])
			{
				set_config_count('blog_num_posts', 1);
			}
		break;
	}
	
	$post_data['post_id'] = $post_id;
	
	// Update archives table
	if($archive_id != $new_archive_id)
	{
		$update_ary = array('post_count_real = post_count_real + 1');
		
		if($post_data['post_approved'])
		{
			$update_ary[] = 'post_count = post_count + 1';
		}
		
		$sql = 'UPDATE ' . BLOG_ARCHIVES_TABLE . '
			SET ' . implode(',', $update_ary) . "
			WHERE archive_id = $new_archive_id";
		$db->sql_query($sql);
	}
		
	// Set the categories
	if(!empty($post_cats))
	{
		foreach($post_cats as $cid)
		{
			$cat_ary[] = array(
				'post_id'	=> $post_id,
				'cat_id'	=> $cid,
			);
		}

		$db->sql_multi_insert(BLOG_POSTS_TRACK_TABLE, $cat_ary);
	}
	
	// Insert the tags
	if(!empty($post_tags))
	{
		foreach($post_tags as $tag)
		{
			$tag_ary[] = array(
				'post_id'	=> $post_id,
				'tag_name'	=> $tag,
			);
		}
		
		$db->sql_multi_insert(BLOG_TAGS_TABLE, $tag_ary);
	}
	
	return $post_id;
}

?>