<?php
/*
*
* @name functions_rss.php
* @package phpBB3 cBB Blog
* @version $Id: functions_rss.php,v1.0.1 01/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class blog_rss
{
	var $data = array(
		'encoding'	=> 'UTF-8',
		'type'		=> '',
	);

	var $rss_data;
	var $rss_type;
	var $rss_ttl = 86400;
	
	var $tags = array(
		'rss'	=> array('name'	=> 'channel', 	'tags'	=> 'item|image'),
		'feed'	=> array('name'	=> 'feed', 		'tags'	=> 'entry'),
	);

	function _clear($text, $data)
	{
		$replaces = array(
			'#<object([^>]+)>.*?</object>#is',
			'#<blockquote>.*?</blockquote>#is',
			'#<\/?[^>]+>#s'
		);
		
		// Delete objects and line breaks
		if($data['direction'] == BLG_SCROLL_H)
		{
			$text = preg_replace($replaces, '', $text);
			$text = str_replace(array("\r", "\n", '<br />'), ' ', $text);
		}
		
		return $text;
	}
	
	function get_build_time($date = '')
	{
		$build_time = 0;
		
		if($date)
		{
			$build_time = $date;
		}
		else
		{
			switch($this->rss_type)
			{
				case 'feed':
					if(empty($this->rss_data['feed']['updated']))
					{
						$build_time = (sizeof($this->rss_data['entry']) ? $this->rss_data['entry'][0]['updated'] : 0);
					}
					else
					{
						$build_time = $this->rss_data['feed']['updated'];
					}
				break;

				case 'rss':
					if(empty($this->rss_data['channel']['lastBuildDate']))
					{
						$build_time = (sizeof($this->rss_data['item']) ? $this->rss_data['item'][0]['pubDate'] : 0);
					}
					else
					{
						$build_time = $this->rss_data['channel']['lastBuildDate'];
					}
				break;
			}
		}

		if($build_time = @strtotime($build_time))
		{
			return $build_time;
		}
		
		return time();
	}
	
	function read($url)
	{
		global $cache;
		
		if(!$url)
		{
			return false;
		}
		
		/* Reset the values */
		$this->rss_type = '';
		$this->rss_data = $this->data;

		$file_text = @file_get_contents(str_replace('&amp;', '&', $url));
		
		if(!$file_text)
		{
			return false;
		}
		
		// Get Encoding
		$this->rss_data['encoding'] = $this->data['encoding'];

		if(preg_match('#encoding=[\'\"]+([\w\d\-\_]+)[\'\"]+#', $file_text, $out))
		{
			$this->rss_data['encoding'] = $out[1];
		}
		
		// Determine the rss type and language
		preg_match('#<(rss|feed)+.*?(?:xml:lang\=\"([a-z\-]+)\")?[^>]*>#', $file_text, $out);
		
		switch($out[1])
		{
			case 'rss':
			case 'feed':
				$this->rss_type = $this->rss_data['type'] = $out[1];
				
				if(!empty($out[2]))
				{
					$this->rss_data['language'] = $out[2];
				}

				$this->_get($file_text);
			break;
			
			default:
				return false;
			break;	
		}
		
		$this->rss_type = $this->rss_data['type'];
		
		return true;
	}
	
	function get($id, $name = '', $ary = false)
	{
		if($name)
		{
			if($id === false)
			{
				$ary = $this->rss_data[$this->tags[$this->rss_type]['name']];

				return (isset($ary[$name]) ? $ary[$name] : '');
			}
			else if(is_array($ary) && isset($ary[$id][$name]))
			{
				return $ary[$id][$name];
			}

			return '';
		}

		if(isset($this->rss_data[$id]))
		{
			return $this->rss_data[$id];
		}
		
		return array();
	}
	
	function _get($file_text)
	{
		$channel_name = $this->tags[$this->rss_type]['name'];

		if(!preg_match('#<(' . $channel_name . ').*?>(.*?)<\/\\1>#si', $file_text, $out_channel))
		{
			return false;
		}
		
		$file_text = $out_channel[2];
		unset($out_channel);

		// Get the item,  textinput and image variables
		while(preg_match('#<(' . $this->tags[$this->rss_type]['tags'] . ').*?>(.*?)<\/\\1>#si', $file_text, $out))
		{
			if(!isset($this->rss_data[$out[1]]))
			{
				$this->rss_data[$out[1]] = array();
			}
			
			$this->rss_data[$out[1]][] = $this->_compose($out[2]);

			$file_text = str_replace($out[0], '', $file_text);
		}
		
		// And finally, get the channel variables
		$this->rss_data[$channel_name] = $this->_compose($file_text);
	}

	function adjust($text)
	{
		$text = str_replace(array('<![CDATA[', ']]>'), '', $text);
		
		$text = trim($text);
		
		$text = preg_replace('#<img[^>]*src="([^"]*)"[^>]*/?>#', '<img src="\\1" />', $text);
		
		if($text && $this->rss_data['encoding'] != $this->data['encoding'])
		{
			$text = iconv($this->rss_data['encoding'], $this->data['encoding'] . '//TRANSLIT', $text);
		}

		return $text;	
	}

	function _compose($text)
	{
		$ary = array();
		
		preg_match_all('#<([\w\:]+)[^>]*>(.*?)<\/\\1>#si', $text, $out, PREG_SET_ORDER);

		foreach($out as $r)
		{
			$ary[$r[1]] = $this->adjust($r[2]);
		}
		
		return $ary;
	}
}
?>
