<?php
/*
*
* @name functions_seo.php
* @package phpBB3 cBB Blog
* @version $Id: functions_seo.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

class blog_seo
{
	var $path	= '';
	var $url	= '';
	var $params	= array();
	var $cache	= array();
	
	function blog_seo()
	{
		if(!defined('PHPBB_USE_BOARD_URL_PATH'))
		{
			define('PHPBB_USE_BOARD_URL_PATH', true);
		}
		
		$this->regex['url_find'] = array(
			'#&([a-z]+)(acute|grave|circ|cedil|tilde|uml|lig|ring|caron|slash);#i',
			'#&(amp;)?[^;]+;#i',
			'#[^a-z0-9\.]#i',
			'#(^|-)[a-z]{1,2}(?=-)#i', // Small words
			'#[-]+#');

		$this->regex['url_replace'] = array('\1', '-', '-', '-', '-');
		
		// Find cat_id
		$ci = request_var('ci', '');
		$cp	= request_var('cp', '');
		
		if($ci)
		{
			$cat_ary = blog::get_category('ids');
			
			$_REQUEST['c'] = -1;

			foreach($cat_ary as $cat_id => $cat_data)
			{
				if($cat_data['cat_url'] == $ci)
				{
					// Compare cat parent
					if($cat_data['parent_id'])
					{
						$parent_id = $cat_data['parent_id'];
						
						if($cat_ary[$parent_id]['cat_url'] != $cp)
						{
							continue;
						}
					}
					
					$_REQUEST['c'] = $cat_id;
					break;
				}
			}
		}
	}
	
	function check()
	{
		$check_seo = request_var('checkseo', '');
		
		$args	= func_get_args();
		$type	= array_shift($args);
		
		if(sizeof($args))
		{
			foreach($args as $value)
			{
				if($check_seo && !$value)
				{
					continue;
				}
				
				if($type != 'cat')
				{
					$value = $this->format_url($value, $type);
				}
				
				if($value == $check_seo)
				{
					return true;
				}
			}
		}
		
		return false;
	}
	
	function format_url($url, $type = '')
	{
		global $user;

		$url = preg_replace('#\[.*\]#U','', $url);
		$url = htmlentities($url, ENT_COMPAT, 'UTF-8');
		
		if(isset($user->lang['blog_seo_replaces']))
		{
		   $url = str_replace(array_keys($user->lang['blog_seo_replaces']), array_values($user->lang['blog_seo_replaces']), $url);
		}
		$url = preg_replace($this->regex['url_find'] , $this->regex['url_replace'], $url);
		$url = strtolower(trim($url, '-'));
		
		return (empty($url) ? $type : $url);
	}
	
	function prepare_url($type, $data, $alt = '')
	{
		$id = max(0, $data[$type . '_id']);

		if(!empty($data[$type . '_url']))
		{
			$this->cache[$type][$id] = $data[$type . '_url'];
		}
		else if($alt)
		{
			$alt = (!empty($data[$alt]) ? $alt : $type . '_' . $alt);

			if(!empty($data[$alt]))
			{
				$this->cache[$type][$id] = $this->format_url($data[$alt], $type);
			}
		}
	}

	function rewrite(&$filename, &$params)
	{
		global $blog_hook, $phpEx;

		if(!preg_match("#(.*?/)([a-z]+)(\.$phpEx)#", $filename, $url_parts))
		{
			return false;
		}
		
		if(method_exists($this, '_' . $url_parts[2]))
		{
			$this->url		= $url_parts[2] . $url_parts[3];
			$this->path 	= $url_parts[1];
			$this->params 	= $params;
			
			if(strpos($this->path, 'http') !== 0)
			{
				$this->path = generate_board_url() . '/' . $blog_hook->pathname;
			}
			
			call_user_func(array($this, '_' . $url_parts[2]));
			
			$filename	= $this->path . ((substr($this->path, -1) != '/') ? '/' : '') . $this->url;
			$params		= $this->params;
		}
	}
	
	function unformat()
	{
		global $user, $phpEx;
		
		$getvars = explode('&', $user->page['query_string']);
		$params = array();
		
		$params['c'] = request_var('c', 0);
		
		foreach($getvars as $var)
		{
			$p = explode('=', $var, 2);
			$params[$p[0]] = $p[1];
		}
		
		unset($params['seo'], $params['ci'], $params['checkseo']);
		
		return array(str_replace(".$phpEx", '', $user->page['page_name']), false, $params);
	}
	
	function _index()
	{
		$this->url = '';

		if(!empty($this->params['c']))
		{
			$cat_data = blog::get_category($this->params['c']);
			
			if($cat_data['parent_id'])
			{
				$parent_name = blog::get_category($cat_data['parent_id'], 'cat_url');
				$this->path .= $parent_name . '/';
			}
			
			$this->path .= $cat_data['cat_url'] . '/';
		}
		
		if(!empty($this->params['a']))
		{
			$this->url = 'a/' . $this->params['a'] . '/';
		}
		
		unset($this->params['a'], $this->params['c']);
	}
	
	function _file()
	{
		$this->path .= 'resources/';
		$this->url = $this->params['i'] . '/';
		
		if(!empty($this->cache['attach'][$this->params['i']]))
		{
			$this->url .= $this->cache['attach'][$this->params['i']];
		}
		
		unset($this->params['i']);
	}
	
	function _rss()
	{
		$this->path .= 'rss/';
		
		if(!empty($this->params['c']))
		{
			$cat_name = blog::get_category($this->params['c'], 'cat_url');
			$this->path .= $cat_name . '/';
		}
		
		$this->url = $this->params['mode'] . '.html';
		
		unset($this->params['c'], $this->params['mode']);
	}
	
	function _search()
	{
		$this->url = 's/';
	}
	
	function _view()
	{
		if(!empty($this->params['c']))
		{
			$cat_data = blog::get_category($this->params['c']);
			
			if($cat_data['parent_id'])
			{
				$parent_name = blog::get_category($cat_data['parent_id'], 'cat_url');
				$this->path .= $parent_name . '/';
			}
			
			$this->path .= $cat_data['cat_url'] . '/';
		}
		
		$this->url = 'post';
		
		if(!empty($this->cache['post'][$this->params['p']]))
		{
			$this->url = $this->cache['post'][$this->params['p']];
		}
		
		$this->url .= '-p' . $this->params['p'] . '.html';
		
		unset($this->params['c'], $this->params['p']);
	}
}

?>