<?php
/*
*
* @name html_message_parser.php
* @package phpBB3 cBB Blog
* @version $Id: html_message_parser.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

class html_parse_message
{
	var $clear_resume = false;
	var $message;
	var $bbcode_template;

	function parse($allow_magic_url, $allow_smilies, $allow_img_bbcode = true, $allow_flash_bbcode = true, $allow_quote_bbcode = true, $allow_url_bbcode = true, $update_this_message = true, $mode = 'post')
	{
		global $config;

		foreach (array('chars', 'smilies', 'urls', 'font_size', 'img_height', 'img_width') as $key)
		{
			if (!isset($config['max_' . $mode . '_' . $key]))
			{
				$config['max_' . $mode . '_' . $key] = 0;
			}
		}

		if($allow_smilies)
		{
			$this->smilies($config['max_' . $mode . '_smilies']);
		}
	}

	function decode_message($text = false)
	{
		$match = array(
			'#<!\-\- s(.*?) \-\-><img src="\{SMILIES_PATH\}\/.*? \/><!\-\- s\1 \-\->#'
		);
		
		$replace = array('\1');

		if($text === false)
		{
			$this->message = preg_replace($match, $replace, $this->message);
		}
		else
		{
			return preg_replace($match, $replace, $text);
		}
	}
	
	function format_display($allow_bbcode, $allow_magic_url, $allow_smilies, $update_this_message = true)
	{
		// Replace naughty words such as farty pants
		$new_text = $this->message;
		$new_text = censor_text($new_text);
		$new_text = htmlspecialchars_decode($new_text);
		
		// Do some operations with code tags
		preg_match_all('#<code>(.*?)</code>#is', $new_text, $matches);
	
		foreach($matches[1] as $match_id => $match_text)
		{
			$match_text = htmlspecialchars($this->decode_message($match_text));
			$match_text = bbcode_nl2br($match_text);
			$match_text = str_replace("\t", '&nbsp;&nbsp;&nbsp;', $match_text);

			$new_text = str_replace($matches[0][$match_id], '<code>' . $match_text . '</code>', $new_text);
		}
		
		$new_text = blog::smiley_text($new_text, !$allow_smilies);
		
		if($this->clear_resume)
		{
			blog::clear_resume_tags($new_text);
		}
		
		if($allow_bbcode)
		{
			$this->bbcode_cache_init();
	
			$replaces = array(
				'<blockquote>'	=> 'quote_open',
				'</blockquote>'	=> 'quote_close',
				'<code>'		=> 'code_open',
				'</code>'		=> 'code_close',
			);
			
			foreach($replaces as $find => $tpl_name)
			{
				$new_text = str_replace($find, $this->bbcode_tpl($tpl_name), $new_text);
			}
		}
		
		if($update_this_message)
		{
			$this->message = $new_text;
		}
		
		return $new_text;
	}
	
	function bbcode_cache_init()
	{
		global $phpbb_root_path, $user;

		$template_filename = $phpbb_root_path . 'styles/' . $user->theme['template_path'] . '/template/bbcode.html';

		if (!@file_exists($template_filename))
		{
			if (isset($user->theme['template_inherits_id']) && $user->theme['template_inherits_id'])
			{
				$template_filename = $phpbb_root_path . 'styles/' . $user->theme['template_inherit_path'] . '/template/bbcode.html';
				
				if (!@file_exists($template_filename))
				{
					trigger_error('NO_BBCODE_TEMPLATE', E_USER_ERROR);
				}
			}
			else
			{
				trigger_error('NO_BBCODE_TEMPLATE', E_USER_ERROR);
			}
		}
		
		if (empty($this->bbcode_template))
		{
			if (($tpl = @file_get_contents($template_filename)) === false)
			{
				trigger_error('NO_BBCODE_TEMPLATE', E_USER_ERROR);
			}

			// replace \ with \\ and then ' with \'.
			$tpl = str_replace('\\', '\\\\', $tpl);
			$tpl = str_replace("'", "\'", $tpl);

			// strip newlines and indent
			$tpl = preg_replace("/\n[\n\r\s\t]*/", '', $tpl);

			// Turn template blocks into PHP assignment statements for the values of $bbcode_tpl..
			$this->bbcode_template = array(
				'quote_open'	=> '<blockquote><div>',
				'quote_close'	=> '</div></blockquote>',
				'code_open'		=> '<code>',
				'code_close'	=> '</code>',
			);

			$matches = preg_match_all('#<!-- BEGIN (.*?) -->(.*?)<!-- END (?:.*?) -->#', $tpl, $match);

			for ($i = 0; $i < $matches; $i++)
			{
				if (empty($match[1][$i]))
				{
					continue;
				}

				$this->bbcode_template[$match[1][$i]] = $this->bbcode_tpl_replace($match[2][$i]);
			}
		}
	}
	
	/**
	* Return bbcode template
	*/
	function bbcode_tpl($tpl_name)
	{
		return (isset($this->bbcode_template[$tpl_name]) ? $this->bbcode_template[$tpl_name] : false);
	}
	
	/**
	* Return bbcode template replacement
	*/
	function bbcode_tpl_replace($tpl)
	{
		global $user;

		$tpl = preg_replace('/{L_([A-Z_]+)}/e', "(!empty(\$user->lang['\$1'])) ? \$user->lang['\$1'] : ucwords(strtolower(str_replace('_', ' ', '\$1')))", $tpl);

		return trim($tpl);
	}
	
	/**
	* Parse Smilies
	*/
	function smilies($max_smilies = 0)
	{
		global $db, $user;
		static $match;
		static $replace;

		// See if the static arrays have already been filled on an earlier invocation
		if (!is_array($match))
		{
			$match = $replace = array();

			// NOTE: obtain_* function? chaching the table contents?

			// For now setting the ttl to 10 minutes
			switch ($db->sql_layer)
			{
				case 'mssql':
				case 'mssql_odbc':
				case 'mssqlnative':
					$sql = 'SELECT *
						FROM ' . SMILIES_TABLE . '
						ORDER BY LEN(code) DESC';
				break;

				case 'firebird':
					$sql = 'SELECT *
						FROM ' . SMILIES_TABLE . '
						ORDER BY CHAR_LENGTH(code) DESC';
				break;

				// LENGTH supported by MySQL, IBM DB2, Oracle and Access for sure...
				default:
					$sql = 'SELECT *
						FROM ' . SMILIES_TABLE . '
						ORDER BY LENGTH(code) DESC';
				break;
			}
			$result = $db->sql_query($sql, 600);

			while ($row = $db->sql_fetchrow($result))
			{
				if (empty($row['code']))
				{
					continue;
				}

				// (assertion)
				$match[] = preg_quote($row['code'], '#');
				$replace[] = '<!-- s' . $row['code'] . ' --><img src="{SMILIES_PATH}/' . $row['smiley_url'] . '" alt="' . $row['code'] . '" title="' . $row['emotion'] . '" /><!-- s' . $row['code'] . ' -->';
			}
			$db->sql_freeresult($result);
		}

		if (sizeof($match))
		{
			if ($max_smilies)
			{
				$num_matches = preg_match_all('#(?<=^|[\n .])(?:' . implode('|', $match) . ')(?![^<>]*>)#', $this->message, $matches);
				unset($matches);

				if ($num_matches !== false && $num_matches > $max_smilies)
				{
					$this->warn_msg[] = sprintf($user->lang['TOO_MANY_SMILIES'], $max_smilies);
					return;
				}
			}

			// Make sure the delimiter # is added in front and at the end of every element within $match
			$this->message = trim(preg_replace(explode(chr(0), '#(?<=^|[\n .])' . implode('(?![^<>]*>)#' . chr(0) . '#(?<=^|[\n .])', $match) . '(?![^<>]*>)#'), $replace, $this->message));
		}
	}
}

?>