<?php
/*
*
* @name mcp_posts.php
* @package phpBB3 cBB Blog
* @version $Id: mcp_posts.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

class mcp_posts
{
	var $tpl_name;
	var $page_title;

	function main($mode, $id)
	{
		global $config, $db, $template, $auth, $user, $action, $cat_id, $phpEx;
		global $blog_hook;
		
		include($blog_hook->root_path . 'includes/functions_posting.' . $phpEx);

		$this->tpl_name = 'mcp_posts';

		$start			= request_var('start', 0);
		$sd 			= request_var('sd', 'DESC');
		$sk 			= request_var('sk', 't');
		$post_id_list 	= request_var('post_id_list', array(0));
		
		$sort_dirs 		= array('ASC' => 'ASCENDING', 'DESC' => 'DESCENDING');
		$sort_by_text 	= array('t' => 'POST_TIME', 'pt' => 'POST_TITLE');
		$sort_by_sql	= array('t' => 'p.post_time', 'pt' => 'p.post_title');

		if($action && empty($post_id_list))
		{
			$action = '';
		}
		
		switch($action)
		{
			case 'approve':
				if(!$auth->acl_get('m_blog_approve'))
				{
					trigger_error('NOT_AUTHORISED');
				}
				
				$posts_data = array();
				
				$sql = 'SELECT post_id, archive_id, post_approved
					FROM ' . BLOG_POSTS_TABLE . '
					WHERE ' . $db->sql_in_set('post_id', $post_id_list);
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$post_id = (int)$row['post_id'];
					$posts_data[$post_id] = $row;
				}
				$db->sql_freeresult($result);
				
				blog_approve_posts($posts_data, true);
			break;

			case 'delete':
				if(!$auth->acl_get('m_blog_delete'))
				{
					trigger_error('NOT_AUTHORISED');
				}
				
				if(confirm_box(true))
				{
					blog_delete_posts($post_id_list);
				}
				else
				{
					$s_hidden_fields = build_hidden_fields(array(
						'm'				=> $mode,
						'i'				=> $id,
						'post_id_list'	=> $post_id_list,
						'action'		=> 'delete',
					));
					
					confirm_box(false, 'DELETE_POSTS', $s_hidden_fields);
				}
			break;
			
			case 'move':
				if(confirm_box(true))
				{
					$cat_id_list = request_var('category', array(0 => 0));
					
					if(sizeof($cat_id_list) && sizeof($post_id_list))
					{
						// Remove previous categories of the posts
						$sql = 'DELETE FROM ' . BLOG_POSTS_TRACK_TABLE . '
							WHERE ' . $db->sql_in_set('post_id', $post_id_list);
						$db->sql_query($sql);
						
						// Insert the new categories
						$sql_ary = array();
						
						foreach($post_id_list as $row_post_id)
						{
							foreach($cat_id_list as $row_cat_id)
							{
								$sql_ary[] = array(
									'post_id'	=> $row_post_id,
									'cat_id'	=> $row_cat_id,
								);
							}
						}

						$db->sql_multi_insert(BLOG_POSTS_TRACK_TABLE, $sql_ary);
					}
				}
				else
				{
					blog::build_category_tree(false);
					
					$s_hidden_fields = build_hidden_fields(array(
						'm'				=> $mode,
						'i'				=> $id,
						'post_id_list'	=> $post_id_list,
						'action'		=> 'move',
					));
					
					confirm_box(false, 'MOVE_POSTS', $s_hidden_fields, 'blog/mcp_move.html');
				}
			break;
			
			case 'sticky':
				$sql = 'UPDATE ' . BLOG_POSTS_TABLE . '
					SET post_fixed = 1
					WHERE ' . $db->sql_in_set('post_id', $post_id_list);
				$db->sql_query($sql);
			break;
		}

		$authed_cats = $post_rows = array();
		
		// Get authed cats
		if($cat_id)
		{
			if(blog::category_auth($cat_id))
			{
				$authed_cats[] = $cat_id;
			}
		}
		else
		{
			$cat_rows = blog::get_category('ids');

			foreach($cat_rows as $row_cat_id => $row)
			{
				if(blog::category_auth($row))
				{
					$authed_cats[] = $row_cat_id;
				}
			}
		}

		if(!sizeof($authed_cats))
		{
			trigger_error('NOT_AUTHORISED');
		}

		// Main SQL
		$sql_ary = array(
			'SELECT'	=> 'p.*, u.username, u.user_colour',
			'FROM'		=> array(
				BLOG_POSTS_TABLE		=> 'p',
				BLOG_POSTS_TRACK_TABLE	=> 'pt',
			),
			
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'u.user_id = p.poster_id'
				)
			),
			'WHERE'		=> 'pt.post_id = p.post_id
				AND ' . $db->sql_in_set('pt.cat_id', $authed_cats),
			'ORDER_BY'	=> 'p.post_time DESC',
		);
		
		if(isset($sort_by_sql[$sk]))
		{
			$sql_ary['ORDER_BY'] = $sort_by_sql[$sk] . ' ' . (($sd == 'ASC') ? 'ASC' : 'DESC');
		}
		
		// Count the posts
		$count_sql_ary = array(
			'SELECT'	=> 'COUNT(p.post_id) AS total_posts',
			'FROM'		=> $sql_ary['FROM'],
			'WHERE'		=> $sql_ary['WHERE'],
		);

		$sql = $db->sql_build_query('SELECT', $count_sql_ary);
		$result = $db->sql_query($sql);
		$total_posts = (int)$db->sql_fetchfield('total_posts');
		$db->sql_freeresult($result);

		$sql = $db->sql_build_query('SELECT', $sql_ary);
		$result = $db->sql_query_limit($sql, $config['blog_posts_per_page'], $start);
		while($row = $db->sql_fetchrow($result))
		{
			$post_id = (int)$row['post_id'];
			
			$post_rows[$post_id] = $row;
			$post_rows[$post_id]['cats'] = array();
		}
		$db->sql_freeresult($result);
		
		// Get all the categories for the posts
		blog::append_post_categories($post_rows);
		
		foreach($post_rows as $post_id => $row)
		{
			$params = array(
				'm'		=> $mode,
				'i'		=> $id,
			);

			if(!empty($blog_hook->seo))
			{
				$blog_hook->seo->prepare_url('post', $row, 'title');
			}
			
			$template->assign_block_vars('postrow', array(
				'ID'			=> $row['post_id'],
				
				'S_CATEGORIES'	=> blog::row_categories($row, 'mcp', $params),
				'S_TITLE'		=> $row['post_title'],
				'S_AUTHOR'		=> get_username_string('full', $row['poster_id'], $row['username'], $row['user_colour'], $row['post_username']),
				'S_TIME'		=> $user->format_date($row['post_time']),

				'U_VIEW_DETAILS'	=> blog::append_sid('view', $row),
			));
		}
		
		// Build category select
		$row_count = 0;

		$template->assign_block_vars('jumpbox_cats', array(
			'CAT_NAME'		=> $user->lang('SELECT_CATEGORY'),
			'SELECTED'		=> (!$cat_id) ? true : false,
		));
		
		blog::build_category_tree($cat_id);
		
		$u_pagination = blog::append_sid('mcp', false, array(
			'm'		=> $mode,
			'i' 	=> $id,
			'sd'	=> $sd,
			'sk'	=> $sk,
		));
		
		$template->assign_vars(array(
			'L_EXPLAIN'			=> $user->lang['MODERATE_POSTS_EXPLAIN'],
			'L_TITLE'			=> $user->lang['MODERATE_POSTS'],
			'PAGINATION'		=> generate_pagination($u_pagination, $total_posts, $config['blog_posts_per_page'], $start),
			'PAGE_NUMBER'		=> on_page($total_posts, $config['blog_posts_per_page'], $start),

			'S_SELECT_SORT_DIR'		=> $this->make_select('sd', $sort_dirs, $sd),
			'S_SELECT_SORT_KEY'		=> $this->make_select('sk', $sort_by_text, $sk),

			'U_JUMPBOX_ACTION'	=> blog::append_sid('mcp', false, array(
				'm'		=> $mode,
				'i'		=> $id,
			))
		));
	}
	
	function make_select($key, $options, $select = false)
	{
		global $user;

		$string = '<select class="blog-inputbox" name="' . $key . '">';

		foreach($options as $value => $lang)
		{
			$selected	= (($value == $select) ? ' selected="selected"' : '');
			$title		= (isset($user->lang[$lang]) ? $user->lang[$lang] : $lang);
	
			$string .= '<option value="' . $value . '"' . $selected . '>' . $title . '</option>';
		}
		
		$string .= '</select>';
	
		return $string;
	}
}
?>