<?php
/*
*
* @name mcp_queue.php
* @package phpBB3 cBB Blog
* @version $Id: mcp_queue.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

class mcp_queue
{
	var $tpl_name;
	var $page_title;

	function main($mode, $id)
	{
		global $config, $db, $template, $user, $auth, $action, $phpEx;
		global $blog_hook;

		$start	= request_var('start', 0);
		
		switch($id)
		{
			case 'unapproved_posts':
				$this->tpl_name		= 'mcp_queue_posts';
				$this->page_title	= 'UNAPPROVED_POSTS';
				
				$post_id_list = request_var('post_id_list', array(0));
				
				if($action && empty($post_id_list))
				{
					$action = '';
				}

				switch($action)
				{
					case 'delete':
						if(!$auth->acl_get('m_blog_delete'))
						{
							trigger_error('NOT_AUTHORISED');
						}
						
						if(confirm_box(true))
						{
							include($blog_hook->root_path . 'includes/functions_posting.' . $phpEx);
							blog_delete_posts($post_id_list);
						}
						else
						{
							$s_hidden_fields = build_hidden_fields(array(
								'm'				=> $mode,
								'i'				=> $id,
								'post_id_list'	=> $post_id_list,
								'action'		=> 'delete',
							));
							
							confirm_box(false, 'DELETE_POSTS', $s_hidden_fields);
						}
					break;
					
					case 'approve':
						if(!$auth->acl_get('m_blog_approve'))
						{
							trigger_error('NOT_AUTHORISED');
						}
						
						include($blog_hook->root_path . 'includes/functions_posting.' . $phpEx);
						
						$posts_data = array();
						
						$sql = 'SELECT post_id, archive_id, post_approved
							FROM ' . BLOG_POSTS_TABLE . '
							WHERE ' . $db->sql_in_set('post_id', $post_id_list);
						$result = $db->sql_query($sql);
						while($row = $db->sql_fetchrow($result))
						{
							$post_id = (int)$row['post_id'];
							$posts_data[$post_id] = $row;
						}
						$db->sql_freeresult($result);
						
						blog_approve_posts($posts_data, true);
					break;
				}

				$post_rows = array();
				
				// count the post for pagination
				$sql = 'SELECT COUNT(*) AS total
					FROM ' . BLOG_POSTS_TABLE . '
					WHERE post_approved = 0';
				$result = $db->sql_query($sql);
				$total = (int)$db->sql_fetchfield('total');
				$db->sql_freeresult($result);

				$sql = 'SELECT p.*, u.username, u.user_colour
					FROM ' . BLOG_POSTS_TABLE . ' p
						LEFT JOIN ' . USERS_TABLE . ' u ON(u.user_id = p.poster_id)
					WHERE p.post_approved = 0
					ORDER BY p.post_time DESC';
				$result = $db->sql_query_limit($sql, $config['blog_posts_per_page'], $start);
				while($row = $db->sql_fetchrow($result))
				{
					$post_id = (int)$row['post_id'];
					
					$post_rows[$post_id] = $row;
					$post_rows[$post_id]['cats'] = array();
				}
				$db->sql_freeresult($result);
				
				// Get all the categories for the posts
				blog::append_post_categories($post_rows);
				
				foreach($post_rows as $post_id => $row)
				{
					if(!empty($blog_hook->seo))
					{
						$blog_hook->seo->prepare_url('post', $row, 'title');
					}

					$template->assign_block_vars('postrow', array(
						'ID'			=> $row['post_id'],
						
						'S_CATEGORIES'	=> blog::row_categories($row, 'mcp', array(
							'm'		=> 'posts',
							'i'		=> 'post_list',
						)),

						'S_TITLE'		=> $row['post_title'],
						'S_AUTHOR'		=> get_username_string('full', $row['poster_id'], $row['username'], $row['user_colour'], $row['post_username']),
						'S_TIME'		=> $user->format_date($row['post_time']),
					
						'U_VIEW_DETAILS'	=> blog::append_sid('view', false, array(
							'p'	=> $row['post_id'],
						)),
					));
				}
				
				$template->assign_vars(array(
					'L_EXPLAIN'			=> $user->lang['UNAPPROVED_POSTS_EXPLAIN'],
					'L_TITLE'			=> $user->lang['UNAPPROVED_POSTS'],
					'PAGINATION'		=> generate_pagination(blog::append_sid('mcp', false, array('m' => $mode, 'i' => $id)), $total, $config['blog_posts_per_page'], $start),
					'PAGE_NUMBER'		=> on_page($total, $config['blog_posts_per_page'], $start),
				));
			break;
			
			case 'spam_comments':
				$this->tpl_name		= 'mcp_queue_comments';
				$this->page_title	= 'SPAM_COMMENTS';
				
				$cmnt_id_list = request_var('cmnt_id_list', array(0));
				
				if($action && empty($cmnt_id_list))
				{
					$action = '';
				}

				switch($action)
				{
					case 'delete':
						if(!$auth->acl_get('m_blog_cmnt_delete'))
						{
							trigger_error('NOT_AUTHORISED');
						}
						
						if(confirm_box(true))
						{
							$sql = 'DELETE FROM ' . BLOG_COMMENTS_TABLE . '
								WHERE ' . $db->sql_in_set('comment_id', $cmnt_id_list);
							$db->sql_query($sql);
						}
						else
						{
							$s_hidden_fields = build_hidden_fields(array(
								'm'				=> $mode,
								'i'				=> $id,
								'cmnt_id_list'	=> $cmnt_id_list,
								'action'		=> 'delete',
							));
							
							confirm_box(false, 'DELETE_COMMENTS', $s_hidden_fields);
						}
					break;
					
					case 'nospam':
						$sql = 'UPDATE ' . BLOG_COMMENTS_TABLE . '
							SET spam_mark = 0
							WHERE ' . $db->sql_in_set('comment_id', $cmnt_id_list);
						$db->sql_query($sql);
					break;
				}

				// Count the total comments for pagination
				$sql = 'SELECT COUNT(*) AS total
					FROM ' . BLOG_COMMENTS_TABLE . '
					WHERE spam_mark = 1';
				$result = $db->sql_query($sql);
				$total = (int)$db->sql_fetchfield('total');
				$db->sql_freeresult($result);
				
				// Get the comments
				$sql = 'SELECT c.*, p.post_title, p.post_url, u.username AS member_name, u.user_colour
					FROM ' . BLOG_COMMENTS_TABLE . ' c
						LEFT JOIN ' . BLOG_POSTS_TABLE . ' p ON(p.post_id = c.post_id)
						LEFT JOIN ' . USERS_TABLE . ' u ON(u.user_id = c.user_id)
					WHERE c.spam_mark = 1
					ORDER BY c.comment_time DESC';
				$result = $db->sql_query_limit($sql, $config['blog_cmnt_per_page'], $start);
				while($row = $db->sql_fetchrow($result))
				{
					if($row['user_id'] != ANONYMOUS)
					{
						$row['username'] = $row['member_name'];
					}
					
					if(!empty($blog_hook->seo))
					{
						$blog_hook->seo->prepare_url('post', $row, 'title');
					}
					
					$template->assign_block_vars('commentrow', array(
						'ID'			=> $row['comment_id'],
						'S_TEXT'		=> $row['comment_text'],
						'S_AUTHOR'		=> get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']),
						'S_TIME'		=> $user->format_date($row['comment_time']),
						'S_POST_TITLE'	=> $row['post_title'],
						
						'U_POST_VIEW'	=> blog::append_sid('view', false, array('p' => $row['post_id'])),
					));
				}
				$db->sql_freeresult($result);
				
				$template->assign_vars(array(
					'L_EXPLAIN'			=> $user->lang['SPAM_COMMENTS_EXPLAIN'],
					'L_TITLE'			=> $user->lang['SPAM_COMMENTS'],
					'PAGINATION'		=> generate_pagination(blog::append_sid('mcp', false, array('m' => $mode, 'i' => $id)), $total, $config['blog_cmnt_per_page'], $start),
					'PAGE_NUMBER'		=> on_page($total, $config['blog_cmnt_per_page'], $start),
				));
			break;
		}
	}
}
?>