<?php
/*
*
* @name index.php
* @package cBB Blog
* @version $Id: index.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
define('IN_BLOG', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : '../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/functions_display.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup();

if(empty($blog_hook->enabled))
{
	redirect(append_sid($phpbb_root_path . "index.$phpEx"));
}

$cat_id 		= request_var('c', 0);
$archive		= request_var('a', '');
$start			= request_var('start', 0);

$post_rows = $sql_ary = $authed_cats = array();
$total_posts		= 0;
$bbcode_bitfield	= '';

$cat_rows = blog::get_category('ids');

// Get authed cats
if($cat_id)
{
	if(!isset($cat_rows[$cat_id]))
	{
		trigger_error('NO_CATEGORY');
	}

	if(blog::category_auth($cat_id))
	{
		$authed_cats[] = $cat_id;
	}
}
else
{
	foreach($cat_rows as $id => $row)
	{
		if(blog::category_auth($row))
		{
			$authed_cats[] = $id;
		}
	}
}

if(!sizeof($authed_cats))
{
	trigger_error('NOT_AUTHORISED');
}

// Main SQL
$sql_ary = array(
	'SELECT'	=> 'DISTINCT p.*, u.username, u.user_colour',
	'FROM'		=> array(
		BLOG_POSTS_TABLE		=> 'p',
		BLOG_POSTS_TRACK_TABLE	=> 'pt',
	),
	
	'LEFT_JOIN'	=> array(
		array(
			'FROM'	=> array(USERS_TABLE => 'u'),
			'ON'	=> 'u.user_id = p.poster_id'
		)
	),
	'WHERE'		=> 'pt.post_id = p.post_id
		AND ' . $db->sql_in_set('pt.cat_id', $authed_cats) . '
		AND p.post_time < ' . time(),
	'ORDER_BY'	=> 'p.post_time DESC',
);

if($archive)
{
	list($archive_month, $archive_year) = explode('-', $archive, 2);

	$sql = 'SELECT *
		FROM ' . BLOG_ARCHIVES_TABLE . '
		WHERE archive_month = ' . (int)$archive_month . '
		AND archive_year = ' .  (int)$archive_year;
	$result = $db->sql_query($sql);
	$archive_data = $db->sql_fetchrow($result);
	$db->sql_freeresult($result);
	
	if(!$archive_data)
	{
		// error
		trigger_error('NO_ARCHIVE');
	}
	
	// Get only the posts for the current archive in authed cats
	$sql_ary['WHERE'] .= ' AND p.archive_id = ' . $archive_data['archive_id'];
}
else if(!$cat_id)
{
	$sql_ary['ORDER_BY'] = 'p.post_fixed DESC, p.post_time DESC';
}

if(!$auth->acl_get('m_blog_approve'))
{
	$sql_ary['WHERE'] .= ' AND p.post_approved = 1';
}

// Count the posts
$count_sql_ary = array(
	'SELECT'	=> 'COUNT(DISTINCT p.post_id) AS total_posts',
	'FROM'		=> $sql_ary['FROM'],
	'WHERE'		=> $sql_ary['WHERE'],
);

$sql = $db->sql_build_query('SELECT', $count_sql_ary);
$result = $db->sql_query($sql);
$total_posts = (int)$db->sql_fetchfield('total_posts');
$db->sql_freeresult($result);

$post_rows = array();

// Get the posts
$sql = $db->sql_build_query('SELECT', $sql_ary);
$result = $db->sql_query_limit($sql, $config['blog_posts_per_page'], $start);
while($row = $db->sql_fetchrow($result))
{
	$post_id = (int)$row['post_id'];
	$post_rows[$post_id] = $row;
	
	if($row['post_type'] == POST_TYPE_BBCODE)
	{
		$bbcode_bitfield = $bbcode_bitfield | base64_decode($row['bbcode_bitfield']);
	}
}
$db->sql_freeresult($result);

// Get all the categories for the posts
blog::append_post_categories($post_rows);

// Instantiate BBCode if need be
if($bbcode_bitfield !== '')
{
	if(!class_exists('bbcode'))
	{
		include($phpbb_root_path . 'includes/bbcode.' . $phpEx);
	}

	$bbcode = new bbcode(base64_encode($bbcode_bitfield));
}

foreach($post_rows as $post_id => $row)
{
	if(!sizeof($row['cats']))
	{
		continue;
	}
	
	if(!empty($blog_hook->seo))
	{
		$blog_hook->seo->prepare_url('post', $row, 'title');
	}
	
	$row_action		= blog::append_sid('view', $row, false, true);
	$actions_ary	= array();

	if($row['allow_comments'])
	{
		$s_comments	= (($row['comment_count'] == 1) ? $user->lang['POST_COMMENT_ONE'] : (($row['comment_count']) ? $user->lang('POST_COMMENTS', $row['comment_count']) : $user->lang['POST_COMMENT_PUBLISH']));
		
		if(blog::auth('cmnt_post'))
		{
			$actions_ary[] = '<a href="' . $row_action . '#send-comment">' . $s_comments . '</a>';
		}
		else if($row['comment_count'])
		{
			$actions_ary[] = '<a href="' . $row_action . '#comments">' . $s_comments . '</a>';
		}
	}

	if(blog::auth('edit', $row))
	{
		$actions_ary[] = '<a href="' . blog::append_sid('posting', $row, array('mode' => 'edit', 'start' => $start), true) . '">' . $user->lang['EDIT'] . '</a>';
	}
	
	if(blog::auth('delete', $row))
	{
		$actions_ary[] = '<a class="js-blog" href="' . blog::append_sid('index', $row, array('c' => $cat_id, 'start' => $start)) . '#delete_entry">' . $user->lang['DELETE'] . '</a>';
	}
	
	switch($row['post_type'])
	{
		case POST_TYPE_BBCODE:
			$row['post_text'] = censor_text($row['post_text']);

			// Strip some bbcode tags
			$row['post_text'] = preg_replace('#\[(code|quote|flash)(\:[0-9a-z]{5,})\].*?\[\/\\1\\2\]#is', '', $row['post_text']);
			
			// Second parse bbcode here
			if($row['bbcode_bitfield'])
			{
				$bbcode->bbcode_second_pass($row['post_text'], $row['bbcode_uid'], $row['bbcode_bitfield']);
			}
			
			$row['post_text'] = blog::smiley_text($row['post_text']);
		break;
		
		case POST_TYPE_HTML:
			if(!isset($html_message_parser))
			{
				if(!class_exists('html_parse_message'))
				{
					include($blog_hook->root_path . 'includes/html_message_parser.' . $phpEx);
				}
				$html_message_parser = new html_parse_message();
			}

			$html_message_parser->message = $row['post_text'];
			$row['post_text'] = $html_message_parser->format_display(false, true, true);
		break;
	}
		
	$template->assign_block_vars('postrow', array(
		'S_TITLE'		=> $row['post_title'],
		'S_CONTENT'		=> blog::generate_resume($row),
		
		'S_HEADER'		=> $user->lang('POST_HEADER', get_username_string('full', $row['poster_id'], $row['username'], $row['user_colour'], $row['post_username']), $user->format_date($row['post_time'], $config['blog_date_format'])),
		'S_CATEGORIES'	=> $user->lang('POST_CATEGORIES', blog::row_categories($row)),
		'S_ACTIONS'		=> implode(' | ', $actions_ary),
		'UNAPPROVED_IMG'	=> (!$row['post_approved']) ? $user->img('icon_topic_unapproved', 'BLOG_POST_UNAPPROVED') : '',
		
		'U_ACTION'		=> $row_action,
	));
	
	unset($post_rows[$post_id]);
}

$u_pagination = blog::append_sid('index', false, array(
	'a'	=> $archive,
	'c' => $cat_id,
));

$template->assign_vars(array(
	'FULL_PAGINATION'	=> ($config['blog_pagination'] == 'full') ? true : false,
	'PAGINATION'		=> generate_pagination($u_pagination, $total_posts, $config['blog_posts_per_page'], $start),
	'PAGE_NUMBER'		=> on_page($total_posts, $config['blog_posts_per_page'], $start),
	
	'SHOW_ARCHIVE_INFO'		=> ($cat_id || $archive) ? true : false,

	'U_BLOG_MCP'	=> ($auth->acl_gets(array('m_blog_delete', 'm_blog_approve', 'm_blog_cmnt_delete')) ? blog::append_sid('mcp', false, (($cat_id) ? array('m' => 'posts', 'i' => 'post_list', 'c' => $cat_id) : false)) : ''),
	'U_BLOG_POST'	=> (blog::auth('post') ? blog::append_sid('posting', false, array('mode' => 'post'), $cat_id) : ''),
));

// Build the navigate links
if(isset($archive_data))
{
	$month_name		= $blog_hook->month_names[$archive_data['archive_month']];
	$computed_name	= $user->lang['datetime'][$month_name] . ' ' . $archive_data['archive_year'];
	
	$template->assign_var('S_ARCHIVE_TITLE', $user->lang('ARCHIVE_MONTH', $computed_name));

	blog::build_navbar(array(
		'archive_id' 	=> $archive_data['archive_month'] . '-' . $archive_data['archive_year'],
		'archive_data'	=> $computed_name,
	));
}
else if($cat_id)
{
	$template->assign_vars(array(
		'S_ARCHIVE_TITLE'		=> $user->lang('ARCHIVE_CAT', blog::get_category($cat_id, 'cat_name')),
		'S_ARCHIVE_DESC'		=> blog::get_category($cat_id, 'cat_description'),
	));

	blog::build_navbar(array(
		'cat_id' 	=> $cat_id,
	));
}
else
{
	if($config['blog_index_stats'])
	{
		$sql = 'SELECT COUNT(*) AS comment_count
			FROM ' . BLOG_COMMENTS_TABLE . '
			WHERE spam_mark = 0';
		$result = $db->sql_query($sql);
		$total_comments = (int)$db->sql_fetchfield('comment_count');
		$db->sql_freeresult($result);

		$template->assign_vars(array(
			'SHOW_STATISTICS'		=> true,
			'BLOG_TOTAL_CATEGORIES'	=> $user->lang('BLOG_TOTAL_CATEGORIES', sizeof($cat_rows)),
			'BLOG_TOTAL_COMMENTS'	=> $user->lang('BLOG_TOTAL_COMMENTS', $total_comments),
			'BLOG_TOTAL_POSTS'		=> $user->lang('BLOG_TOTAL_POSTS', $config['blog_num_posts']),
		));
	}

	blog::build_navbar(false);
}

// Output page
page_header($user->lang['BLOG']);

blog::get_widgets();

$template->set_filenames(array(
	'body' => 'blog/index_body.html'
));

page_footer(false);

?>
