<?php
/*
*
* @name schema_install.php
* @package cBB Blog Installer
* @version $Id: schema_install.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

if(!defined('IN_PHPBB') || !defined('IN_INSTALL'))
{
	exit;
}

$now = getdate(time());

$this->schema = array(
	/* Table definitions */
	'table_add' => array(
		BLOG_ARCHIVES_TABLE => array(
			'COLUMNS' => array(
				'archive_id'		=> array('UINT:3', NULL, 'auto_increment'),
				'archive_month'		=> array('UINT:2', 0),
				'archive_year'		=> array('VCHAR:5', ''),
				'post_count'		=> array('UINT:8', 0),
				'post_count_real'	=> array('UINT:8', 0),
			),
			'PRIMARY_KEY'	=> 'archive_id',
		),
		
		BLOG_ATTACHMENTS_TABLE => array(
			'COLUMNS' => array(
				'attach_id'			=> array('UINT:8', NULL, 'auto_increment'),
				'post_id'			=> array('UINT:8', 0),
				'poster_id'			=> array('UINT:8', 0),
				'physical_filename'	=> array('VCHAR', ''),
				'real_filename'		=> array('VCHAR', ''),
				'extension'			=> array('VCHAR:100', ''),
				'mimetype'			=> array('VCHAR:100', ''),
				'filesize'			=> array('UINT:20', 0),
				'filetime'			=> array('UINT:11', 0),
				'is_orphan'			=> array('BOOL', 0),
			),
			'PRIMARY_KEY'	=> 'attach_id',
		),

		BLOG_CATS_TABLE => array(
			'COLUMNS' => array(
				'cat_id'			=> array('UINT:3', NULL, 'auto_increment'),
				'parent_id'			=> array('UINT:3', 0),
				'cat_name'			=> array('VCHAR', ''),
				'cat_url'			=> array('VCHAR', ''),
				'cat_description'	=> array('TEXT', ''),
				'cat_enabled'		=> array('BOOL', 1),
				'cat_groups'		=> array('VCHAR:40', ''),
			),
			'PRIMARY_KEY'	=> 'cat_id',
		),
		
		BLOG_COMMENTS_TABLE => array(
			'COLUMNS' => array(
				'comment_id'		=> array('UINT:8', NULL, 'auto_increment'),
				'post_id'			=> array('UINT:8', 0),
				'user_id'			=> array('UINT:8', 0),
				'user_ip'			=> array('VCHAR:40', ''),
				'username'			=> array('VCHAR', ''),
				'user_email'		=> array('VCHAR:100', ''),
				'comment_text'		=> array('TEXT', ''),
				'comment_time'		=> array('UINT:11', 0),
				'spam_mark'			=> array('BOOL', 0),
			),
			'PRIMARY_KEY'	=> 'comment_id',
		),

		BLOG_POSTS_TABLE => array(
			'COLUMNS' => array(
				'post_id'			=> array('UINT:8', NULL, 'auto_increment'),
				'archive_id'		=> array('UINT:3', 0),
				'poster_id'			=> array('UINT:8', 0),
				'post_username'		=> array('VCHAR', ''),
				'post_approved'		=> array('BOOL', 1),
				'post_fixed'		=> array('BOOL', 0),
				'post_title'		=> array('VCHAR', ''),
				'post_url'			=> array('VCHAR', ''),
				'post_text'			=> array('TEXT', ''),
				'post_time'			=> array('UINT:11', 0),
				'post_type'			=> array('UINT:1', POST_TYPE_BBCODE),
				'allow_comments'	=> array('BOOL', 1),
				'comment_count'		=> array('UINT:8', 0),
				'view_count'		=> array('UINT:8', 0),
				'bbcode_uid'		=> array('VCHAR:8', ''),
				'bbcode_bitfield'	=> array('VCHAR', ''),
			),
			'PRIMARY_KEY'	=> 'post_id',
		),

		BLOG_POSTS_TRACK_TABLE => array(
			'COLUMNS' => array(
				'post_id'			=> array('UINT:8', 0),
				'cat_id'			=> array('UINT:3', 0),
			),
			'KEYS'			=> array(
				'post_id'		=> array('INDEX', 'post_id'),
			),
		),
		
		BLOG_SOURCES_TABLE	=> array(
			'COLUMNS'	=> array(
				'source_id'			=> array('UINT:4', NULL, 'auto_increment'),
				'source_cats'		=> array('VCHAR', ''),
				'source_title'		=> array('VCHAR', ''),
				'source_url'		=> array('VCHAR', ''),
				'source_gc'			=> array('UINT:11', 0),
				'source_last_gc'	=> array('UINT:11', 0),
				'source_poster'		=> array('UINT:8', 0),
			),
			'PRIMARY_KEY'	=> 'source_id',
		),
		
		BLOG_TAGS_TABLE	=> array(
			'COLUMNS'	=> array(
				'tag_id'			=> array('UINT:8', NULL, 'auto_increment'),
				'post_id'			=> array('UINT:8', 0),
				'tag_name'			=> array('VCHAR', ''),
			),
			'PRIMARY_KEY'	=> 'tag_id',
		),
		
		BLOG_WIDGETS_TABLE	=> array(
			'COLUMNS'	=> array(
				'widget_id'				=> array('UINT:4', NULL, 'auto_increment'),
				'widget_title'			=> array('VCHAR', ''),
				'widget_basename'		=> array('VCHAR:100', ''),
				'widget_options'		=> array('TEXT', ''),
				'widget_order'			=> array('UINT:2', 1),
				'widget_style'			=> array('UINT:2', WIDGET_STYLE_FULL),
			),
			'PRIMARY_KEY'	=> 'widget_id',
		),
	),
	
	'table_column_add' => array(
		array(USERS_TABLE, 'user_blog_posts',	array('UINT:8', 0)),
	),
	
	// Lets add a config settings
	'config_add' => array(
		array('blog_allow_attachments', true),
		array('blog_allow_comments', true),
		array('blog_attach_extensions', 'gif|jpg|png|zip'),
		array('blog_attach_gallery', true),
		array('blog_cmnt_maxlength', 600),
		array('blog_cmnt_order', 'DESC'),
		array('blog_cmnt_per_page', 10),
		array('blog_cron_lock', 0, true),
		array('blog_date_format', 'j F, Y'),
		array('blog_domain', ''),
		array('blog_enabled', true),
		array('blog_index_stats', true),
		array('blog_max_attachments', 3),
		array('blog_max_filesize', 1048576),
		array('blog_num_posts', 0),
		array('blog_pagination', 'full'),
		array('blog_post_type', POST_TYPE_BBCODE),
		array('blog_posts_per_page', 25),
		array('blog_related_posts', 0),
		array('blog_resume_chars', 200),
		array('blog_seo_enabled', false),
		array('blog_share_buttons', true),
		array('blog_version', $this->new_version),
	),

	/* SQL definitions */
	'row_insert' => array(
		BLOG_ARCHIVES_TABLE => array(
			array(
				'archive_month'		=> $now['mon'],
				'archive_year'		=> $now['year'],
				'post_count'		=> 1,
				'post_count_real'	=> 1,
			)
		),
		
		BLOG_CATS_TABLE => array(
			array(
				'parent_id'			=> 0,
				'cat_name'			=> $user->lang['EXAMPLE_CATEGORY'],
				'cat_url'			=> $user->lang['EXAMPLE_CATEGORY_SEO'],
				'cat_description'	=> '',
				'cat_enabled'		=> 1,
				'cat_groups'		=> 'all',
			)
		),
		
		BLOG_POSTS_TABLE => array(
			array(
				'archive_id'		=> 1,
				'poster_id'			=> $user->data['user_id'],
				'post_approved'		=> 1,
				'post_fixed'		=> 0,
				'post_title'		=> $user->lang['EXAMPLE_POST_TITLE'],
				'post_url'			=> $user->lang['EXAMPLE_POST_TITLE_SEO'],
				'post_text'			=> $user->lang['EXAMPLE_POST_TEXT'],
				'post_time'			=> $now[0],
				'post_type'			=> POST_TYPE_BBCODE,
				'allow_comments'	=> 1,
				'comment_count'		=> 0,
				'view_count'		=> 0,
				'bbcode_uid'		=> 'o9dnbln0',
				'bbcode_bitfield'	=> 'cA=='
			)
		),
		
		BLOG_POSTS_TRACK_TABLE => array(
			array(
				'post_id'	=> 1,
				'cat_id'	=> 1
			)
		),
		
		BLOG_WIDGETS_TABLE => array(
			array(
				'widget_title'		=> $user->lang['SEARCH'],
				'widget_basename'	=> 'search',
				'widget_options'	=> '',
				'widget_order'		=> 1,
				'widget_style'		=> WIDGET_STYLE_FULL,
			),
			array(
				'widget_title'		=> $user->lang['RECENT_ENTRIES'],
				'widget_basename'	=> 'recent_entries',
				'widget_options'	=> serialize(array('post_num' => 5)),
				'widget_order'		=> 2,
				'widget_style'		=> WIDGET_STYLE_FULL,
			),
			array(
				'widget_title'		=> $user->lang['CATEGORIES'],
				'widget_basename'	=> 'category',
				'widget_options'	=> '',
				'widget_order'		=> 3,
				'widget_style'		=> WIDGET_STYLE_FULL,
			),
			array(
				'widget_title'		=> $user->lang['ARCHIVES'],
				'widget_basename'	=> 'archive',
				'widget_options'	=> serialize(array('show_post_count' => 1)),
				'widget_order'		=> 4,
				'widget_style'		=> WIDGET_STYLE_FULL,
			),
			array(
				'widget_title'		=> $user->lang['RSS'],
				'widget_basename'	=> 'rss',
				'widget_options'	=> '',
				'widget_order'		=> 5,
				'widget_style'		=> WIDGET_STYLE_FULL,
			),
		),
	),

	//Define modules to install
	'module_check' => 0, // $this->schema['modules'] array ID to check or false to disable

	'modules' => array(
		array('acp', 'ACP_CAT_DOT_MODS', 'ACP_CAT_BLOG'),
		array('acp', 'ACP_CAT_BLOG', array(
			'module_basename'	=> 'blog',
			'modes'				=> array('main', 'category', 'attach', 'widget', 'tools'),
		)),
	),

	// Define permissions to add and set
	'permission_add' => array(
		array('a_blog_manage', true),
		
		array('m_blog_approve', true),
		array('m_blog_cmnt_delete', true),
		array('m_blog_delete', true),
		array('m_blog_edit', true),
		array('m_blog_forum', true),
		
		array('u_blog_delete', true),
		array('u_blog_edit', true),
		array('u_blog_forum', true),
		array('u_blog_noapprove', true),
		array('u_blog_post', true),
		array('u_blog_cmnt_delete', true),
		array('u_blog_cmnt_post', true),
	),

	'permission_set' => array(
		array('ROLE_ADMIN_STANDARD', 'a_blog_manage'),

		array('ROLE_USER_FULL', 'u_blog_delete'),
		array('ROLE_USER_FULL', 'u_blog_edit'),
		array('ROLE_USER_FULL', 'u_blog_forum'),
		array('ROLE_USER_FULL', 'u_blog_noapprove'),
		array('ROLE_USER_FULL', 'u_blog_post'),
		array('ROLE_USER_FULL', 'u_blog_cmnt_post'),
		
		array('ROLE_MOD_FULL', 'm_blog_approve'),
		array('ROLE_MOD_FULL', 'm_blog_delete'),
		array('ROLE_MOD_FULL', 'm_blog_edit'),
		array('ROLE_MOD_FULL', 'm_blog_cmnt_delete'),
		array('ROLE_MOD_FULL', 'm_blog_forum'),
	)
);

function sync_blog_stats()
{
	global $phpbb_root_path, $phpEx;
	
	$filename = $phpbb_root_path . "includes/acp/acp_blog.$phpEx";
	
	if(!class_exists('acp_blog') && file_exists($filename))
	{
		include($filename);
	}

	if(!class_exists('acp_blog'))
	{
		return false;
	}
	
	return acp_blog::sync_stats();
}
?>
