<?php
/*
*
* @name posting.php
* @package cBB Blog
* @version $Id: posting.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
define('IN_BLOG', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : '../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/functions_display.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup(array('posting', 'mods/blog/posting'));

if(empty($blog_hook->enabled))
{
	redirect(append_sid($phpbb_root_path . "index.$phpEx"));
}

$post_id	= request_var('p', 0);
$cat_id		= request_var('c', 0);
$mode		= request_var('mode', '');
$start		= request_var('start', 0);

$submit		= (isset($_POST['submit'])) ? true : false;
$preview	= (isset($_POST['preview'])) ? true : false;

$error = array();

if(in_array($mode, array('approve', 'edit')))
{
	// Get post info
	$sql = 'SELECT p.*, u.username
		FROM ' . BLOG_POSTS_TABLE . ' p
			LEFT JOIN ' . USERS_TABLE . " u ON(u.user_id = p.poster_id)
		WHERE p.post_id = $post_id";
	$result = $db->sql_query($sql);
	$post_data = $db->sql_fetchrow($result);
	$db->sql_freeresult($result);
	
	if(!$post_data)
	{
		trigger_error('NO_ENTRY');
	}
	
	$template->assign_vars(array(
		'L_TITLE'		=> $post_data['post_title'],
		'U_TITLE'		=> blog::append_sid('view', $post_data),
	));
}

if($mode == 'approve' || $submit)
{
	include($blog_hook->root_path . 'includes/functions_posting.' . $phpEx);
}

switch($mode)
{
	case 'approve':
		if(!blog::auth('approve', $post_data))
		{
			trigger_error('NO_AUTH_OPERATION');
		}

		blog_approve_posts($post_data);
		
		if(!empty($blog_hook->seo))
		{
			$blog_hook->seo->prepare_url('post', $post_data, 'title');
		}

		$u_redirect = blog::append_sid('view', $post_data, false, $cat_id);
		
		$message = $user->lang['POST_APPROVED'];
		$message .= '<br /><br />' . $user->lang('RETURN_PAGE', '<a href="' . $u_redirect  . '">', '</a>');
	
		meta_refresh(3, $u_redirect);
		trigger_error($message);
	break;

	case 'edit':
		if(!blog::auth('edit', $post_data))
		{
			trigger_error('NO_AUTH_OPERATION');
		}
		
		// Get post categories
		$post_data['cats'] = array();

		$sql = 'SELECT cat_id
			FROM ' . BLOG_POSTS_TRACK_TABLE . "
			WHERE post_id = $post_id";
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			$post_data['cats'][] = $row['cat_id'];
		}
		$db->sql_freeresult($result);
		
		// Get post tags
		$post_data['tags'] = array();

		$sql = 'SELECT tag_name
			FROM ' . BLOG_TAGS_TABLE . "
			WHERE post_id = $post_id";
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			$post_data['tags'][] = $row['tag_name'];
		}
		$db->sql_freeresult($result);
		
	// no break
		
	case 'post':
		if($mode == 'post')
		{
			// Always deny from guests and bots
			if(!blog::auth('post') || $user->data['user_type'] == USER_IGNORE)
			{
				trigger_error('NO_AUTH_OPERATION');
			}
			
			// Default post data
			$post_data = array(
				'post_id'			=> 0,
				'archive_id'		=> 0,
				'poster_id'			=> $user->data['user_id'],
				'post_username'		=> '',
				'post_approved'		=> (blog::auth('noapprove') ? 1 : 0),
				'post_fixed'		=> 0,
				'post_title'		=> '',
				'post_text'			=> '',
				'post_time'			=> time(),
				'post_type'			=> $config['blog_post_type'],
				'allow_comments'	=> $config['blog_allow_comments'],
				'comment_count'		=> 0,
				
				'bbcode_uid'		=> '',
				'bbcode_bitfield'	=> '',
				
				'username'		=> $user->data['username'],
				'cats'			=> ($cat_id) ? array($cat_id) : array(),
				'tags'			=> array(),
			);
		}

		$bbcode_status	= ($config['allow_bbcode']) ? true : false;
		$smilies_status	= ($config['allow_smilies']) ? true : false;
		$img_status		= ($bbcode_status) ? true : false;
		$url_status		= ($config['allow_post_links']) ? true : false;
		$flash_status	= ($bbcode_status && $config['allow_post_flash']) ? true : false;
		$quote_status	= true;
		
		switch($post_data['post_type'])
		{
			case POST_TYPE_BBCODE:
				include($phpbb_root_path . 'includes/message_parser.' . $phpEx);
				$message_parser = new parse_message();
				
				if($post_data['bbcode_uid'])
				{
					$message_parser->bbcode_uid = $post_data['bbcode_uid'];
				}
			
				$parse_options = array($bbcode_status, $url_status, $smilies_status, $img_status, $flash_status, $quote_status, $url_status);
			break;
			
			case POST_TYPE_HTML:
				include($blog_hook->root_path . 'includes/html_message_parser.' . $phpEx);
				$message_parser = new html_parse_message();

				$parse_options	= array($url_status, $smilies_status, $img_status, $flash_status, $quote_status, $url_status);
			break;
			
			default:
				trigger_error('NO_ENTRY');
			break;
		}
		
		if (!empty($post_data['post_text']))
		{
			$message_parser->message = &$post_data['post_text'];
			unset($post_data['post_text']);
		}

		if($submit || $preview)
		{
			$username 	= ($auth->acl_get('a_blog_manage') ? trim(utf8_normalize_nfc(request_var('username', '', true))) : $user->data['username']);
			$post_tags 	= utf8_normalize_nfc(request_var('post_tags', '', true));

			$post_data['post_title']		= utf8_normalize_nfc(request_var('post_title', '', true));
			$message_parser->message		= utf8_normalize_nfc(request_var('post_text', '', true));
			$post_data['post_fixed']		= request_var('sticky', false);
			$post_data['post_time']			= request_var('post_time', time());
			$post_data['post_type']			= request_var('post_type', $config['blog_post_type']);
			$post_data['allow_comments']	= request_var('allow_comments', false);
			$post_data['cats']				= request_var('category', array(0));
			$post_data['tags']				= ($post_tags) ? explode(',', $post_tags) : array();

			// Find new poster id
			$old_username = ($post_data['poster_id'] == ANONYMOUS) ? $post_data['post_username'] : $post_data['username'];
			
			if($username != $old_username)
			{
				$sql = 'SELECT user_id
					FROM ' . USERS_TABLE . "
					WHERE username = '" . $db->sql_escape($username) . "'
					OR username_clean = '" . $db->sql_escape($username) . "'";
				$result = $db->sql_query($sql);
				$user_id = $db->sql_fetchfield('user_id');
				$db->sql_freeresult($result);
				
				if(!$user_id || $user_id == ANONYMOUS)
				{
					$error[] = $user->lang['NO_USER'];
				}
				
				$post_data['username']	= $username;
				$post_data['old_poster_id'] = $post_data['poster_id'];
				$post_data['poster_id']	= $user_id;
			}
			
			if(!$post_data['post_title'] || $post_data['post_title'] == $user->lang['POST_TITLE'] || !$message_parser->message)
			{
				$error[] = $user->lang['INCOMPLETE_FIELDS'];
			}
			
			if(!sizeof($post_data['cats']))
			{
				$error[] = $user->lang['NO_CATEGORIES'];
			}
			
			call_user_func_array(array($message_parser, 'parse'), $parse_options);
			
			if($submit && !sizeof($error))
			{
				// Insert or update the post
				$sql_ary = array(
					'post_id'			=> $post_data['post_id'],
					'archive_id'		=> $post_data['archive_id'],
					'poster_id'			=> $post_data['poster_id'],
					'post_username'		=> $post_data['post_username'],
					'post_approved'		=> $post_data['post_approved'],
					'post_fixed'		=> $post_data['post_fixed'],
					'post_title'		=> $post_data['post_title'],
					'post_text'			=> $message_parser->message,
					'post_time'			=> $post_data['post_time'],
					'post_type'			=> $post_data['post_type'],
					'allow_comments'	=> $post_data['allow_comments'],
					'comment_count'		=> ($post_data['allow_comments']) ? $post_data['comment_count'] : 0,
					
					'bbcode_uid'		=> ($post_data['post_type'] == POST_TYPE_BBCODE) ? $message_parser->bbcode_uid : '',
					'bbcode_bitfield'	=> ($post_data['post_type'] == POST_TYPE_BBCODE) ? $message_parser->bbcode_bitfield : '',
				
					'old_poster_id'		=> (isset($post_data['old_poster_id']) ? $post_data['old_poster_id'] : 0),
				);
				
				blog_submit_post($mode, $sql_ary, $post_data['cats'], $post_data['tags']);
				
				if(!empty($blog_hook->seo))
				{
					$blog_hook->seo->prepare_url('post', $sql_ary, 'title');
				}
				
				$redirect_url = blog::append_sid('view', false, array('p' => $sql_ary['post_id']), true);

				meta_refresh(3, $redirect_url);

				$message = array();
				$message[] = ($mode == 'edit') ? $user->lang['POST_EDITED'] : $user->lang['POST_STORED'];
				$message[] = $user->lang('VIEW_ENTRY', '<a href="' . $redirect_url . '">', '</a>');
				$message[] = $user->lang('RETURN_MAIN_PAGE', blog::append_sid('index'));
				
				trigger_error(implode('<br /><br />', $message));
			}
		}
		
		// Preview
		if($preview && !sizeof($error))
		{
			$preview_message = $message_parser->format_display($bbcode_status, $url_status, $smilies_status, false);

			$preview_title = censor_text($post_data['post_title']);
			
			blog::parse_attachments('preview', $post_id, $preview_message);

			$template->assign_vars(array(
				'PREVIEW_TITLE'		=> $preview_title,
				'PREVIEW_MESSAGE'	=> $preview_message,
				'PREVIEW_TAGS'		=> implode(', ', $post_data['tags']),

				'S_DISPLAY_PREVIEW'		=> true,
			));
		}
		
		blog::build_category_tree($post_data['cats']);
		
		/* Build more used tags */
		$sql = 'SELECT COUNT(*) AS tag_count, tag_name
			FROM ' . BLOG_TAGS_TABLE . '
			GROUP BY tag_name
			ORDER BY tag_count';
		$result = $db->sql_query_limit($sql, 10);
		while($row = $db->sql_fetchrow($result))
		{
			$template->assign_block_vars('used_tags', array(
				'TAG'	=> $row['tag_name'],
			));
		}
		$db->sql_freeresult($result);
		
		if(!$submit && !$preview)
		{
			blog::delete_orphan_attachments();
		}

		$attach_count = blog::parse_attachments('edit', $post_id, $message_parser->message);
		
		$message_parser->decode_message((($row['post_type'] == POST_TYPE_BBCODE) ? $post_data['bbcode_uid'] : false));
		
		$template->assign_vars(array(
			'S_ERROR'				=> (sizeof($error) ? implode('<br />', $error) : ''),

			'POST_ID'				=> $post_id,
			'S_POST_ALLOW_COMMENTS'	=> (int)$post_data['allow_comments'],
			'S_POST_AUTHOR'			=> ($post_data['poster_id'] == ANONYMOUS) ? $post_data['post_username'] : $post_data['username'],
			'S_POST_STICKY'			=> (int)$post_data['post_fixed'],
			'S_POST_TAGS'			=> (sizeof($post_data['tags']) ? implode(',', $post_data['tags']) : ''),
			'S_POST_TEXT'			=> $message_parser->message,
			'S_POST_TITLE'			=> $post_data['post_title'],
			'S_POST_TIME'			=> $post_data['post_time'],
			
			'S_HTML_BUTTONS'		=> ($post_data['post_type'] == POST_TYPE_HTML) ? true : false,
			'S_BBCODE_BUTTONS'		=> ($post_data['post_type'] == POST_TYPE_BBCODE && $bbcode_status) ? true : false,

			'S_ATTACH_GALLERY'			=> ($config['blog_attach_gallery']) ? true : false,
			'S_CAN_SELECT_AUTHOR'		=> ($auth->acl_get('a_blog_manage') ? true : false),
			'S_CAN_USE_ATTACHMENTS'		=> ($config['blog_allow_attachments']) ? true : false,
			'S_CAN_SUBMIT_ATTACHMENT'	=> ($config['blog_max_attachments'] && $attach_count >= $config['blog_max_attachments']) ? false : true,
			
			'S_BBCODE_FLASH'		=> $flash_status,
			'S_BBCODE_IMG'			=> $img_status,
			'S_BBCODE_QUOTE'		=> $quote_status,
			'S_LINKS_ALLOWED'		=> $url_status,
			'S_SMILIES_ALLOWED'		=> $smilies_status,

			'S_HIDDEN_FIELDS'			=> build_hidden_fields(array(
				'post_type'		=> $post_data['post_type'],
			)),
			
			'U_ACTION'				=> blog::append_sid('posting', false, array(
				'mode'	=> $mode,
				'c'		=> $cat_id,
				'p'		=> $post_id,
			)),

			'U_FIND_USERNAME'		=> blog::append_sid('search', false, array(
				'mode'	=> 'user',
				'form'	=> 'blog_form',
				'field'	=> 'username',
			))
		));
		
		// Build custom bbcodes array
		display_custom_bbcodes();
		
		generate_smilies();

		blog::build_navbar(array(
			'cat_id' 	=> $cat_id,
		));
	break;
}

switch($mode)
{
	case 'post':
		$page_title = $user->lang['POST_ADD'];
	break;
	
	case 'edit':
		$page_title = $user->lang['POST_EDIT'];
	break;
}

$template->assign_vars(array(
	'L_POST_A'	=> $page_title,
));

// Output page
page_header($page_title);

$template->set_filenames(array(
	'body' => 'blog/posting_body.html'
));

page_footer(false);

function generate_smilies()
{
	global $db, $template, $config, $phpbb_root_path;

	$sql = 'SELECT *
		FROM ' . SMILIES_TABLE . '
		ORDER BY smiley_order';
	$result = $db->sql_query($sql, 3600);
	
	$smilies = array();
	while ($row = $db->sql_fetchrow($result))
	{
		if (empty($smilies[$row['smiley_url']]))
		{
			$smilies[$row['smiley_url']] = $row;
		}
	}
	$db->sql_freeresult($result);

	if (sizeof($smilies))
	{
		$root_path = (defined('PHPBB_USE_BOARD_URL_PATH') && PHPBB_USE_BOARD_URL_PATH) ? generate_board_url() . '/' : $phpbb_root_path;

		foreach ($smilies as $row)
		{
			$template->assign_block_vars('smiley', array(
				'SMILEY_CODE'	=> $row['code'],
				'A_SMILEY_CODE'	=> addslashes($row['code']),
				'SMILEY_IMG'	=> $root_path . $config['smilies_path'] . '/' . $row['smiley_url'],
				'SMILEY_WIDTH'	=> $row['smiley_width'],
				'SMILEY_HEIGHT'	=> $row['smiley_height'],
				'SMILEY_DESC'	=> $row['emotion'])
			);
		}
	}
}

?>
