<?php
/*
*
* @name rss.php
* @package cBB Blog
* @version $Id: rss.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
define('IN_BLOG', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : '../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup();

if(empty($blog_hook->enabled))
{
	trigger_error('NO_FEED_ENABLED');
}

$mode		= request_var('mode', '');
$cat_id		= request_var('c', 0);

$date_format 		= 'D, d M Y H:i:s \G\M\T';
$feed_updated_time	= $config['board_startdate'];
$feed_link			= blog::append_sid('rss', false, array('mode' => $mode));

$rdf = $rows = array();

switch($mode)
{
	case 'comments':
		$sql = 'SELECT c.*, p.post_title, p.post_url, u.username AS member_name
			FROM ' . BLOG_COMMENTS_TABLE . ' c
				LEFT JOIN ' . BLOG_POSTS_TABLE . ' p ON(p.post_id = c.post_id)
				LEFT JOIN ' . USERS_TABLE . ' u ON(u.user_id = c.user_id)
			WHERE spam_mark = 0
			ORDER BY comment_time DESC';
		$result = $db->sql_query_limit($sql, 15);
		while($row = $db->sql_fetchrow($result))
		{
			if(!empty($blog_hook->seo))
			{
				$blog_hook->seo->prepare_url('post', $row, 'title');
			}

			$link = blog::append_sid('view', false, array('p' => $row['post_id'])) . '#comment-' . $row['comment_id'];
			$link = blog::strip_vars($link, true);
			
			if($row['user_id'] != ANONYMOUS)
			{
				$row['username'] = $row['member_name'];
			}
			
			$rows[] = "\t<item>";
			$rows[] = "\t\t<dc:creator>" . $row['username'] . '</dc:creator>';
			$rows[] = "\t\t<pubDate>" . gmdate($date_format, $row['comment_time']) . '</pubDate>';
			$rows[] = "\t\t<guid isPermaLink=\"false\">$link</guid>";
			$rows[] = "\t\t<link>$link</link>";
			$rows[] = "\t\t<title>" . $user->lang('RSS_COMMENT_TITLE', $row['post_title'], $row['username']) . '</title>';
			$rows[] = "\t\t<description>" . censor_text($row['comment_text']) . '</description>';
			$rows[] = "\t</item>";
			
			$feed_updated_time = max($feed_updated_time, $row['comment_time']);
		}
		$db->sql_freeresult($result);
	break;

	case 'posts':
		$cat_rows			= blog::get_category('ids');
		$post_rows 			= array();
		$bbcode_bitfield	= '';

		// Get authed cats
		if($cat_id)
		{
			if(blog::category_auth($cat_id))
			{
				$authed_cats[] = $cat_id;
			}
		}
		else
		{
			foreach($cat_rows as $id => $row)
			{
				if(blog::category_auth($row))
				{
					$authed_cats[] = $id;
				}
			}
		}
		
		if(!sizeof($authed_cats))
		{
			break;
		}

		$sql_ary = array(
			'SELECT'	=> 'DISTINCT p.*, u.username',
			'FROM'		=> array(
				BLOG_POSTS_TABLE 		=> 'p',
				BLOG_POSTS_TRACK_TABLE	=> 'pt',
			),
			
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'u.user_id = p.poster_id'
				)
			),
			'WHERE'		=> 'pt.post_id = p.post_id
				AND ' . $db->sql_in_set('pt.cat_id', $authed_cats) . '
				AND p.post_time < ' . time() . '
				AND p.post_approved = 1',
			'ORDER_BY'	=> 'p.post_time DESC'
		);

		// Get the posts
		$sql = $db->sql_build_query('SELECT', $sql_ary);
		$result = $db->sql_query_limit($sql, 15);
		while($row = $db->sql_fetchrow($result))
		{
			$post_id = (int)$row['post_id'];
			$post_rows[$post_id] = $row;
			
			if($row['post_type'] == POST_TYPE_BBCODE)
			{
				$bbcode_bitfield = $bbcode_bitfield | base64_decode($row['bbcode_bitfield']);
			}
		}
		$db->sql_freeresult($result);
		
		// Get all the categories for the posts
		blog::append_post_categories($post_rows);
		
		// Instantiate BBCode if need be
		if ($bbcode_bitfield !== '')
		{
			if(!class_exists('bbcode'))
			{
				include($phpbb_root_path . 'includes/bbcode.' . $phpEx);
			}

			$bbcode = new bbcode(base64_encode($bbcode_bitfield));
		}
		
		// Get all attachments in one query
		$attach_rows = array();

		$sql = 'SELECT *
			FROM ' . BLOG_ATTACHMENTS_TABLE . '
			WHERE ' . $db->sql_in_set('post_id', array_keys($post_rows));
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			$attach_id = (int)$row['attach_id'];
			$attach_rows[$attach_id] = $row;
		}
		$db->sql_freeresult($result);

		foreach($post_rows as $post_id => $row)
		{
			if(!sizeof($row['cats']))
			{
				continue;
			}
			
			if(!empty($blog_hook->seo))
			{
				$blog_hook->seo->prepare_url('post', $row, 'title');
			}
			
			$link = blog::append_sid('view', $row);
			$link = blog::strip_vars($link, true);
			
			switch($row['post_type'])
			{
				case POST_TYPE_BBCODE:
					$message = censor_text($row['post_text']);
					
					// Second parse bbcode here
					if($row['bbcode_bitfield'])
					{
						$bbcode->bbcode_second_pass($message, $row['bbcode_uid'], $row['bbcode_bitfield']);
					}
					
					$message = blog::smiley_text($message);
					blog::clear_resume_tags($message);
				break;
				
				case POST_TYPE_HTML:
					if(!isset($html_message_parser))
					{
						if(!class_exists('html_parse_message'))
						{
							include($blog_hook->root_path . 'includes/html_message_parser.' . $phpEx);
						}
						$html_message_parser = new html_parse_message();
					}

					$html_message_parser->message = $row['post_text'];
					$message = $html_message_parser->format_display(false, true, true);
				break;
			}
			
			blog::parse_attachments('view', $row['post_id'], $message, $attach_rows);
			
			$rows[] = "\t<item>";
			$rows[] = "\t\t<dc:creator>" . $row['username'] . '</dc:creator>';
			$rows[] = "\t\t<pubDate>" . gmdate($date_format, $row['post_time']) . '</pubDate>';
			$rows[] = "\t\t<guid isPermaLink=\"true\">$link</guid>";
			$rows[] = "\t\t<link>$link</link>";
			$rows[] = "\t\t<title>" . $row['post_title'] . '</title>';
			$rows[] = "\t\t<description>" . $message . '</description>';
			$rows[] = "\t</item>";
			
			$feed_updated_time = max($feed_updated_time, $row['post_time']);
		}
		unset($post_rows);
	break;
}

// RSS feed header
$rdf[] = '<?xml version="1.0" encoding="UTF-8"?>';
$rdf[] = '<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom" xmlns:dc="http://purl.org/dc/elements/1.1/">';
$rdf[] = '<channel>';
$rdf[] = "\t<atom:link href=\"$feed_link\" rel=\"self\" type=\"application/rss+xml\" />";
$rdf[] = "\t<title>" . $config['sitename'] . '</title>';
$rdf[] = "\t<description>" . $config['site_desc'] . '</description>';
$rdf[] = "\t<link>" . blog::append_sid('index') . '</link>';
$rdf[] = "\t<lastBuildDate>" . gmdate($date_format, $feed_updated_time) . '</lastBuildDate>';
$rdf[] = "\t<language>". $user->data['user_lang'] .'</language>';
$rdf[] = "\t<ttl>6</ttl>";

$rdf = array_merge($rdf, $rows);

// RSS feed footer
$rdf[] = "</channel>";
$rdf[] = "</rss>";

// Gzip compression
if ($config['gzip_compress'])
{
	if (@extension_loaded('zlib') && !headers_sent())
	{
		ob_start('ob_gzhandler');
	}
}

header('Content-Type: application/rss+xml; charset=UTF-8');
header("Last-Modified: " . gmdate($date_format, $feed_updated_time));

// Output the feed
blog::print_content(implode("\n", $rdf));

garbage_collection();
exit_handler();

?>
