<?php
/*
*
* @name view.php
* @package cBB Blog
* @version $Id: view.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*/

/**
* @ignore
*/
define('IN_PHPBB', true);
define('IN_BLOG', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : '../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
include($phpbb_root_path . 'includes/functions_display.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup();

$post_id	= request_var('p', 0);
$cat_id		= request_var('c', 0);
$mode		= request_var('mode', '');

if(isset($_POST['cmntloader']))
{
	if(!empty($blog_hook->enabled))
	{
		include($blog_hook->root_path . 'includes/functions_comment.' . $phpEx);
		$comment_loader = new comment_loader($post_id);
		$comment_loader->parse();
	}
	
	garbage_collection();
	exit_handler();
}

if(empty($blog_hook->enabled))
{
	redirect(append_sid($phpbb_root_path . "index.$phpEx"));
}

$sql_ary = array(
	'SELECT'	=> 'p.*, pt.cat_id, u.username, u.user_colour',
	'FROM'		=> array(BLOG_POSTS_TABLE => 'p'),
	
	'LEFT_JOIN'	=> array(
		array(
			'FROM'	=> array(BLOG_POSTS_TRACK_TABLE => 'pt'),
			'ON'	=> 'pt.post_id = p.post_id'
		),
		array(
			'FROM'	=> array(USERS_TABLE => 'u'),
			'ON'	=> 'u.user_id = p.poster_id'
		)
	),
	
	'WHERE'		=> "p.post_id = $post_id
		AND p.post_time < " . time() . '
		AND pt.cat_id ' . (($cat_id) ? "= $cat_id" : 'IS NOT NULL'),
);

if(!$auth->acl_get('m_blog_approve'))
{
	$sql_ary['WHERE'] .= ' AND p.post_approved = 1';
}

$sql = $db->sql_build_query('SELECT', $sql_ary);
$result = $db->sql_query($sql);
$post_data = $db->sql_fetchrow($result);
$db->sql_freeresult($result);

if(!$post_data)
{
	trigger_error('NO_ENTRY');
}

if(!empty($blog_hook->seo))
{
	$blog_hook->seo->prepare_url('post', $post_data, 'title');

	if(!$blog_hook->seo->check('post', $post_data['post_url'], $post_data['post_title']))
	{
		redirect(blog::append_sid('view', $post_data, false, $cat_id));
	}
}

// Get all the categories for the post
blog::append_post_categories($post_data, $post_id);

if($cat_id && !blog::category_auth($cat_id))
{
	if(sizeof($post_data['cats']) == 1)
	{
		trigger_error('NO_ENTRY');
	}
	
	redirect(blog::append_sid('view', $post_data));
}

// Get and parse post tags
$post_tags = array();

$sql = 'SELECT tag_name
	FROM ' . BLOG_TAGS_TABLE . "
	WHERE post_id = $post_id";
$result = $db->sql_query($sql);
while($row = $db->sql_fetchrow($result))
{
	$post_tags[] = '<a href="' . blog::append_sid('search', false, array('tag' => urlencode(strtolower($row['tag_name'])))) . '">' . $row['tag_name'] . '</a>';
}
$db->sql_freeresult($result);

switch($post_data['post_type'])
{
	case POST_TYPE_BBCODE:
		$message = censor_text($post_data['post_text']);

		// Parse bbcode if bbcode uid stored and bbcode enabled
		if ($post_data['bbcode_uid'])
		{
			if (!class_exists('bbcode'))
			{
				include($phpbb_root_path . 'includes/bbcode.' . $phpEx);
			}

			$bbcode = new bbcode($post_data['bbcode_bitfield']);
			$bbcode->bbcode_second_pass($message, $post_data['bbcode_uid']);
		}

		$message = bbcode_nl2br($message);
		$message = blog::smiley_text($message);
		
		blog::clear_resume_tags($message);
	break;
	
	case POST_TYPE_HTML:
		include($blog_hook->root_path . 'includes/html_message_parser.' . $phpEx);
		$message_parser = new html_parse_message();
		
		$message_parser->message = $post_data['post_text'];
		$message_parser->clear_resume = true;
				
		$message = $message_parser->format_display(true, false, true);
	break;
	
	default:
		trigger_error('NO_ENTRY');
	break;
}

$permanent_link = blog::append_sid('view', false, array('p' => $post_id), false, true);
$permanent_link	= blog::strip_vars($permanent_link, true);

blog::parse_attachments('view', $post_id, $message);

$template->assign_vars(array(
	'POST_ID'				=> $post_data['post_id'],
	'POST_TITLE'			=> $post_data['post_title'],
	'POST_CONTENT'			=> $message,
	'POST_HEADER'			=> $user->lang('POST_HEADER', get_username_string('full', $post_data['poster_id'], $post_data['username'], $post_data['user_colour'], $post_data['post_username']), $user->format_date($post_data['post_time'], $config['blog_date_format'])),
	'POST_TAGS'				=> (sizeof($post_tags) ? implode(', ', $post_tags) : ''),
	
	'S_ALLOW_COMMENTS'		=> ($post_data['allow_comments']) ? true : false,
	'S_CAN_COMMENT'			=> ($post_data['allow_comments'] && blog::auth('cmnt_post')) ? true : false,
	'S_CATEGORIES'			=> $user->lang('POST_CATEGORIES', blog::row_categories($post_data)),
	'S_COMMENT_MAXLENGTH'	=> $config['blog_cmnt_maxlength'],
	'S_HAVE_COMMENTS'		=> ($post_data['comment_count']) ? true : false,
	'S_PERMANENT_LINK'		=> $user->lang('POST_PERMANENT_LINK', $permanent_link),

	'SHOW_SHARE_BUTTONS'	=> $config['blog_share_buttons'],
	'UNAPPROVED_IMG'		=> (!$post_data['post_approved']) ? str_replace('/>', 'class="js-resized" />', $user->img('icon_topic_unapproved', 'BLOG_POST_UNAPPROVED')) : '',

	'U_APPROVE'				=> (!$post_data['post_approved'] && blog::auth('approve', $post_data)) ? blog::append_sid('posting', false, array('mode' => 'approve', 'p' => $post_id), $cat_id) : '',
	'U_COMMENT_ACTION'		=> blog::append_sid('view', false, array('p' => $post_id)),
	'U_DELETE'				=> (blog::auth('delete', $post_data) ? blog::append_sid('index', false, array('c' => $cat_id, 'p' => $post_id)) . '#delete_entry' : ''),
	'U_EDIT'				=> (blog::auth('edit', $post_data) ? blog::append_sid('posting', false, array('mode' => 'edit', 'p' => $post_id), $cat_id) : ''),
	'U_FORUM_POST'			=> (blog::auth('forum', $post_data) && $post_data['post_type'] == POST_TYPE_BBCODE) ? blog::append_sid('view', $post_data) . '#forum_post_entry' : '',
	'U_PRINT'				=> blog::append_sid('view', false, array('p' => $post_id, 'mode' => 'print'), $cat_id),
	'U_SHARE_LINK'			=> $permanent_link,
	'U_SHARE_LINK_ENCODED'	=> urlencode($permanent_link),
));

// Related Entries
if($config['blog_related_posts'])
{
	preg_match_all('#([a-z0-9]{4,})#', strtolower($post_data['post_title']), $matches);

	if(!empty($matches[1]))
	{
		$like_exp	= $postrow = array();
		$matches[1]	= array_unique($matches[1]);

		foreach($matches[1] as $value)
		{
			$like_exp[] = 'LOWER(post_title) ' . $db->sql_like_expression($db->any_char . trim($value) . $db->any_char);
		}

		$sql = 'SELECT post_id, post_title, post_url
			FROM ' . BLOG_POSTS_TABLE . "
			WHERE post_id <> $post_id
			AND post_time < " . time() . '
			AND post_approved = 1
			AND (' . implode(' OR ', $like_exp) . ')
			ORDER BY RAND()';
		$result = $db->sql_query_limit($sql, $config['blog_related_posts']);
		while($row = $db->sql_fetchrow($result))
		{
			$postrow[$row['post_id']] = $row;
		}
		$db->sql_freeresult($result);
		
		if(sizeof($postrow))
		{
			blog::append_post_categories($postrow);

			foreach($postrow as $row)
			{
				if(!empty($blog_hook->seo))
				{
					$blog_hook->seo->prepare_url('post', $row, 'title');
				}

				$template->assign_block_vars('related', array(
					'MINI_POST_IMG'		=> '<img class="js-resized" src="' . $user->img('icon_post_target', 'POST', false, '', 'src') . '" />',
					'S_TITLE'			=> $row['post_title'],
					'U_TITLE'			=> blog::append_sid('view', $row, false, true),
				));
			}
		}
	}
}

blog::build_navbar(array(
	'cat_id' 		=> $cat_id,
	'post_id'		=> $post_id,
	'post_title'	=> $post_data['post_title'],
));

// Output page
page_header($user->lang['BLOG'] . ' | ' . $post_data['post_title']);

blog::get_widgets();

$template->set_filenames(array(
	'body' => 'blog/view_' . (($mode == 'print') ? 'print' : 'body') . '.html'
));

page_footer(false);

?>
