<?php
/*
*
* @name acp_blog.php
* @package phpBB3 cBB Blog
* @version $Id: acp_blog.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if(!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class acp_blog
{
	var $u_action;
	var $new_config = array();
	var $page_title = '';

	function main($id, $mode)
	{
		global $db, $user, $template, $cache, $config, $blog_hook, $phpEx;
		
		$user->add_lang('mods/blog/acp');

		$action = request_var('action', '');
		$action = (isset($_POST['save']) ? 'save' : $action);
		
		$submit = (isset($_POST['submit']) ? true : false);
		$error	= $notice = array();
		
		if(empty($blog_hook))
		{
			trigger_error('MOD_NO_INSTALLED');
		}
		
		if(!defined('BLOG_VERSION'))
		{
			include($blog_hook->root_path . 'includes/constants.' . $phpEx);
		}
		
		if(!class_exists('blog'))
		{
			include($blog_hook->root_path . 'includes/functions.' . $phpEx);
		}

		switch($mode)
		{
			case 'main':
				$this->tpl_name		= 'blog/acp_main';

				$display_vars = array(
					'legend1'		=> 'GENERAL_OPTIONS',
					'blog_enabled'			=> array('lang' => 'BLOG_ENABLED',			'validate' => 'bool',		'type' => 'radio:yes_no'),
					'blog_seo_enabled'		=> array('lang' => 'BLOG_SEO_ENABLED',		'validate' => 'bool',		'type' => 'radio:yes_no'),
					'blog_index_stats'		=> array('lang' => 'BLOG_INDEX_STATS',		'validate' => 'bool',		'type' => 'radio:yes_no'),
					'blog_share_buttons'	=> array('lang' => 'BLOG_SHARE_BUTTONS',	'validate' => 'bool',		'type' => 'radio:yes_no'),
					'blog_related_posts'	=> array('lang' => 'BLOG_RELATED_POSTS',	'validate' => 'int:0:50',	'type' => 'text:3:2'),
					'blog_pagination'		=> array('lang' => 'BLOG_PAGINATION_TYPE',	'validate' => 'string',		'type' => 'select',	'method' => 'make_select',	'params' => array('pagination_type', '{CONFIG_VALUE}')),
					'blog_domain'			=> array('lang' => 'BLOG_DOMAIN',			'validate' => 'string',		'type' => 'text:40:255'),

					'legend2'		=> 'ENTRIES',
					'blog_date_format'		=> array('lang' => 'BLOG_DATE_FORMAT',		'validate' => 'string',		'type' => 'select',	'method' => 'make_select',	'params' => array('date_format', '{CONFIG_VALUE}')),
					'blog_posts_per_page'	=> array('lang' => 'BLOG_POSTS_PER_PAGE',	'validate' => 'int:1:100',	'type' => 'text:3:3'),
					'blog_resume_chars'		=> array('lang' => 'BLOG_RESUME_CHARS',		'validate' => 'int:1:1000',	'type' => 'text:4:4'),
					'blog_post_type'		=> array('lang' => 'BLOG_POST_TYPE',		'validate' => 'int',		'type' => 'select',	'method' => 'make_select',	'params' => array('post_type', '{CONFIG_VALUE}')),
					
					'legend3'		=> 'COMMENTS',
					'blog_allow_comments'	=> array('lang' => 'BLOG_ALLOW_COMMENTS',	'validate' => 'bool',			'type' => 'radio:yes_no'),
					'blog_cmnt_maxlength'	=> array('lang' => 'BLOG_CMNT_MAXLENGTH',	'validate' => 'int:1:5000',		'type' => 'text:5:4'),
					'blog_cmnt_per_page'	=> array('lang' => 'BLOG_CMNT_PER_PAGE',	'validate' => 'int',			'type' => 'text:3:2'),
					'blog_cmnt_order'		=> array('lang' => 'BLOG_CMNT_ORDER',		'validate' => 'string',			'type' => 'select',	'method' => 'make_select',	'params' => array('cmnt_order', '{CONFIG_VALUE}')),
				);
				
				$this->new_config = $config;
				$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc(request_var('config', array('' => ''), true)) : $this->new_config;
			
				// We validate the complete config if whished
				validate_config_vars($display_vars, $cfg_array, $error);
				
				// Do not write values if there is an error
				if (sizeof($error))
				{
					$submit = false;
				}
				
				// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
				foreach ($display_vars as $config_name => $null)
				{
					if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
					{
						continue;
					}

					$this->new_config[$config_name] = $config_value = $cfg_array[$config_name];

					if ($submit)
					{
						set_config($config_name, $config_value);
					}
				}
				
				if ($submit)
				{
					add_log('admin', 'LOG_BLOG_CONFIG');
					$notice[] = 'CONFIG_UPDATED';
				}
				
				/* Count entries */
				$sql = 'SELECT COUNT(*) AS total_entries
					FROM ' . BLOG_POSTS_TABLE;
				$result = $db->sql_query($sql);
				$total_entries = (int)$db->sql_fetchfield('total_entries');
				$db->sql_freeresult($result);
				
				/* Count categories */
				$total_categories = count(blog::get_category('ids'));
				
				/* Count tags */
				$sql = 'SELECT COUNT(DISTINCT(tag_name)) AS total_tags
					FROM ' . BLOG_TAGS_TABLE;
				$result = $db->sql_query($sql);
				$total_tags = (int)$db->sql_fetchfield('total_tags');
				$db->sql_freeresult($result);
				
				// Count comments total
				$sql = 'SELECT COUNT(*) AS total
					FROM ' . BLOG_COMMENTS_TABLE;
				$result = $db->sql_query($sql);
				$comments_total = (int)$db->sql_fetchfield('total');
				$db->sql_freeresult($result);
				
				// Count comments marked as spam
				$sql = 'SELECT COUNT(*) AS total
					FROM ' . BLOG_COMMENTS_TABLE . '
					WHERE spam_mark = 1';
				$result = $db->sql_query($sql);
				$comments_spam = (int)$db->sql_fetchfield('total');
				$db->sql_freeresult($result);
				
				// Count attachments
				$total_attachments = array();

				$sql = 'SELECT filesize
					FROM ' . BLOG_ATTACHMENTS_TABLE . '
					GROUP BY real_filename';
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$total_attachments[] = (int)$row['filesize'];
				}
				$db->sql_freeresult($result);
				
				$template->assign_vars(array(
					'TOTAL_ATTACHMENTS'			=> count($total_attachments),
					'TOTAL_ATTACHMENTS_SIZE'	=> get_formatted_filesize(array_sum($total_attachments)),
					'TOTAL_ENTRIES'				=> $total_entries,
					'TOTAL_CATEGORIES'			=> $total_categories,
					'TOTAL_TAGS'				=> $total_tags,
					
					'TOTAL_COMMENTS'			=> $comments_total,
					'TOTAL_COMMENTS_SPAM'		=> $comments_spam,
				));
				
				// Check version
				$this->check_version_info(request_var('versioncheck_force', false));
			break;
			
			case 'category':
				$this->tpl_name = 'blog/acp_category';
				
				$cat_id = request_var('id', 0);
				
				switch($action)
				{
					case 'delete':
						if(!$cat_id)
						{
							$error[] = 'NO_CATEGORY';
							break;
						}
						
						if (confirm_box(true))
						{
							$subcat_action	= request_var('subcat_action', '');
							$entry_action	= request_var('entry_action', '');
							$new_cat 		= request_var('new_cat', 0);
							$cat_ids		= array($cat_id);
							
							$parents = blog::get_category('parents');
							
							if(!$subcat_action || !$entry_action)
							{
								$error[] = 'NO_ACTION';
								break;
							}
							
							if(isset($parents[$cat_id]))
							{
								switch($subcat_action)
								{
									case 'remove':
										$cat_ids = array_merge($cat_ids, $parents[$cat_id]);
									break;
									
									case 'retain':
										$sql = 'UPDATE ' . BLOG_CATS_TABLE . '
											SET parent_id = 0
											WHERE ' . $db->sql_in_set('cat_id', $parents[$cat_id]);
										$db->sql_query($sql);
									break;
								}
							}

							$entry_ary = array();
							
							$sql = 'SELECT post_id
								FROM ' . BLOG_POSTS_TRACK_TABLE . '
								WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
							$result = $db->sql_query($sql);
							while($row = $db->sql_fetchrow($result))
							{
								$post_id = (int)$row['post_id'];
								$entry_ary[$post_id] = $post_id;
							}
							$db->sql_freeresult($result);

							switch($entry_action)
							{
								case 'retain':
									if(!$new_cat)
									{
										$error[] = 'NO_CATEGORY';
										break;
									}
									
									// Delete new cat from the posts (safe from duplicate content)
									$sql = 'DELETE FROM ' . BLOG_POSTS_TRACK_TABLE . '
										WHERE ' . $db->sql_in_set('post_id', $entry_ary) . "
										AND cat_id = $new_cat";
									$db->sql_query($sql);
									
									$sql = 'UPDATE ' . BLOG_POSTS_TRACK_TABLE . "
										SET cat_id = $new_cat
										WHERE " . $db->sql_in_set('post_id', $entry_ary) . '
										AND ' . $db->sql_in_set('cat_id', $cat_ids);
									$db->sql_query($sql);
								break;
								
								case 'delete':
									$table_ary = array(BLOG_POSTS_TABLE, BLOG_POSTS_TRACK_TABLE, BLOG_COMMENTS_TABLE, BLOG_TAGS_TABLE);
									$archive_post_count = array();
									
									// Get post count per archive to update
									$sql = 'SELECT archive_id, post_approved
										FROM ' . BLOG_POSTS_TABLE . '
										WHERE ' . $db->sql_in_set('post_id', $entry_ary);
									$result = $db->sql_query($sql);
									while($row = $db->sql_fetchrow($result))
									{
										$archive_id = (int)$row['archive_id'];
										
										if(!isset($archive_post_count[$archive_id]))
										{
											$archive_post_count[$archive_id] = array(
													'post_count'		=> 0,
													'post_count_real'	=> 0,
											);
										}
										
										if($row['post_approved'])
										{
											$archive_post_count[$archive_id]['post_count']++;
										}

										$archive_post_count[$archive_id]['post_count_real']++;
									}
									$db->sql_freeresult($result);
									
									// Update post count in each archive
									foreach($archive_post_count as $archive_id => $count)
									{
										$sql = 'UPDATE ' . BLOG_ARCHIVES_TABLE . '
											SET post_count = post_count - ' . $count['post_count'] . ',
												post_count_real = post_count_real - ' . $count['post_count_real'] . '
											WHERE archive_id = ' . $archive_id;
										$db->sql_query($sql);
									}
									
									// Delete all avaliable data for posts
									foreach($table_ary as $table)
									{
										$sql = 'DELETE FROM ' . $table . '
											WHERE ' . $db->sql_in_set('post_id', $entry_ary);
										$db->sql_query($sql);
									}
								break;
							}
							
							if(!sizeof($error))
							{
								$cat_name = blog::get_category($cat_id, 'cat_name');

								$sql = 'DELETE FROM ' . BLOG_CATS_TABLE . '
									WHERE ' . $db->sql_in_set('cat_id', $cat_ids);
								$db->sql_query($sql);
								
								add_log('admin', 'LOG_BLOG_CATEGORY_DELETE', $cat_name);
								
								$notice[] = 'CATEGORY_DELETED';
							}
							
							$cache->destroy('_blog_categories');
						}
						else
						{
							$cat_ids = blog::get_category('ids');
							
							$s_category_options = '<option value="">' . $user->lang['CATEGORY_SELECT'] . '</option>';
							
							foreach($cat_ids as $cid => $cdata)
							{
								if($cid != $cat_id)
								{
									$s_category_options .= '<option value="' . $cid . '">' . $cdata['cat_name'] . '</option>';
								}
							}
							
							$template->assign_vars(array(
								'S_CATEGORY_OPTIONS'	=> $s_category_options,
							));
							
							confirm_box(false, $user->lang['CATEGORY_DELETE_CONFIRM'], build_hidden_fields(array(
								'id'		=> $cat_id,
								'action'	=> 'delete',
							)), 'blog/category_delete.html');
						}
					break;

					case 'edit':
						if(!$cat_id)
						{
							$error[] = 'NO_CATEGORY';
							break;
						}
						
						$cat_data = blog::get_category($cat_id);
						
						unset($cat_data['cat_id']);

					// no break;

					case 'add':
						$display_vars = array(
							'legend1'		=> 'GENERAL_OPTIONS',
							'cat_enabled'		=> array('lang' => 'BLOG_CAT_ENABLED',		'validate' => 'boll',			'type' => 'radio:yes_no'),
							'parent_id'			=> array('lang' => 'BLOG_CAT_PARENT',		'type' => 'select',		'method' => 'make_select',	'params' => array('cat_parent', '{CONFIG_VALUE}', $cat_id)),
							'cat_name'			=> array('lang' => 'BLOG_CAT_NAME',			'validate' => 'string:1:255',	'type' => 'text:50:255'),
							'cat_description'	=> array('lang' => 'BLOG_CAT_DESCRIPTION',	'validate' => 'string:0:255',	'type' => 'textarea:5:45'),
							'cat_groups'		=> array('lang' => 'BLOG_CAT_GROUPS',		'type' => 'custom',		'method' => 'groups_select',	'params' => array('{CONFIG_VALUE}')),
						);
					
						if($action == 'add')
						{
							$cat_data = array(
								'parent_id'			=> 0,
								'cat_name'			=> '',
								'cat_url'			=> '',
								'cat_description'	=> '',
								'cat_enabled'		=> 1,
								'cat_groups'		=> 'all',
							);
						}

						$this->new_config = $cfg_array = $cat_data;
						
						if(isset($_REQUEST['config']))
						{
							$cfg_array = utf8_normalize_nfc(request_var('config', array('' => ''), true));
							$cfg_array['cat_groups'] = (isset($_POST['all_groups']) ? 'all' : implode(',', request_var('group', array(0))));
						
							validate_config_vars($display_vars, $cfg_array, $error);
						}

						// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
						foreach($display_vars as $config_name => $null)
						{
							if(strpos($config_name, 'legend') !== false)
							{
								continue;
							}

							$this->new_config[$config_name] = $cfg_array[$config_name];
						}
						
						// Do not write values if there is an error
						if(sizeof($error))
						{
							$submit = false;
						}

						if($submit)
						{
							if(!class_exists('blog_seo'))
							{
								include($blog_hook->root_path . 'includes/functions_seo.' . $phpEx);
							}
							
							$blog_seo = new blog_seo();
							$this->new_config['cat_url'] = $blog_seo->format_url($this->new_config['cat_name']);
		
							// Check if category already exists
							$sql = 'SELECT cat_id
								FROM ' . BLOG_CATS_TABLE . "
								WHERE cat_url = '" . $db->sql_escape($this->new_config['cat_url']) . "'
								AND parent_id = " . $this->new_config['parent_id'];
							$result = $db->sql_query($sql);
							$current_cat_id = (int)$db->sql_fetchfield('cat_id');
							$db->sql_freeresult($result);
							
							if($current_cat_id)
							{
								$error[] = 'CATEGORY_ALREADY_EXISTS';
							}
							else
							{
								if($action == 'add')
								{
									$sql = 'INSERT INTO ' . BLOG_CATS_TABLE . ' ' . $db->sql_build_array('INSERT', $this->new_config);
									add_log('admin', 'LOG_BLOG_CATEGORY_ADD', $this->new_config['cat_name']);
								}
								else
								{
									$sql = 'UPDATE ' . BLOG_CATS_TABLE . '
										SET ' . $db->sql_build_array('UPDATE', $this->new_config) . "
										WHERE cat_id = $cat_id";
										
									add_log('admin', 'LOG_BLOG_CATEGORY_EDIT', $cat_data['cat_name']);
								}
								
								$db->sql_query($sql);
								
								$cache->destroy('_blog_categories');

								$notice[] = 'CONFIG_UPDATED';
								break;
							}
						}

						$template->assign_vars(array(
							'S_EDIT'		=> true,
	
							'S_HIDDEN_FIELDS'	=> build_hidden_fields(array(
								'id'			=> $cat_id,
								'action'		=> $action,
							)),

							'U_BACK'			=> $this->u_action,
						));
					break;
				}
				
				$parents 	= blog::get_category('parents', '', true);
				$rows		= blog::get_category('ids', '', true);
				
				foreach($parents[0] as $row_id)
				{
					if(!isset($rows[$row_id]))
					{
						continue;
					}

					$template->assign_block_vars('catrow', array(
						'S_TITLE'	=> $rows[$row_id]['cat_name'],
						'S_DESC'	=> $rows[$row_id]['cat_description'],
						
						'U_EDIT'	=> $this->u_action . '&amp;action=edit&amp;id=' . $row_id,
						'U_DELETE'	=> $this->u_action . '&amp;action=delete&amp;id=' . $row_id,
					));
					
					if(!empty($parents[$row_id]))
					{
						foreach($parents[$row_id] as $child_id)
						{
							$template->assign_block_vars('catrow.child', array(
								'S_TITLE'	=> $rows[$child_id]['cat_name'],
								'S_DESC'	=> $rows[$child_id]['cat_description'],
								
								'U_EDIT'	=> $this->u_action . '&amp;action=edit&amp;id=' . $child_id,
								'U_DELETE'	=> $this->u_action . '&amp;action=delete&amp;id=' . $child_id,
							));
						}
					}
				}
				
				$template->assign_vars(array(
					'U_ADD'			=> $this->u_action . '&amp;action=add',
				));
			break;
			
			case 'attach':
				$this->tpl_name = 'blog/acp_attachments';
				
				$display_vars = array(
					'legend1'		=> 'ACP_ATTACHMENT_SETTINGS',
					'blog_allow_attachments'	=> array('lang' => 'BLOG_ALLOW_ATTACHMENTS',	'validate' => 'bool',		'type' => 'radio:yes_no'),
					'blog_attach_gallery'		=> array('lang' => 'BLOG_ATTACH_GALLERY',		'validate' => 'bool',		'type' => 'radio:yes_no'),
					'blog_max_filesize'			=> array('lang' => 'BLOG_MAX_FILESIZE',			'validate' => 'string',		'type' => 'custom', 'method' => 'max_filesize'),
					'blog_max_attachments'		=> array('lang' => 'BLOG_MAX_ATTACHMENTS',		'validate' => 'int',		'type' => 'text:3:3'),
					'blog_attach_extensions'	=> array('lang' => 'BLOG_ATTACH_EXTENSIONS',	'type' => 'custom', 		'method' => 'attach_extensions'),
				);
				
				$cfg_array = $this->new_config = $config;
				
				if(isset($_REQUEST['config']))
				{
					$cfg_array = utf8_normalize_nfc(request_var('config', array('' => ''), true));
					$cfg_array['blog_attach_extensions'] = implode('|', request_var('blog_attach_extensions', array(0 => '')));
				}

				// We validate the complete config if whished
				validate_config_vars($display_vars, $cfg_array, $error);
				
				// Do not write values if there is an error
				if (sizeof($error))
				{
					$submit = false;
				}
				
				// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
				foreach ($display_vars as $config_name => $null)
				{
					if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
					{
						continue;
					}

					$this->new_config[$config_name] = $config_value = $cfg_array[$config_name];

					if($config_name == 'blog_max_filesize')
					{
						$size_var = request_var($config_name, '');
						$this->new_config[$config_name] = $config_value = ($size_var == 'kb') ? round($config_value * 1024) : (($size_var == 'mb') ? round($config_value * 1048576) : $config_value);
					}

					if ($submit)
					{
						set_config($config_name, $config_value);
					}
				}
				
				if ($submit)
				{
					add_log('admin', 'LOG_BLOG_CONFIG');
					$notice[] = 'CONFIG_UPDATED';
				}
			break;
			
			case 'widget':
				$this->tpl_name = 'blog/acp_widgets';
				
				$widget_id = request_var('id', 0);
				
				switch($action)
				{
					case 'edit':
						$sql = 'SELECT *
							FROM ' . BLOG_WIDGETS_TABLE . "
							WHERE widget_id = $widget_id";
						$result = $db->sql_query($sql);
						$widget_data = $db->sql_fetchrow($result);
						$db->sql_freeresult($result);
						
						if(!$widget_data)
						{
							break;
						}

					// no break
					
					case 'add':
						$display_vars = array(
							'legend1'			=> 'WIDGET_SETTINGS',
							'widget_title'		=> array('lang' => 'WIDGET_TITLE',		'validate' => 'string:1:100',	'type' => 'text:30:100'),
							'widget_style'		=> array('lang' => 'WIDGET_STYLE',		'type' => 'select',		'method' => 'make_select',		'params' => array('widget_style', '{CONFIG_VALUE}')),
						);

						if($action == 'add')
						{
							$basename = request_var('basename', '');
							
							if(!file_exists($blog_hook->root_path . 'widgets/' . $basename . '_widget.php'))
							{
								break;
							}
							
							$widget_data = array(
								'widget_title'		=> (isset($user->lang['widgets'][$basename]) ? $user->lang['widgets'][$basename] : $basename),
								'widget_basename'	=> $basename,
								'widget_options'	=> '',
								'widget_style'		=> WIDGET_STYLE_FULL,
							);
						}
						
						$class_name			= $widget_data['widget_basename'] . '_widget';
						$widget_filename	= $blog_hook->root_path . "widgets/$class_name.php";

						if(file_exists($widget_filename) && !class_exists($class_name))
						{
							include($widget_filename);
						}

						if(!class_exists($class_name))
						{
							$error[] = 'WIDGET_NO_EXISTS';
							break;
						}
						
						// Initialize the widget class
						$widget_class = new $class_name();
						
						$has_config = $widget_class->get_config();

						if($has_config && is_array($widget_class->display_vars))
						{
							$display_vars['legend2'] = 'WIDGET_OPTIONS';
							$display_vars += $widget_class->display_vars;
							
							if($action == 'edit')
							{
								$widget_data += @unserialize($widget_data['widget_options']);
							}
						}
						
						$this->new_config = $widget_data;

						if(isset($_REQUEST['config']))
						{
							$this->new_config = array_merge($this->new_config, utf8_normalize_nfc(request_var('config', array('' => ''), true)));
						}
						
						// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
						foreach($display_vars as $config_name => $vars)
						{
							if(strpos($config_name, 'legend') !== false)
							{
								continue;
							}

							if(!isset($this->new_config[$config_name]) && isset($vars['default']))
							{
								$this->new_config[$config_name] = $vars['default'];
								continue;
							}
						}

						if($submit)
						{
							// We validate the complete config if whished
							validate_config_vars($display_vars, $this->new_config, $error);

							// Do not write values if there is an error
							if(sizeof($error))
							{
								$submit = false;
							}
						}
						
						if($submit)
						{
							$options_ary = array();
							
							if($has_config)
							{
								foreach($widget_class->display_vars as $config_name => $data)
								{
									if(isset($this->new_config[$config_name]))
									{
										$options_ary[$config_name] = $this->new_config[$config_name];
									}
								}
							}
							
							$sql_ary = array(
								'widget_title'		=> $this->new_config['widget_title'],
								'widget_basename'	=> $widget_data['widget_basename'],
								'widget_options'	=> ($has_config) ? @serialize($options_ary) : '',
								'widget_style'		=> $this->new_config['widget_style'],
							);
							
							if($action == 'add')
							{
								$sql = 'SELECT MAX(widget_order) as max_widget_order
									FROM ' . BLOG_WIDGETS_TABLE;
								$result = $db->sql_query($sql);
								$max_widget_order = (int) $db->sql_fetchfield('max_widget_order');
								$db->sql_freeresult($result);

								$sql_ary['widget_order'] = $max_widget_order + 1;
							}
							
							if($action == 'edit')
							{               
								$sql = 'UPDATE ' . BLOG_WIDGETS_TABLE . '
									SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
									WHERE widget_id = ' . $widget_data['widget_id'];

								$notice[] = 'WIDGET_UPDATED';
							}
							else
							{
								$sql = 'INSERT INTO ' . BLOG_WIDGETS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);

								$notice[] = 'WIDGET_ADDED';
							}

							$db->sql_query($sql);
							
							$cache->destroy('_blog_widgets');
							
							add_log('admin', 'LOG_BLOG_WIDGET', $user->lang($sql_ary['widget_title']));

							break;
						}

						$template->assign_vars(array(
							'S_EDIT'		=> true,
							
							'S_HIDDEN_FIELDS'	=> build_hidden_fields(array(
								'action'	=> $action,
								'id'		=> $widget_id,
								'basename'	=> $widget_data['widget_basename'],
							)),
						));
					break;
					
					case 'delete':
						$sql = 'DELETE FROM ' . BLOG_WIDGETS_TABLE . "
							WHERE widget_id = $widget_id";
						$db->sql_query($sql);
						
						$cache->destroy('_blog_widgets');

						$notice[] = 'WIDGET_DELETED';
					break;
					
					case 'save':
						$widget_positions = request_var('widget', array(0 => 0));
						
						if(!empty($widget_positions))
						{
							$save_error = false;

							foreach($widget_positions as $position => $wid)
							{
								$sql = 'UPDATE ' . BLOG_WIDGETS_TABLE . '
									SET widget_order = ' . ((int)$position + 1) . "
									WHERE widget_id = $wid";

								if(!$db->sql_query($sql))
								{
									$save_error = true; 
									break;
								}
							}

							if($save_error)
							{
								$error[] = 'WIDGET_NO_SAVED';
								break;
							}
							
							$cache->destroy('_blog_widgets');
							
							$notice[] = 'WIDGET_SAVED';
						}
					break;
				}
				
				$avaliable_widgets = $installed_widgets = array();

				$sql = 'SELECT *
					FROM ' . BLOG_WIDGETS_TABLE . '
					ORDER BY widget_order ASC';
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$template->assign_block_vars('widget', array(
						'ID'		=> $row['widget_id'],
						'S_TITLE'	=> (isset($user->lang[$row['widget_title']]) ? $user->lang[$row['widget_title']] : $row['widget_title']),
						
						'U_EDIT'	=> $this->u_action . '&amp;action=edit&amp;id=' . $row['widget_id'],
						'U_DELETE'	=> $this->u_action . '&amp;action=delete&amp;id=' . $row['widget_id'],
					));
					
					$installed_widgets[] = $row['widget_basename'];
				}
				$db->sql_freeresult($result);
				
				// Get avaliable widgets
				$widgets_dir = $blog_hook->root_path . 'widgets/';

				if($dir = @opendir($widgets_dir))
				{
					while (($file = @readdir($dir)) !== false)
					{
						if (preg_match('#^([^&\'"<>]+)_widget\.php$#i', $file, $match))
						{
							if(!in_array($match[1], $installed_widgets))
							{
								$avaliable_widgets[$match[1]] = $match[1];
							}
						}
					}
					closedir($dir);

					asort($avaliable_widgets);
				}
				
				if(sizeof($avaliable_widgets))
				{
					$avaliable_options = '<option value="">' . $user->lang['WIDGET_ADD'] . '</option>';

					foreach($avaliable_widgets as $basename => $info)
					{
						$title = (isset($user->lang['widgets'][$basename]) ? $user->lang['widgets'][$basename] : $basename);

						$avaliable_options .= '<option value="' . $basename . '">' . $title . '</option>';
					}	
					unset($avaliable_widgets);
					
					$template->assign_var('S_AVALIABLE_OPTIONS', $avaliable_options);
				}
			break;
			
			case 'tools':
				$this->tpl_name = 'blog/acp_tools';

				$action 	= request_var('action', '');
				$source_id	= request_var('id', 0);
				
				if($action)
				{
					switch($action)
					{
						case 'source_edit':
							$sql = 'SELECT s.*, u.username
								FROM ' . BLOG_SOURCES_TABLE . ' s
									LEFT JOIN ' . USERS_TABLE . " u ON(u.user_id = s.source_poster)
								WHERE s.source_id = $source_id";
							$result = $db->sql_query($sql);
							$source_data = $db->sql_fetchrow($result);
							$db->sql_freeresult($result);
							
							if(!$source_data)
							{
								$error[] = 'NO_SOURCE';
								break;
							}

						// no break;

						case 'source_add':
							$this->page_title = 'BLOG_SOURCES';

							$display_vars = array(
								'legend1'		=> 'SOURCE_OPTIONS',
								'source_title'		=> array('lang' => 'SOURCE_TITLE',		'validate' => 'string:1:255',	'type' => 'text:50:255'),
								'source_url'		=> array('lang' => 'SOURCE_URL',		'validate' => 'string:0:255',	'type' => 'text:50:255'),
								'source_gc'			=> array('lang' => 'SOURCE_GC',			'validate' => 'int',			'type' => 'select',		'method' => 'make_select',	'params' => array('source_gc', '{CONFIG_VALUE}')),
								'source_cats'		=> array('lang' => 'SOURCE_CATS',		'type' => 'custom',				'method' => 'build_category_select',				'params' => array('{CONFIG_VALUE}')),
								'username'			=> array('lang' => 'SOURCE_POSTER',		'validate' => 'string',			'type' => 'custom',		'method' => 'make_username_input',	'params' => array('{CONFIG_VALUE}')),
							);
						
							if($action == 'source_add')
							{
								$source_data = array(
									'source_cats'		=> '',
									'source_title'		=> '',
									'source_url'		=> 'http://',
									'source_gc'			=> 86400,
									'source_poster'		=> $user->data['user_id'],
									
									'username'			=> $user->data['username'],
								);
							}

							$this->new_config = $cfg_array = $source_data;
							
							if(isset($_REQUEST['config']))
							{
								$cfg_array = utf8_normalize_nfc(request_var('config', array('' => ''), true));
								$cfg_array['source_cats'] = request_var('category', array(0));

								validate_config_vars($display_vars, $cfg_array, $error);
								
								if(empty($cfg_array['source_cats']))
								{
									$error[] = 'NO_CATEGORY';
								}
								
								if(!preg_match('#^(http[s]?:\/\/[^\/]+\/)(?:(.*?)\/)?([^\/]+\.[a-zA-Z0-9]+)$#', $cfg_array['source_url']))
								{
									$error[] = 'SOURCE_URL_INVALID';
								}
							}

							// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
							foreach($display_vars as $config_name => $null)
							{
								if(strpos($config_name, 'legend') !== false)
								{
									continue;
								}

								$this->new_config[$config_name] = $cfg_array[$config_name];
							}

							if($submit && ($this->new_config['username'] != $source_data['username']))
							{
								$sql = 'SELECT user_id
									FROM ' . USERS_TABLE . "
									WHERE username_clean = '" . $db->sql_escape(utf8_clean_string($this->new_config['username'])) . "'
									AND user_type <> " . USER_IGNORE;
								$result = $db->sql_query($sql);
								$this->new_config['source_poster'] = (int)$db->sql_fetchfield('user_id');
								$db->sql_freeresult($result);
								
								if(!$this->new_config['source_poster'])
								{
									$error[] = 'NO_USER';
								}
							}
							
							// Do not write values if there is an error
							if(sizeof($error))
							{
								$submit = false;
							}
							
							if($submit)
							{
								$sql_ary = array(
									'source_cats'	=> (is_array($this->new_config['source_cats']) ? implode('|', $this->new_config['source_cats']) : $this->new_config['source_cats']),
									'source_title'	=> $this->new_config['source_title'],
									'source_url'	=> $this->new_config['source_url'],
									'source_gc'		=> $this->new_config['source_gc'],
									'source_poster'	=> $this->new_config['source_poster'],
								);
								
								if($action == 'source_add')
								{
									$sql = 'INSERT INTO ' . BLOG_SOURCES_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
								
									add_log('admin', 'LOG_BLOG_SOURCE_ADD', $this->new_config['source_title']);
								}
								else
								{
									$sql = 'UPDATE ' . BLOG_SOURCES_TABLE . '
										SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
										WHERE source_id = ' . $source_id;
										
									add_log('admin', 'LOG_BLOG_SOURCE_EDIT', $this->new_config['source_title']);
								}
								
								$db->sql_query($sql);

								$notice[] 	= 'CONFIG_UPDATED';
								$action	= $this->page_title = '';
							}

							$template->assign_vars(array(
								'S_HIDDEN_FIELDS'	=> build_hidden_fields(array(
									'id'			=> $source_id,
									'action'		=> $action,
								)),

								'U_ACTION'			=> $this->u_action,
								'U_BACK'			=> $this->u_action,
							));
						break;
						
						default:
							if(!confirm_box(true))
							{
								switch($action)
								{
									case 'stats':
										$confirm_lang = 'RESYNC_STATS_CONFIRM';
									break;

									default:
										$confirm_lang = 'CONFIRM_OPERATION';
									break;
								}

								confirm_box(false, $user->lang[$confirm_lang], build_hidden_fields(array(
									'i'			=> $id,
									'mode'		=> $mode,
									'action'	=> $action,
									'id'		=> $source_id,
								)));
							}
							else
							{
								switch($action)
								{
									case 'source_delete':
										// Get source title for log
										$sql = 'SELECT source_title
											FROM ' . BLOG_SOURCES_TABLE . "
											WHERE source_id = $source_id";
										$result = $db->sql_query($sql);
										$source_title = (string)$db->sql_fetchfield('source_title');
										$db->sql_freeresult($result);
										
										if($source_title)
										{
											// Remove source
											$sql = 'DELETE FROM ' . BLOG_SOURCES_TABLE . "
												WHERE source_id = $source_id";
											$db->sql_query($sql);

											add_log('admin', 'LOG_BLOG_SOURCE_DELETE', $source_title);
											$notice[] = 'CONFIG_UPDATED';
										}
									break;

									case 'stats':
										if($this->sync_stats())
										{
											add_log('admin', 'LOG_RESYNC_POSTCOUNTS');
											$notice[] = 'BLOG_RESYNC_SUCCESS';
										}
									break;
								}
							}
							
							$action = '';
						break;
					}
				}
				
				if(!$action)
				{
					$sql = 'SELECT s.*, u.username, u.user_colour
						FROM ' . BLOG_SOURCES_TABLE . ' s
							LEFT JOIN ' . USERS_TABLE . ' u ON(u.user_id = s.source_poster)
						ORDER BY LOWER(s.source_title)';
					$result = $db->sql_query($sql);
					while($row = $db->sql_fetchrow($result))
					{
						preg_match('#^(http[s]?:\/\/[^\/]+\/)(?:(.*?)\/)?([^\/]+\.[a-zA-Z0-9]+)$#', $row['source_url'], $matches);

						$template->assign_block_vars('source', array(
							'S_POSTER'	=> get_username_string('full', $row['source_poster'], $row['username'], $row['user_colour']),
							'S_TITLE'	=> $row['source_title'],
							'S_URL'		=> $row['source_url'],
							'S_URL_SHORT'	=> ($matches[2]) ? $matches[1] . '... /' . $matches[3] : $row['source_url'],
							
							'U_DELETE'	=> $this->u_action . '&amp;action=source_delete&amp;id=' . $row['source_id'],
							'U_EDIT'	=> $this->u_action . '&amp;action=source_edit&amp;id=' . $row['source_id'],
						));
					}
					$db->sql_freeresult($result);
					
					$template->assign_vars(array(
						'MAIN_PAGE'			=> true,
						'U_SOURCE_ADD'		=> $this->u_action . '&amp;action=source_add',
					));
				}
			break;
		}

		add_form_key('acp_blog');
		
		// Output relevant page
		if(!empty($display_vars))
		{
			foreach ($display_vars as $config_key => $vars)
			{
				if (!is_array($vars) && strpos($config_key, 'legend') === false)
				{
					continue;
				}

				if (strpos($config_key, 'legend') !== false)
				{
					$template->assign_block_vars('options', array(
						'S_LEGEND'		=> true,
						'LEGEND'		=> (isset($user->lang[$vars])) ? $user->lang[$vars] : $vars)
					);

					continue;
				}

				$type = explode(':', $vars['type']);

				$content = build_cfg_template($type, $config_key, $this->new_config, $config_key, $vars);

				if (empty($content))
				{
					continue;
				}

				$template->assign_block_vars('options', array(
					'KEY'			=> $config_key,
					'TITLE'			=> (isset($user->lang[$vars['lang']])) ? $user->lang[$vars['lang']] : $vars['lang'],
					'S_EXPLAIN'		=> (isset($user->lang[$vars['lang'] . '_EXPLAIN']) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : ''),
					'CONTENT'		=> $content,
				));

				unset($display_vars[$config_key]);
			}
		}
		
		if(!$this->page_title)
		{
			$this->page_title	= 'BLOG_' . strtoupper($mode) . '_TITLE';
		}

		$template->assign_vars(array(
			'BLOG_IN_ADMIN'		=> true,
			'L_TITLE'			=> $user->lang[$this->page_title],
			'L_TITLE_EXPLAIN'	=> $user->lang[$this->page_title . '_EXPLAIN'],

			'U_ACTION'		=> $this->u_action
		));
		
		if(sizeof($error) || sizeof($notice))
		{
			$notice_rows = (sizeof($error) ? $error : $notice);
			
			foreach($notice_rows as $id => $value)
			{
				if(isset($user->lang[$value]))
				{
					$notice_rows[$id] = $user->lang[$value];
				}
			}
			
			$template->assign_vars(array(
				'NOTICE'		=> implode('<br />', $notice_rows),
				'NOTICE_ERROR'	=> (sizeof($error) ? true : false),
			));
		}
	}
	
	/**
	 * groups_select
	 * Create a select with all groups
	*/
	function groups_select($select_groups)
	{
		global $db, $user;

		$output = '';
		$group_ary = array();

		if(!empty($select_groups))
		{
			$group_ary = (is_array($select_groups) ? $select_groups : explode(',', $select_groups));
		}

		// Get all the groups
		$sql = 'SELECT DISTINCT group_type, group_name, group_id
			FROM ' . GROUPS_TABLE . '
			ORDER BY group_type DESC, group_name ASC';
		$result = $db->sql_query($sql);

		$output = '<select style="width: 100%;" name="group[]" id="groups" multiple="multiple" size="5"
			onchange="group_change(this, 1)">';

		while($row = $db->sql_fetchrow($result))
		{
			$s_selected = (in_array($row['group_id'], $group_ary)) ? ' selected="selected"' : '';
			$output .= '<option' . (($row['group_type'] == GROUP_SPECIAL) ? ' class="sep"' : '') . ' value="' . $row['group_id'] . '"' . $s_selected . '>' . (($row['group_type'] == GROUP_SPECIAL) ? $user->lang['G_' . $row['group_name']] : $row['group_name']) . '</option>';
		}
		$db->sql_freeresult($result);

		$output .= '</select>';
		$output .= '<br/><label><input type="checkbox" class="radio" name="all_groups" onclick="group_change(this, 2)" value="1"' . (($select_groups == 'all' || isset($_POST['all_groups'])) ? ' checked="checked"' : '') . '/> ' . $user->lang['ALL_GROUPS'] . '</label>';

		return $output;
	}

	/**
	 * make_select
	 * Build select input for config
	*/
	function make_select($type, $select = false, $extra = false)
	{
		global $user;
	
		$string = '';
		
		switch($type)
		{
			case 'cat_parent':
				$cat_ary 	= blog::get_category(false);
				$ary		= array('NO_PARENT' => 0);
				
				foreach($cat_ary['parents'][0] as $parent_id)
				{
					if($parent_id == $extra)
					{
						continue;
					}

					$row = $cat_ary['ids'][$parent_id];
					$ary[$row['cat_name']] = $row['cat_id'];
				}
			break;
			
			case 'cmnt_order':
				$ary = array(
					'ORDER_ASC'		=> 'ASC',
					'ORDER_DESC'	=> 'DESC',
				);
			break;
			
			case 'date_format':
				$ary = array(
					'DATE_FORMAT_JFY'	=> 'j F, Y',
					'DATE_FORMAT_DMY'	=> 'd M Y',
					'DATE_FORMAT_MJSY'	=> 'M jS, y',
					'DATE_FORMAT_DMDY'	=> 'D M d, Y',
					'DATE_FORMAT_FJSY'	=> 'F jS, Y',
				);
			break;
			
			case 'pagination_type':
				$ary = array(
					'PAGINATION_FULL'	=> 'full',
					'PAGINATION_SIMPLE'	=> 'simple',
				);
			break;

			case 'post_type':
				$ary = array(
					'BLOG_TYPE_BBCODE'	=> POST_TYPE_BBCODE,
					'BLOG_TYPE_HTML'	=> POST_TYPE_HTML
				);
			break;
			
			case 'source_gc':
				$ary = array(
					'1_HOUR'	=> 3600,
					'2_HOURS'	=> 7200,
					'6_HOURS'	=> 21600,
					'1_DAY'		=> 86400,
					'7_DAYS'	=> 604800,
				);
			break;

			case 'widget_style':
				$ary = array(
					'WIDGET_STYLE_FULL'		=> WIDGET_STYLE_FULL,
					'WIDGET_STYLE_SIMPLE'	=> WIDGET_STYLE_SIMPLE,
					'WIDGET_STYLE_NONE'		=> WIDGET_STYLE_NONE,
				);
			break;
			
			default:
				return '';
			break;	
		}
	
		foreach($ary as $lang => $value)
		{
			$selected	= (($value == $select) ? ' selected="selected"' : '');
			$title		= (isset($user->lang[$lang]) ? $user->lang[$lang] : $lang);
	
			$string .= '<option value="' . $value . '"' . $selected . '>' . $title . '</option>';
		}
	
		return $string;
	}
	
	function build_category_select($selected_ids = '')
	{
		if(!is_array($selected_ids))
		{
			$selected_ids = explode('|', $selected_ids);
		}
		
		$parents 	= blog::get_category('parents');
		$rows		= blog::get_category('ids');
		$cat_rows	= array();
		
		$output		= '<div class="category-list-selectable">
		<ul class="category-list">';
		
		foreach($parents[0] as $row_id)
		{
			if(!isset($rows[$row_id]))
			{
				continue;
			}
			
			if(!$rows[$row_id]['cat_enabled'])
			{
				continue;
			}
			
			$cat_rows[$row_id] = $rows[$row_id];
			$cat_rows[$row_id]['childs'] = array();
			
			if(!empty($parents[$row_id]))
			{
				foreach($parents[$row_id] as $subrow_id)
				{
					$cat_rows[$row_id]['childs'][$subrow_id] = $rows[$subrow_id];
				}
			}
		}
		
		foreach($cat_rows as $cat_id => $cat_data)
		{
			$output .= '<li><label><input class="radio" type="checkbox" name="category[]" value="' . $cat_id . '"' . (in_array($cat_id, $selected_ids) ? ' checked="checked"' : '') . ' /> ' . $cat_data['cat_name'] . '</label>';
			
			if(!empty($cat_data['childs']))
			{
				$output .= '<ul class="sublist">';
				
				foreach($cat_data['childs'] as $child_id => $child_data)
				{
					$output .= '<li><label><input class="radio" type="checkbox" name="category[]" value="' . $child_id . '"' . (in_array($child_id, $selected_ids) ? ' checked="checked"' : '') . ' /> ' . $child_data['cat_name'] . '</label></li>';
				}
				
				$output .= '</ul>';
			}
			
			$output .= '</li>';
		}
		
		$output .= '</ul>
		</div>';
		
		return $output;
	}
	
	function make_username_input($select = '')
	{
		global $user, $phpbb_root_path, $phpEx;

		$u_find_username = append_sid("{$phpbb_root_path}memberlist.$phpEx", 'mode=searchuser&amp;form=acp_blog_tools&amp;field=username&amp;select_single=true');

		return '<input type="text" id="username" name="config[username]" size="20" value="' . $select . '" />
			<a href="' . $u_find_username . '" onclick="find_username(this.href); return false;">' . $user->lang['FIND_USERNAME'] . '</a>';
	}
	
	function max_filesize($value, $key = '')
	{
		// Determine size var and adjust the value accordingly
		$filesize = get_formatted_filesize($value, false, array('mb', 'kb', 'b'));
		$size_var = $filesize['si_identifier'];
		$value = $filesize['value'];

		return '<input type="text" id="' . $key . '" size="8" maxlength="15" name="config[' . $key . ']" value="' . $value . '" /> <select name="' . $key . '">' . size_select_options($size_var) . '</select>';
	}
	
	function attach_extensions($value, $key = '')
	{
		global $db, $user, $blog_hook;

		$output = '';
		$ext_ary = array();

		if(!empty($value))
		{
			$ext_ary = (is_array($value) ? $value : explode('|', $value));
		}
		
		$ext_text = @file_get_contents($blog_hook->root_path . 'includes/extensions.txt');
		
		if(!$ext_text)
		{
			return '';
		}
		
		$extensions = explode("\n", $ext_text);

		$output = '<select style="width: 100px;" name="' . $key . '[]" id="extensionss" multiple="multiple" size="5">';

		foreach($extensions as $ext)
		{
			$ext = trim($ext);

			$s_selected = (in_array($ext, $ext_ary)) ? ' selected="selected"' : '';
			$output .= '<option value="' . $ext . '"' . $s_selected . '>' . $ext . '</option>';
		}

		$output .= '</select>';

		return $output;
	}
	
	function sync_stats()
	{
		global $config, $db;
	
		$sql = 'SELECT COUNT(post_id) AS stat
			FROM ' . BLOG_POSTS_TABLE . '
			WHERE post_approved = 1';
		$result = $db->sql_query($sql);
		set_config('blog_num_posts', (int) $db->sql_fetchfield('stat'));
		$db->sql_freeresult($result);
		
		// Resync post counts
		$start = $max_post_id = 0;

		// Find the maximum post ID, we can only stop the cycle when we've reached it
		$sql = 'SELECT MAX(post_id) as max_post_id
			FROM ' . BLOG_POSTS_TABLE;
		$result = $db->sql_query($sql);
		$max_post_id = (int) $db->sql_fetchfield('max_post_id');
		$db->sql_freeresult($result);

		if($max_post_id)
		{
			$step = ($config['blog_num_posts']) ? (max((int) ($config['blog_num_posts'] / 5), 20000)) : 20000;
			$db->sql_query('UPDATE ' . USERS_TABLE . ' SET user_blog_posts = 0');

			while($start < $max_post_id)
			{
				$sql = 'SELECT COUNT(post_id) AS num_posts, poster_id
					FROM ' . BLOG_POSTS_TABLE . '
					WHERE post_id BETWEEN ' . ($start + 1) . ' AND ' . ($start + $step) . '
					AND post_approved = 1
					GROUP BY poster_id';
				$result = $db->sql_query($sql);
				if($row = $db->sql_fetchrow($result))
				{
					do
					{
						$sql = 'UPDATE ' . USERS_TABLE . " SET user_blog_posts = user_blog_posts + {$row['num_posts']} WHERE user_id = {$row['poster_id']}";
						$db->sql_query($sql);
					}
					while($row = $db->sql_fetchrow($result));
				}
				$db->sql_freeresult($result);

				$start += $step;
			}
			
			return true;
		}
		
		return false;
	}


	/**
	 * Check the version information of cBB Blog
	 *
	 * @param bool $force_update Ignores cached data. Defaults to false.
	 * @param int $ttl Cache version information for $ttl seconds. Defaults to 259200 (72 hours).
	 *
	 */
	function check_version_info($force_update = false, $ttl = 259200)
	{
		global $config, $template, $cache;

		$info = $cache->get('blog_versioncheck');

		if ($info === false || $force_update)
		{
			$errstr = '';
			$errno = 0;

			$info = get_remote_file('www.canidev.com', '/app/info', 'cbb-blog-10x.txt', $errstr, $errno);

			if ($info === false)
			{
				$cache->destroy('blog_versioncheck');
				return false;
			}

			$cache->put('blog_versioncheck', $info, $ttl);
		}

		$current_version = (isset($config['blog_version']) ? $config['blog_version'] : '1.0.0');
		$latest_version_info = ($info) ? explode("\n", $info, 2) : array($current_version, '#');
		
		$latest_version = trim($latest_version_info[0]);

		$template->assign_vars(array(
			'S_VERSION_NOT_UP_TO_DATE'	=> version_compare($current_version, $latest_version, '<') ? true : false,

			'U_VERSIONCHECK_FORCE'	=> $this->u_action . '&amp;versioncheck_force=1',
			'U_MORE_INFORMATION'	=> trim($latest_version_info[1]),
		));
	}
}

?>
