<?php
/*
*
* @name hook_blog.php
* @package phpBB3 cBB Blog
* @version $Id: hook_blog.php,v1.0.1 10/06/2013 $
*
* @copyright (c) 2013 CaniDev
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class blog_hook
{
	var $pathname	= 'blog/';

	var $enabled	= false;
	var $seo		= false;
	var $root_path	= '';
	var $domain		= '';
	
	var $month_names = array('', 'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December');

	function blog_hook()
	{
		global $config, $phpbb_root_path, $phpEx;
		
		$this->root_path = $phpbb_root_path . $this->pathname;
		
		if(!empty($config['blog_version']) && !empty($config['blog_enabled']) && !defined('IN_INSTALL'))
		{
			$this->enabled = true;

			include($this->root_path . 'includes/constants.' . $phpEx);
		}
			
		if($config['blog_domain'])
		{
			$this->domain = ((strpos($config['blog_domain'], 'http') === false) ? 'http://' : '') . $config['blog_domain'];

			if(substr($this->domain, -1) != '/')
			{
				$this->domain .= '/';
			}

			if(defined('IN_BLOG') && !defined('PHPBB_USE_BOARD_URL_PATH'))
			{
				define('PHPBB_USE_BOARD_URL_PATH', true);
				$config['force_server_vars'] = true;
			}
		}
		
		if(defined('IN_BLOG') && $this->enabled)
		{
			if(!class_exists('blog'))
			{
				include($this->root_path . 'includes/functions.' . $phpEx);
			}
			
			if($config['blog_seo_enabled'] || !empty($_GET['seo']) || !empty($_GET['checkseo']))
			{
				if(!class_exists('blog_seo'))
				{
					include($this->root_path . 'includes/functions_seo.' . $phpEx);
				}
				
				$this->seo = new blog_seo();
			}
		}
	}
	
	function setup()
	{
		global $template, $config, $user, $phpEx;
		
		if(!$this->enabled && !defined('ADMIN_START'))
		{
			return false;
		}
		
		$user->add_lang('mods/blog/main');
		
		$web_path = ($this->domain) ? $this->domain : (!empty($this->seo) ? generate_board_url() . '/' . $this->pathname : $this->root_path);
		
		$template->assign_vars(array(
			'BLOG_ENABLED'		=> ($this->enabled && defined('IN_BLOG')) ? true : false,
			'S_BLOG_VERSION'	=> (!empty($config['blog_version']) ? $config['blog_version'] : ''),
			'T_BLOG_PATH'		=> $web_path,
			'U_BLOG'			=> append_sid($web_path),
		));
		
		if(isset($_POST['blogajax']))
		{
			include($this->root_path . 'includes/functions_ajax.' . $phpEx);
			blog_ajax::parse();
			exit_handler();
		}
		
		if(defined('IN_BLOG'))
		{
			// Redirect to correct subdomain if needed
			if($this->domain && strpos($this->domain, $user->host) === false)
			{
				$redirect = $this->domain . substr($_SERVER['REQUEST_URI'], strlen($user->page['script_path']), strlen($_SERVER['REQUEST_URI']));
				redirect($redirect, false, true);
			}

			// Redirect to "normal" page is seo is disabled and url is in seo format
			if(!empty($this->seo) && !$config['blog_seo_enabled'])
			{
				$params = $this->seo->unformat();
				unset($this->seo);
				
				$redirect = call_user_func_array(array('blog', 'append_sid'), $params);
				redirect($redirect);
			}
		}
	}
	
	function append_sid(&$hook, $url, $params, $is_amp, $session_id)
	{
		global $phpbb_root_path;
		
		if(defined('IN_BLOG') && ($this->domain || !empty($this->seo)) && ($this->enabled || defined('IN_ERROR_HANDLER')))
		{
			if(preg_match('#^((' . preg_quote($phpbb_root_path, '#') . ')?(' . preg_quote($this->pathname, '#') . ')?).*#', $url, $match))
			{
				if($match[1])
				{
					$board_url = (!empty($match[3]) ? $this->domain : generate_board_url() . '/');

					$url = $board_url . substr($url, strlen($match[1]), strlen($url));
					return append_sid($url, $params, $is_amp, $session_id);
				}
			}
		}
	}
	
	function display(&$hook, $handle)
	{
		global $user, $auth, $template, $phpbb_root_path, $phpEx;
		global $blog_hook;
		
		if($this->enabled && $handle == 'body')
		{
			if((defined('IN_BLOG') || $user->page['page_name'] == "viewtopic.$phpEx") && !class_exists('blog'))
			{
				include($blog_hook->root_path . 'includes/functions.' . $phpEx);
			}
			
			if(defined('IN_BLOG'))
			{
				if($this->domain || !empty($this->seo))
				{
					$template->assign_var('ROOT_PATH', generate_board_url() . '/');
				}
				
				if(!defined('IN_BLOG_CRON'))
				{
					blog::load_cron();
				}
			}
			
			switch($user->page['page_name'])
			{
				case "memberlist.$phpEx":
					global $mode;
					
					if($mode == 'viewprofile')
					{
						global $member;
						self::memberlist($member);
					}
				break;

				case "viewtopic.$phpEx":
					global $topic_data;
					
					$topic_data['poster_id'] = $topic_data['topic_poster'];
					
					if(blog::auth('post') && blog::auth('forum', $topic_data))
					{
						$template->assign_vars(array(
							'LOAD_BLOG_FILES'	=> true,
							'U_BLOG_POST'		=> append_sid("{$phpbb_root_path}viewtopic.$phpEx?f=" . $topic_data['forum_id'] . '&amp;t=' . $topic_data['topic_id'] . '#blog_post_entry'),
						));
					}
				break;
			}
		}
	}
	
	static public function user_delete($mode, $user_row)
	{
		global $db, $blog_hook, $config, $phpEx;
		
		if(empty($config['blog_version']))
		{
			return false;
		}
		
		if(!defined('BLOG_POSTS_TABLE'))
		{
			include($blog_hook->root_path . "includes/constants.$phpEx");
		}

		switch($mode)
		{
			case 'retain':
				// Update posts
				$sql = 'UPDATE ' . BLOG_POSTS_TABLE . '
					SET poster_id = ' . ANONYMOUS . ", post_username = '" . $user_row['username'] . "'
					WHERE poster_id = " . $user_row['user_id'];
				$db->sql_query($sql);
				
				// Update comments
				$sql = 'UPDATE ' . BLOG_COMMENTS_TABLE . '
					SET user_id = ' . ANONYMOUS . ", username = '" . $user_row['username'] . "'
					WHERE user_id = " . $user_row['user_id'];
				$db->sql_query($sql);
				
				// Update attachments
				$sql = 'UPDATE ' . BLOG_ATTACHMENTS_TABLE . '
					SET poster_id = ' . ANONYMOUS . '
					WHERE poster_id = ' . $user_row['user_id'];
				$db->sql_query($sql);
			break;
			
			case 'remove':
				if(!class_exists('blog'))
				{
					include($blog_hook->root_path . 'includes/functions.' . $phpEx);
				}
				
				if(!function_exists('blog_delete_posts'))
				{
					include($blog_hook->root_path . 'includes/functions_posting.' . $phpEx);
				}

				// Select the deleted posts
				$post_ids = array();
				
				$sql = 'SELECT post_id
					FROM ' . BLOG_POSTS_TABLE . '
					WHERE poster_id = ' . $user_row['user_id'];
				$result = $db->sql_query($sql);
				while($row = $db->sql_fetchrow($result))
				{
					$post_ids[] = $row['post_id'];
				}
				$db->sql_freeresult($result);
				
				// Delete the posts and the data related to them
				blog_delete_posts($post_ids);
				
				// Delete all the comments from the user
				$sql = 'DELETE FROM ' . BLOG_COMMENTS_TABLE . '
					WHERE user_id = ' . $user_row['user_id'];
				$db->sql_query($sql);
			break;
		}
	}
	
	static public function memberlist($member)
	{
		global $template, $config, $user, $phpEx, $blog_hook;
		
		if(!$blog_hook->enabled || !$member['user_blog_posts'])
		{
			return false;
		}
		
		if(!class_exists('blog'))
		{
			include($blog_hook->root_path . 'includes/functions.' . $phpEx);
		}
		
		if(empty($blog_hook->seo) && $config['blog_seo_enabled'])
		{
			if(!class_exists('blog_seo'))
			{
				include($blog_hook->root_path . 'includes/functions_seo.' . $phpEx);
			}
			
			$blog_hook->seo = new blog_seo();
		}
		
		// Do the relevant calculations
		$memberdays 	= max(1, round((time() - $member['user_regdate']) / 86400));
		$posts_per_day 	= $member['user_blog_posts'] / $memberdays;
		$percentage 	= ($config['blog_num_posts']) ? min(100, ($member['user_blog_posts'] / $config['blog_num_posts']) * 100) : 0;
		
		$template->assign_vars(array(
			'BLOG_POSTS'			=> $member['user_blog_posts'],
			'BLOG_POSTS_DAY'		=> $user->lang('BLOG_POST_DAY', $posts_per_day),
			'BLOG_POSTS_PCT'		=> $user->lang('BLOG_POST_PCT', $percentage),
			
			'U_SEARCH_BLOG_USER'	=> blog::append_sid('search', false, array(
				'author_id'	=> $member['user_id']
			)),
		));
	}
	
	static public function viewtopic(&$postrow, $userrow)
	{
		global $blog_hook;

		if($blog_hook->enabled && !empty($userrow['blog_posts']))
		{
			$postrow['POSTER_BLOG_POSTS'] = $userrow['blog_posts'];
		}
	}
}

$blog_hook = new blog_hook();

/* Register the hook to execute on user->setup() */
$phpbb_hook->register('phpbb_user_session_handler', array($blog_hook, 'setup'));

/* Register the hook to execute on append_sid() function */
$phpbb_hook->register('append_sid', array($blog_hook, 'append_sid'));

/* Register the hook to execute on $template->display() */
$phpbb_hook->register(array('template', 'display'), array($blog_hook, 'display'));

?>